@@ -174,16 +174,16 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [videoPlayerPlugin initialize:&error];
   XCTAssertNil(error);
 
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
   XCTAssertNil(error);
-  XCTAssertNotNil(textureMessage);
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureMessage.textureId)];
+  XCTAssertNotNil(textureId);
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertNotNil(player);
 
   XCTAssertNotNil(player.playerLayer, @"AVPlayerLayer should be present.");
@@ -212,23 +212,22 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   FlutterError *initalizationError;
   [videoPlayerPlugin initialize:&initalizationError];
   XCTAssertNil(initalizationError);
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/hls/bee.m3u8"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&createError];
-  NSInteger textureId = textureMessage.textureId;
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&createError];
 
   // Ensure that the video playback is paused before seeking.
   FlutterError *pauseError;
-  [videoPlayerPlugin pause:textureMessage error:&pauseError];
+  [videoPlayerPlugin pausePlayer:textureId.integerValue error:&pauseError];
 
   XCTestExpectation *initializedExpectation = [self expectationWithDescription:@"seekTo completes"];
-  FVPPositionMessage *message = [FVPPositionMessage makeWithTextureId:textureId position:1234];
-  [videoPlayerPlugin seekTo:message
+  [videoPlayerPlugin seekTo:1234
+                  forPlayer:textureId.integerValue
                  completion:^(FlutterError *_Nullable error) {
                    [initializedExpectation fulfill];
                  }];
@@ -237,7 +236,7 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   // Seeking to a new position should start the display link temporarily.
   OCMVerify([mockDisplayLink setRunning:YES]);
 
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureId)];
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertEqual([player position], 1234);
 
   // Simulate a buffer being available.
@@ -279,15 +278,14 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   FlutterError *initalizationError;
   [videoPlayerPlugin initialize:&initalizationError];
   XCTAssertNil(initalizationError);
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/hls/bee.m3u8"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&createError];
-  NSInteger textureId = textureMessage.textureId;
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&createError];
 
   // Init should start the display link temporarily.
   OCMVerify([mockDisplayLink setRunning:YES]);
@@ -302,7 +300,7 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
       .ignoringNonObjectArgs()
       .andReturn(fakeBufferRef);
   // Simulate a callback from the engine to request a new frame.
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureId)];
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   [player copyPixelBuffer];
   // Since a frame was found, and the video is paused, the display link should be paused again.
   OCMVerify([mockDisplayLink setRunning:NO]);
@@ -330,30 +328,29 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   FlutterError *initalizationError;
   [videoPlayerPlugin initialize:&initalizationError];
   XCTAssertNil(initalizationError);
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/hls/bee.m3u8"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&createError];
-  NSInteger textureId = textureMessage.textureId;
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&createError];
 
   // Ensure that the video is playing before seeking.
   FlutterError *playError;
-  [videoPlayerPlugin play:textureMessage error:&playError];
+  [videoPlayerPlugin playPlayer:textureId.integerValue error:&playError];
 
   XCTestExpectation *initializedExpectation = [self expectationWithDescription:@"seekTo completes"];
-  FVPPositionMessage *message = [FVPPositionMessage makeWithTextureId:textureId position:1234];
-  [videoPlayerPlugin seekTo:message
+  [videoPlayerPlugin seekTo:1234
+                  forPlayer:textureId.integerValue
                  completion:^(FlutterError *_Nullable error) {
                    [initializedExpectation fulfill];
                  }];
   [self waitForExpectationsWithTimeout:30.0 handler:nil];
   OCMVerify([mockDisplayLink setRunning:YES]);
 
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureId)];
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertEqual([player position], 1234);
 
   // Simulate a buffer being available.
@@ -393,19 +390,19 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   FlutterError *initalizationError;
   [videoPlayerPlugin initialize:&initalizationError];
   XCTAssertNil(initalizationError);
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/hls/bee.m3u8"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&createError];
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&createError];
 
   // Run a play/pause cycle to force the pause codepath to run completely.
   FlutterError *playPauseError;
-  [videoPlayerPlugin play:textureMessage error:&playPauseError];
-  [videoPlayerPlugin pause:textureMessage error:&playPauseError];
+  [videoPlayerPlugin playPlayer:textureId.integerValue error:&playPauseError];
+  [videoPlayerPlugin pausePlayer:textureId.integerValue error:&playPauseError];
 
   // Since a buffer hasn't been available yet, the pause should not have stopped the display link.
   OCMVerify(never(), [mockDisplayLink setRunning:NO]);
@@ -421,24 +418,25 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [videoPlayerPlugin initialize:&error];
   XCTAssertNil(error);
 
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
   XCTAssertNil(error);
-  XCTAssertNotNil(textureMessage);
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureMessage.textureId)];
+  XCTAssertNotNil(textureId);
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertNotNil(player);
   AVPlayer *avPlayer = player.player;
 
-  [videoPlayerPlugin dispose:textureMessage error:&error];
+  [self keyValueObservingExpectationForObject:avPlayer keyPath:@"currentItem" expectedValue:nil];
+
+  [videoPlayerPlugin disposePlayer:textureId.integerValue error:&error];
   XCTAssertEqual(videoPlayerPlugin.playersByTextureId.count, 0);
   XCTAssertNil(error);
 
-  [self keyValueObservingExpectationForObject:avPlayer keyPath:@"currentItem" expectedValue:nil];
   [self waitForExpectationsWithTimeout:30.0 handler:nil];
 }
 
@@ -452,16 +450,16 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [videoPlayerPlugin initialize:&error];
   XCTAssertNil(error);
 
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
   XCTAssertNil(error);
-  XCTAssertNotNil(textureMessage);
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureMessage.textureId)];
+  XCTAssertNotNil(textureId);
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertNotNil(player);
   AVPlayer *avPlayer = player.player;
   [avPlayer play];
@@ -561,20 +559,19 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [pluginWithMockAVPlayer initialize:&initializationError];
   XCTAssertNil(initializationError);
 
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [pluginWithMockAVPlayer create:create error:&createError];
-  NSInteger textureId = textureMessage.textureId;
+  NSNumber *textureId = [pluginWithMockAVPlayer createWithOptions:create error:&createError];
 
   XCTestExpectation *initializedExpectation =
       [self expectationWithDescription:@"seekTo has zero tolerance when seeking not to end"];
-  FVPPositionMessage *message = [FVPPositionMessage makeWithTextureId:textureId position:1234];
-  [pluginWithMockAVPlayer seekTo:message
+  [pluginWithMockAVPlayer seekTo:1234
+                       forPlayer:textureId.integerValue
                       completion:^(FlutterError *_Nullable error) {
                         [initializedExpectation fulfill];
                       }];
@@ -600,21 +597,20 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [pluginWithMockAVPlayer initialize:&initializationError];
   XCTAssertNil(initializationError);
 
-  FVPCreateMessage *create = [FVPCreateMessage
+  FVPCreationOptions *create = [FVPCreationOptions
       makeWithAsset:nil
                 uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
         packageName:nil
          formatHint:nil
         httpHeaders:@{}];
   FlutterError *createError;
-  FVPTextureMessage *textureMessage = [pluginWithMockAVPlayer create:create error:&createError];
-  NSInteger textureId = textureMessage.textureId;
+  NSNumber *textureId = [pluginWithMockAVPlayer createWithOptions:create error:&createError];
 
   XCTestExpectation *initializedExpectation =
       [self expectationWithDescription:@"seekTo has non-zero tolerance when seeking to end"];
   // The duration of this video is "0" due to the non standard initiliatazion process.
-  FVPPositionMessage *message = [FVPPositionMessage makeWithTextureId:textureId position:0];
-  [pluginWithMockAVPlayer seekTo:message
+  [pluginWithMockAVPlayer seekTo:0
+                       forPlayer:textureId.integerValue
                       completion:^(FlutterError *_Nullable error) {
                         [initializedExpectation fulfill];
                       }];
@@ -629,15 +625,14 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   [videoPlayerPlugin initialize:&error];
   XCTAssertNil(error);
 
-  FVPCreateMessage *create = [FVPCreateMessage makeWithAsset:nil
-                                                         uri:uri
-                                                 packageName:nil
-                                                  formatHint:nil
-                                                 httpHeaders:@{}];
-  FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+  FVPCreationOptions *create = [FVPCreationOptions makeWithAsset:nil
+                                                             uri:uri
+                                                     packageName:nil
+                                                      formatHint:nil
+                                                     httpHeaders:@{}];
+  NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
 
-  NSInteger textureId = textureMessage.textureId;
-  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureId)];
+  FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
   XCTAssertNotNil(player);
 
   XCTestExpectation *initializedExpectation = [self expectationWithDescription:@"initialized"];
@@ -659,15 +654,13 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
   XCTAssertEqual(avPlayer.timeControlStatus, AVPlayerTimeControlStatusPaused);
 
   // Change playback speed.
-  FVPPlaybackSpeedMessage *playback = [FVPPlaybackSpeedMessage makeWithTextureId:textureId speed:2];
-  [videoPlayerPlugin setPlaybackSpeed:playback error:&error];
+  [videoPlayerPlugin setPlaybackSpeed:2 forPlayer:textureId.integerValue error:&error];
   XCTAssertNil(error);
   XCTAssertEqual(avPlayer.rate, 2);
   XCTAssertEqual(avPlayer.timeControlStatus, AVPlayerTimeControlStatusWaitingToPlayAtSpecifiedRate);
 
   // Volume
-  FVPVolumeMessage *volume = [FVPVolumeMessage makeWithTextureId:textureId volume:0.1];
-  [videoPlayerPlugin setVolume:volume error:&error];
+  [videoPlayerPlugin setVolume:0.1 forPlayer:textureId.integerValue error:&error];
   XCTAssertNil(error);
   XCTAssertEqual(avPlayer.volume, 0.1f);
 
@@ -696,22 +689,22 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
     [videoPlayerPlugin initialize:&error];
     XCTAssertNil(error);
 
-    FVPCreateMessage *create = [FVPCreateMessage
+    FVPCreationOptions *create = [FVPCreationOptions
         makeWithAsset:nil
                   uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
           packageName:nil
            formatHint:nil
           httpHeaders:@{}];
-    FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+    NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
     XCTAssertNil(error);
-    XCTAssertNotNil(textureMessage);
+    XCTAssertNotNil(textureId);
 
-    FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureMessage.textureId)];
+    FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
     XCTAssertNotNil(player);
     weakPlayer = player;
     avPlayer = player.player;
 
-    [videoPlayerPlugin dispose:textureMessage error:&error];
+    [videoPlayerPlugin disposePlayer:textureId.integerValue error:&error];
     XCTAssertNil(error);
   }
 
@@ -750,17 +743,17 @@ NSObject<FlutterPluginRegistry> *GetPluginRegistry(void) {
     [videoPlayerPlugin initialize:&error];
     XCTAssertNil(error);
 
-    FVPCreateMessage *create = [FVPCreateMessage
+    FVPCreationOptions *create = [FVPCreationOptions
         makeWithAsset:nil
                   uri:@"https://flutter.github.io/assets-for-api-docs/assets/videos/bee.mp4"
           packageName:nil
            formatHint:nil
           httpHeaders:@{}];
-    FVPTextureMessage *textureMessage = [videoPlayerPlugin create:create error:&error];
+    NSNumber *textureId = [videoPlayerPlugin createWithOptions:create error:&error];
     XCTAssertNil(error);
-    XCTAssertNotNil(textureMessage);
+    XCTAssertNotNil(textureId);
 
-    FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[@(textureMessage.textureId)];
+    FVPVideoPlayer *player = videoPlayerPlugin.playersByTextureId[textureId];
     XCTAssertNotNil(player);
     weakPlayer = player;
 
