@@ -36,11 +36,19 @@ class SurfaceFrame {
 /// The underlying representation is a [CkSurface], which can be reused by
 /// successive frames if they are the same size. Otherwise, a new [CkSurface] is
 /// created.
-class Surface {
-  Surface();
+class Surface extends DisplayCanvas {
+  Surface({this.isDisplayCanvas = false})
+      : useOffscreenCanvas =
+            Surface.offscreenCanvasSupported && !isDisplayCanvas;
 
   CkSurface? _surface;
 
+  /// Whether or not to use an `OffscreenCanvas` to back this [Surface].
+  final bool useOffscreenCanvas;
+
+  /// If `true`, this [Surface] is used as a [DisplayCanvas].
+  final bool isDisplayCanvas;
+
   /// If true, forces a new WebGL context to be created, even if the window
   /// size is the same. This is used to restore the UI after the browser tab
   /// goes dormant and loses the GL context.
@@ -79,8 +87,14 @@ class Surface {
   /// supported.
   DomCanvasElement? _canvasElement;
 
+  /// Note, if this getter is called, then this Surface is being used as an
+  /// overlay and must be backed by an onscreen <canvas> element.
+  @override
+  final DomElement hostElement = createDomElement('flt-canvas-container');
+
   int _pixelWidth = -1;
   int _pixelHeight = -1;
+  double _currentDevicePixelRatio = -1;
   int _sampleCount = -1;
   int _stencilBits = -1;
 
@@ -96,37 +110,46 @@ class Surface {
     }
   }
 
-  Future<void> rasterizeToCanvas(
-      ui.Size frameSize, RenderCanvas canvas, List<CkPicture> pictures) async {
-    final CkCanvas skCanvas = _surface!.getCanvas();
+  /// The CanvasKit canvas associated with this surface.
+  CkCanvas getCanvas() {
+    return _surface!.getCanvas();
+  }
+
+  void flush() {
+    _surface!.flush();
+  }
+
+  Future<void> rasterizeToCanvas(BitmapSize bitmapSize, RenderCanvas canvas,
+      List<CkPicture> pictures) async {
+    final CkCanvas skCanvas = getCanvas();
     skCanvas.clear(const ui.Color(0x00000000));
     pictures.forEach(skCanvas.drawPicture);
-    _surface!.flush();
+    flush();
 
     if (browserSupportsCreateImageBitmap) {
       JSObject bitmapSource;
-      if (Surface.offscreenCanvasSupported) {
+      if (useOffscreenCanvas) {
         bitmapSource = _offscreenCanvas! as JSObject;
       } else {
         bitmapSource = _canvasElement! as JSObject;
       }
       final DomImageBitmap bitmap = await createImageBitmap(bitmapSource, (
         x: 0,
-        y: _pixelHeight - frameSize.height.toInt(),
-        width: frameSize.width.toInt(),
-        height: frameSize.height.toInt(),
+        y: _pixelHeight - bitmapSize.height,
+        width: bitmapSize.width,
+        height: bitmapSize.height,
       ));
       canvas.render(bitmap);
     } else {
       // If the browser doesn't support `createImageBitmap` (e.g. Safari 14)
       // then render using `drawImage` instead.
       DomCanvasImageSource imageSource;
-      if (Surface.offscreenCanvasSupported) {
+      if (useOffscreenCanvas) {
         imageSource = _offscreenCanvas! as DomCanvasImageSource;
       } else {
         imageSource = _canvasElement! as DomCanvasImageSource;
       }
-      canvas.renderWithNoBitmapSupport(imageSource, _pixelHeight, frameSize);
+      canvas.renderWithNoBitmapSupport(imageSource, _pixelHeight, bitmapSize);
     }
   }
 
@@ -134,7 +157,7 @@ class Surface {
   ///
   /// The given [size] is in physical pixels.
   SurfaceFrame acquireFrame(ui.Size size) {
-    final CkSurface surface = createOrUpdateSurface(size);
+    final CkSurface surface = createOrUpdateSurface(BitmapSize.fromSize(size));
 
     // ignore: prefer_function_declarations_over_variables
     final SubmitCallback submitCallback =
@@ -145,8 +168,39 @@ class Surface {
     return SurfaceFrame(surface, submitCallback);
   }
 
-  ui.Size? _currentCanvasPhysicalSize;
-  ui.Size? _currentSurfaceSize;
+  BitmapSize? _currentCanvasPhysicalSize;
+  BitmapSize? _currentSurfaceSize;
+
+  /// Sets the CSS size of the canvas so that canvas pixels are 1:1 with device
+  /// pixels.
+  void _updateLogicalHtmlCanvasSize() {
+    final double devicePixelRatio =
+        EngineFlutterDisplay.instance.devicePixelRatio;
+    final double logicalWidth = _pixelWidth / devicePixelRatio;
+    final double logicalHeight = _pixelHeight / devicePixelRatio;
+    final DomCSSStyleDeclaration style = _canvasElement!.style;
+    style.width = '${logicalWidth}px';
+    style.height = '${logicalHeight}px';
+    _currentDevicePixelRatio = devicePixelRatio;
+  }
+
+  /// The <canvas> element backing this surface may be larger than the screen.
+  /// The Surface will draw the frame to the bottom left of the <canvas>, but
+  /// the <canvas> is, by default, positioned so that the top left corner is in
+  /// the top left of the window. We need to shift the canvas down so that the
+  /// bottom left of the <canvas> is the the bottom left corner of the window.
+  void positionToShowFrame(BitmapSize frameSize) {
+    assert(isDisplayCanvas,
+        'Should not position Surface if not used as a render canvas');
+    final double devicePixelRatio =
+        EngineFlutterDisplay.instance.devicePixelRatio;
+    final double logicalHeight = _pixelHeight / devicePixelRatio;
+    final double logicalFrameHeight = frameSize.height / devicePixelRatio;
+
+    // Shift the canvas up so the bottom left is in the window.
+    _canvasElement!.style.transform =
+        'translate(0px, ${logicalFrameHeight - logicalHeight}px)';
+  }
 
   /// This is only valid after the first frame or if [ensureSurface] has been
   /// called
@@ -162,7 +216,7 @@ class Surface {
   ///
   /// This also ensures that the gl/grcontext have been populated so
   /// that software rendering can be detected.
-  void ensureSurface([ui.Size size = const ui.Size(1, 1)]) {
+  void ensureSurface([BitmapSize size = const BitmapSize(1, 1)]) {
     // If the GrContext hasn't been setup yet then we need to force initialization
     // of the canvas and initial surface.
     if (_surface != null) {
@@ -175,7 +229,7 @@ class Surface {
   }
 
   /// Creates a <canvas> and SkSurface for the given [size].
-  CkSurface createOrUpdateSurface(ui.Size size) {
+  CkSurface createOrUpdateSurface(BitmapSize size) {
     if (size.isEmpty) {
       throw CanvasKitError('Cannot create surfaces of empty size.');
     }
@@ -183,32 +237,40 @@ class Surface {
     if (!_forceNewContext) {
       // Check if the window is the same size as before, and if so, don't allocate
       // a new canvas as the previous canvas is big enough to fit everything.
-      final ui.Size? previousSurfaceSize = _currentSurfaceSize;
+      final BitmapSize? previousSurfaceSize = _currentSurfaceSize;
       if (previousSurfaceSize != null &&
           size.width == previousSurfaceSize.width &&
           size.height == previousSurfaceSize.height) {
+        final double devicePixelRatio =
+            EngineFlutterDisplay.instance.devicePixelRatio;
+        if (isDisplayCanvas && devicePixelRatio != _currentDevicePixelRatio) {
+          _updateLogicalHtmlCanvasSize();
+        }
         return _surface!;
       }
 
-      final ui.Size? previousCanvasSize = _currentCanvasPhysicalSize;
+      final BitmapSize? previousCanvasSize = _currentCanvasPhysicalSize;
       // Initialize a new, larger, canvas. If the size is growing, then make the
       // new canvas larger than required to avoid many canvas creations.
       if (previousCanvasSize != null &&
           (size.width > previousCanvasSize.width ||
               size.height > previousCanvasSize.height)) {
-        final ui.Size newSize = size * 1.4;
+        final BitmapSize newSize = BitmapSize.fromSize(size.toSize() * 1.4);
         _surface?.dispose();
         _surface = null;
-        if (Surface.offscreenCanvasSupported) {
-          _offscreenCanvas!.width = newSize.width;
-          _offscreenCanvas!.height = newSize.height;
+        _pixelWidth = newSize.width;
+        _pixelHeight = newSize.height;
+        if (useOffscreenCanvas) {
+          _offscreenCanvas!.width = _pixelWidth.toDouble();
+          _offscreenCanvas!.height = _pixelHeight.toDouble();
         } else {
-          _canvasElement!.width = newSize.width;
-          _canvasElement!.height = newSize.height;
+          _canvasElement!.width = _pixelWidth.toDouble();
+          _canvasElement!.height = _pixelHeight.toDouble();
+        }
+        _currentCanvasPhysicalSize = BitmapSize(_pixelWidth, _pixelHeight);
+        if (isDisplayCanvas) {
+          _updateLogicalHtmlCanvasSize();
         }
-        _currentCanvasPhysicalSize = newSize;
-        _pixelWidth = newSize.width.ceil();
-        _pixelHeight = newSize.height.ceil();
       }
     }
 
@@ -253,7 +315,7 @@ class Surface {
   /// This function is expensive.
   ///
   /// It's better to reuse canvas if possible.
-  void _createNewCanvas(ui.Size physicalSize) {
+  void _createNewCanvas(BitmapSize physicalSize) {
     // Clear the container, if it's not empty. We're going to create a new <canvas>.
     if (_offscreenCanvas != null) {
       _offscreenCanvas!.removeEventListener(
@@ -280,6 +342,7 @@ class Surface {
         _cachedContextLostListener,
         false,
       );
+      _canvasElement!.remove();
       _canvasElement = null;
       _cachedContextRestoredListener = null;
       _cachedContextLostListener = null;
@@ -287,10 +350,10 @@ class Surface {
 
     // If `physicalSize` is not precise, use a slightly bigger canvas. This way
     // we ensure that the rendred picture covers the entire browser window.
-    _pixelWidth = physicalSize.width.ceil();
-    _pixelHeight = physicalSize.height.ceil();
+    _pixelWidth = physicalSize.width;
+    _pixelHeight = physicalSize.height;
     DomEventTarget htmlCanvas;
-    if (Surface.offscreenCanvasSupported) {
+    if (useOffscreenCanvas) {
       final DomOffscreenCanvas offscreenCanvas = createDomOffscreenCanvas(
         _pixelWidth,
         _pixelHeight,
@@ -304,6 +367,12 @@ class Surface {
       htmlCanvas = canvas;
       _canvasElement = canvas;
       _offscreenCanvas = null;
+      if (isDisplayCanvas) {
+        _canvasElement!.setAttribute('aria-hidden', 'true');
+        _canvasElement!.style.position = 'absolute';
+        hostElement.append(_canvasElement!);
+        _updateLogicalHtmlCanvasSize();
+      }
     }
 
     // When the browser tab using WebGL goes dormant the browser and/or OS may
@@ -336,7 +405,7 @@ class Surface {
         antialias: _kUsingMSAA ? 1 : 0,
         majorVersion: webGLVersion.toDouble(),
       );
-      if (Surface.offscreenCanvasSupported) {
+      if (useOffscreenCanvas) {
         glContext = canvasKit.GetOffscreenWebGLContext(
           _offscreenCanvas!,
           options,
@@ -368,7 +437,7 @@ class Surface {
 
   void _initWebglParams() {
     WebGLContext gl;
-    if (Surface.offscreenCanvasSupported) {
+    if (useOffscreenCanvas) {
       gl = _offscreenCanvas!.getGlContext(webGLVersion);
     } else {
       gl = _canvasElement!.getGlContext(webGLVersion);
@@ -377,7 +446,7 @@ class Surface {
     _stencilBits = gl.getParameter(gl.stencilBits);
   }
 
-  CkSurface _createNewSurface(ui.Size size) {
+  CkSurface _createNewSurface(BitmapSize size) {
     assert(_offscreenCanvas != null || _canvasElement != null);
     if (webGLVersion == -1) {
       return _makeSoftwareCanvasSurface('WebGL support not detected');
@@ -388,8 +457,8 @@ class Surface {
     } else {
       final SkSurface? skSurface = canvasKit.MakeOnScreenGLSurface(
           _grContext!,
-          size.width.roundToDouble(),
-          size.height.roundToDouble(),
+          size.width.toDouble(),
+          size.height.toDouble(),
           SkColorSpaceSRGB,
           _sampleCount,
           _stencilBits);
@@ -411,7 +480,7 @@ class Surface {
     }
 
     SkSurface surface;
-    if (Surface.offscreenCanvasSupported) {
+    if (useOffscreenCanvas) {
       surface = canvasKit.MakeOffscreenSWCanvasSurface(_offscreenCanvas!);
     } else {
       surface = canvasKit.MakeSWCanvasSurface(_canvasElement!);
@@ -427,6 +496,15 @@ class Surface {
     return true;
   }
 
+  @override
+  bool get isConnected => _canvasElement!.isConnected!;
+
+  @override
+  void initialize() {
+    ensureSurface();
+  }
+
+  @override
   void dispose() {
     _offscreenCanvas?.removeEventListener(
         'webglcontextlost', _cachedContextLostListener, false);
@@ -467,8 +545,8 @@ class CkSurface {
 
   int? get context => _glContext;
 
-  int width() => surface.width().round();
-  int height() => surface.height().round();
+  int width() => surface.width().ceil();
+  int height() => surface.height().ceil();
 
   void dispose() {
     if (_isDisposed) {
