@@ -10,7 +10,7 @@ part of dart._engine;
 const bool _debugDumpPaintCommands = false;
 
 // Returns the squared length of the x, y (of a border radius).
-double _measureBorderRadius(double x, double y) => x*x + y*y;
+double _measureBorderRadius(double x, double y) => x * x + y * y;
 
 /// Records canvas commands to be applied to a [EngineCanvas].
 ///
@@ -300,7 +300,8 @@ class RecordingCanvas {
     _commands.add(command);
   }
 
-  ui.Rect? getDestinationClipBounds() => _paintBounds.getDestinationClipBounds();
+  ui.Rect? getDestinationClipBounds() =>
+      _paintBounds.getDestinationClipBounds();
 
   void drawColor(ui.Color color, ui.BlendMode blendMode) {
     assert(!_recordingEnded);
@@ -525,7 +526,8 @@ class RecordingCanvas {
     _didDraw = true;
     final double left = offset.dx;
     final double top = offset.dy;
-    final PaintDrawImage command = PaintDrawImage(image, offset, paint.paintData);
+    final PaintDrawImage command =
+        PaintDrawImage(image, offset, paint.paintData);
     _paintBounds.growLTRB(
         left, top, left + image.width, top + image.height, command);
     _commands.add(command);
@@ -539,7 +541,7 @@ class RecordingCanvas {
       return;
     }
     final RecordingCanvas pictureRecording = enginePicture.recordingCanvas!;
-    if (pictureRecording._didDraw == true) {
+    if (pictureRecording._didDraw) {
       _didDraw = true;
     }
     renderStrategy.merge(pictureRecording.renderStrategy);
@@ -831,7 +833,8 @@ class PaintTransform extends PaintCommand {
   String toString() {
     String result = super.toString();
     assert(() {
-      result = 'transform(Matrix4.fromFloat32List(Float32List.fromList(<double>[${matrix4.join(', ')}])))';
+      result =
+          'transform(Matrix4.fromFloat32List(Float32List.fromList(<double>[${matrix4.join(', ')}])))';
       return true;
     }());
     return result;
@@ -1407,7 +1410,8 @@ class Ellipse extends PathCommand {
         anticlockwise ? startAngle - endAngle : endAngle - startAngle,
         matrix4,
         bezierPath);
-    targetPath.addPathWithMode(bezierPath, 0, 0, matrix4, SPathAddPathMode.kAppend);
+    targetPath.addPathWithMode(
+        bezierPath, 0, 0, matrix4, SPathAddPathMode.kAppend);
   }
 
   void _drawArcWithBezier(
@@ -1666,7 +1670,8 @@ class RRectCommand extends PathCommand {
   void transform(Float32List matrix4, SurfacePath targetPath) {
     final ui.Path roundRectPath = ui.Path();
     RRectToPathRenderer(roundRectPath).render(rrect);
-    targetPath.addPathWithMode(roundRectPath, 0, 0, matrix4, SPathAddPathMode.kAppend);
+    targetPath.addPathWithMode(
+        roundRectPath, 0, 0, matrix4, SPathAddPathMode.kAppend);
   }
 
   @override
