@@ -4,10 +4,12 @@ part of dart._engine;
 // found in the LICENSE file.
 
 
+
 const bool _debugLogPointerConverter = false;
 
-class _PointerState {
-  _PointerState(this.x, this.y);
+/// The state of the pointer of a specific device (e.g. finger, mouse).
+class _PointerDeviceState {
+  _PointerDeviceState(this.x, this.y);
 
   /// The identifier used in framework hit test.
   int? get pointer => _pointer;
@@ -22,6 +24,43 @@ class _PointerState {
   double y;
 }
 
+class _GlobalPointerState {
+  _GlobalPointerState() {
+    assert(() {
+      registerHotRestartListener(reset);
+      return true;
+    }());
+  }
+
+  // Map from browser pointer identifiers to PointerEvent pointer identifiers.
+  final Map<int, _PointerDeviceState> pointers = <int, _PointerDeviceState>{};
+
+  /// This field is used to keep track of button state.
+  ///
+  /// To normalize pointer events, when we receive pointer down followed by
+  /// pointer up, we synthesize a move event. To make sure that button state
+  /// is correct for move regardless of button state at the time of up event
+  /// we store it on down,hover and move events.
+  int activeButtons = 0;
+
+  _PointerDeviceState ensurePointerDeviceState(int device, double x, double y) {
+    return pointers.putIfAbsent(
+      device,
+      () => _PointerDeviceState(x, y),
+    );
+  }
+
+  /// Resets all pointer states.
+  ///
+  /// This method is invoked during hot reload to make sure we have a clean
+  /// converter after hot reload.
+  void reset() {
+    pointers.clear();
+    _PointerDeviceState._pointerCount = 0;
+    activeButtons = 0;
+  }
+}
+
 /// Converter to convert web pointer data into a form that framework can
 /// understand.
 ///
@@ -44,35 +83,14 @@ class _PointerState {
 class PointerDataConverter {
   PointerDataConverter();
 
-  // Map from browser pointer identifiers to PointerEvent pointer identifiers.
-  final Map<int, _PointerState> _pointers = <int, _PointerState>{};
-
-  /// This field is used to keep track of button state.
-  ///
-  /// To normalize pointer events, when we receive pointer down followed by
-  /// pointer up, we synthesize a move event. To make sure that button state
-  /// is correct for move regardless of button state at the time of up event
-  /// we store it on down,hover and move events.
-  int _activeButtons = 0;
-
-  /// Clears the existing pointer states.
-  ///
-  /// This method is invoked during hot reload to make sure we have a clean
-  /// converter after hot reload.
-  void clearPointerState() {
-    _pointers.clear();
-    _PointerState._pointerCount = 0;
-    _activeButtons = 0;
-  }
-
-  _PointerState _ensureStateForPointer(int device, double x, double y) {
-    return _pointers.putIfAbsent(
-      device,
-      () => _PointerState(x, y),
-    );
-  }
+  // This is made static because the state of pointer devices is global. This
+  // matches how the framework currently handles the state of pointer devices.
+  //
+  // See: https://github.com/flutter/flutter/blob/023e5addaa6e8e294a200cf754afaa1656f14aa6/packages/flutter/lib/src/rendering/binding.dart#L47-L47
+  static final _GlobalPointerState globalPointerState = _GlobalPointerState();
 
   ui.PointerData _generateCompletePointerData({
+    required int viewId,
     required Duration timeStamp,
     required ui.PointerChange change,
     required ui.PointerDeviceKind kind,
@@ -98,14 +116,16 @@ class PointerDataConverter {
     required double scrollDeltaX,
     required double scrollDeltaY,
     required double scale,
+    ui.PointerDataRespondCallback? onRespond,
   }) {
-    assert(_pointers.containsKey(device));
-    final _PointerState state = _pointers[device]!;
+    assert(globalPointerState.pointers.containsKey(device));
+    final _PointerDeviceState state = globalPointerState.pointers[device]!;
     final double deltaX = physicalX - state.x;
     final double deltaY = physicalY - state.y;
     state.x = physicalX;
     state.y = physicalY;
     return ui.PointerData(
+      viewId: viewId,
       timeStamp: timeStamp,
       change: change,
       kind: kind,
@@ -134,16 +154,18 @@ class PointerDataConverter {
       scrollDeltaX: scrollDeltaX,
       scrollDeltaY: scrollDeltaY,
       scale: scale,
+      onRespond: onRespond,
     );
   }
 
   bool _locationHasChanged(int device, double physicalX, double physicalY) {
-    assert(_pointers.containsKey(device));
-    final _PointerState state = _pointers[device]!;
+    assert(globalPointerState.pointers.containsKey(device));
+    final _PointerDeviceState state = globalPointerState.pointers[device]!;
     return state.x != physicalX || state.y != physicalY;
   }
 
   ui.PointerData _synthesizePointerData({
+    required int viewId,
     required Duration timeStamp,
     required ui.PointerChange change,
     required ui.PointerDeviceKind kind,
@@ -169,13 +191,14 @@ class PointerDataConverter {
     required double scrollDeltaY,
     required double scale,
   }) {
-    assert(_pointers.containsKey(device));
-    final _PointerState state = _pointers[device]!;
+    assert(globalPointerState.pointers.containsKey(device));
+    final _PointerDeviceState state = globalPointerState.pointers[device]!;
     final double deltaX = physicalX - state.x;
     final double deltaY = physicalY - state.y;
     state.x = physicalX;
     state.y = physicalY;
     return ui.PointerData(
+      viewId: viewId,
       timeStamp: timeStamp,
       change: change,
       kind: kind,
@@ -215,6 +238,7 @@ class PointerDataConverter {
   /// pointer data and stores it into [result]
   void convert(
     List<ui.PointerData> result, {
+    required int viewId,
     Duration timeStamp = Duration.zero,
     ui.PointerChange change = ui.PointerChange.cancel,
     ui.PointerDeviceKind kind = ui.PointerDeviceKind.touch,
@@ -240,20 +264,22 @@ class PointerDataConverter {
     double scrollDeltaX = 0.0,
     double scrollDeltaY = 0.0,
     double scale = 1.0,
+    ui.PointerDataRespondCallback? onRespond,
   }) {
     if (_debugLogPointerConverter) {
-      print('>> device=$device change=$change buttons=$buttons');
+      print('>> view=$viewId device=$device change=$change buttons=$buttons');
     }
     final bool isDown = buttons != 0;
     if (signalKind == null ||
       signalKind == ui.PointerSignalKind.none) {
       switch (change) {
         case ui.PointerChange.add:
-          assert(!_pointers.containsKey(device));
-          _ensureStateForPointer(device, physicalX, physicalY);
+          assert(!globalPointerState.pointers.containsKey(device));
+          globalPointerState.ensurePointerDeviceState(device, physicalX, physicalY);
           assert(!_locationHasChanged(device, physicalX, physicalY));
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -282,13 +308,14 @@ class PointerDataConverter {
             )
           );
         case ui.PointerChange.hover:
-          final bool alreadyAdded = _pointers.containsKey(device);
-          _ensureStateForPointer(device, physicalX, physicalY);
+          final bool alreadyAdded = globalPointerState.pointers.containsKey(device);
+          globalPointerState.ensurePointerDeviceState(device, physicalX, physicalY);
           assert(!isDown);
           if (!alreadyAdded) {
             // Synthesizes an add pointer data.
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.add,
                 kind: kind,
@@ -318,6 +345,7 @@ class PointerDataConverter {
           }
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -345,17 +373,18 @@ class PointerDataConverter {
               scale: scale,
             )
           );
-          _activeButtons = buttons;
+          globalPointerState.activeButtons = buttons;
         case ui.PointerChange.down:
-          final bool alreadyAdded = _pointers.containsKey(device);
-          final _PointerState state = _ensureStateForPointer(
-            device, physicalX, physicalY);
+          final bool alreadyAdded = globalPointerState.pointers.containsKey(device);
+          final _PointerDeviceState state = globalPointerState.ensurePointerDeviceState(
+              device, physicalX, physicalY);
           assert(isDown);
           state.startNewPointer();
           if (!alreadyAdded) {
             // Synthesizes an add pointer data.
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.add,
                 kind: kind,
@@ -389,6 +418,7 @@ class PointerDataConverter {
             // sending the down event, if necessary.
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.hover,
                 kind: kind,
@@ -418,6 +448,7 @@ class PointerDataConverter {
           }
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -445,12 +476,13 @@ class PointerDataConverter {
               scale: scale,
             )
           );
-          _activeButtons = buttons;
+          globalPointerState.activeButtons = buttons;
         case ui.PointerChange.move:
-          assert(_pointers.containsKey(device));
+          assert(globalPointerState.pointers.containsKey(device));
           assert(isDown);
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -478,11 +510,11 @@ class PointerDataConverter {
               scale: scale,
             )
           );
-          _activeButtons = buttons;
+          globalPointerState.activeButtons = buttons;
         case ui.PointerChange.up:
         case ui.PointerChange.cancel:
-          assert(_pointers.containsKey(device));
-          final _PointerState state = _pointers[device]!;
+          assert(globalPointerState.pointers.containsKey(device));
+          final _PointerDeviceState state = globalPointerState.pointers[device]!;
           assert(!isDown);
           // Cancel events can have different coordinates due to various
           // reasons (window lost focus which is accompanied by window
@@ -498,13 +530,14 @@ class PointerDataConverter {
             // sending the up event, if necessary.
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.move,
                 kind: kind,
                 device: device,
                 physicalX: physicalX,
                 physicalY: physicalY,
-                buttons: _activeButtons,
+                buttons: globalPointerState.activeButtons,
                 obscured: obscured,
                 pressure: pressure,
                 pressureMin: pressureMin,
@@ -527,6 +560,7 @@ class PointerDataConverter {
           }
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -560,6 +594,7 @@ class PointerDataConverter {
             // over (i.e. when "up" or "cancel" is received).
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.remove,
                 kind: kind,
@@ -586,14 +621,15 @@ class PointerDataConverter {
                 scale: scale,
               )
             );
-            _pointers.remove(device);
+            globalPointerState.pointers.remove(device);
           }
         case ui.PointerChange.remove:
-          assert(_pointers.containsKey(device));
-          final _PointerState state = _pointers[device]!;
+          assert(globalPointerState.pointers.containsKey(device));
+          final _PointerDeviceState state = globalPointerState.pointers[device]!;
           assert(!isDown);
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -621,7 +657,7 @@ class PointerDataConverter {
               scale: scale,
             )
           );
-          _pointers.remove(device);
+          globalPointerState.pointers.remove(device);
         case ui.PointerChange.panZoomStart:
         case ui.PointerChange.panZoomUpdate:
         case ui.PointerChange.panZoomEnd:
@@ -633,12 +669,13 @@ class PointerDataConverter {
         case ui.PointerSignalKind.scroll:
         case ui.PointerSignalKind.scrollInertiaCancel:
         case ui.PointerSignalKind.scale:
-          final bool alreadyAdded = _pointers.containsKey(device);
-          _ensureStateForPointer(device, physicalX, physicalY);
+          final bool alreadyAdded = globalPointerState.pointers.containsKey(device);
+          globalPointerState.ensurePointerDeviceState(device, physicalX, physicalY);
           if (!alreadyAdded) {
             // Synthesizes an add pointer data.
             result.add(
               _synthesizePointerData(
+                viewId: viewId,
                 timeStamp: timeStamp,
                 change: ui.PointerChange.add,
                 kind: kind,
@@ -674,6 +711,7 @@ class PointerDataConverter {
             if (isDown) {
               result.add(
                 _synthesizePointerData(
+                  viewId: viewId,
                   timeStamp: timeStamp,
                   change: ui.PointerChange.move,
                   kind: kind,
@@ -703,6 +741,7 @@ class PointerDataConverter {
             } else {
               result.add(
                 _synthesizePointerData(
+                  viewId: viewId,
                   timeStamp: timeStamp,
                   change: ui.PointerChange.hover,
                   kind: kind,
@@ -733,6 +772,7 @@ class PointerDataConverter {
           }
           result.add(
             _generateCompletePointerData(
+              viewId: viewId,
               timeStamp: timeStamp,
               change: change,
               kind: kind,
@@ -758,6 +798,7 @@ class PointerDataConverter {
               scrollDeltaX: scrollDeltaX,
               scrollDeltaY: scrollDeltaY,
               scale: scale,
+              onRespond: onRespond,
             )
           );
         case ui.PointerSignalKind.none:
