@@ -6,14 +6,19 @@ part of dart._engine;
 
 /// Provides accessibility for links.
 class Link extends PrimaryRoleManager {
-  Link(SemanticsObject semanticsObject) : super.withBasics(PrimaryRole.link, semanticsObject);
+  Link(SemanticsObject semanticsObject) : super.withBasics(
+    PrimaryRole.link,
+    semanticsObject,
+    preferredLabelRepresentation: LabelRepresentation.domText,
+  ) {
+    addTappable();
+  }
 
   @override
   DomElement createElement() {
     final DomElement element = domDocument.createElement('a');
-    // TODO(chunhtai): Fill in the real link once the framework sends entire uri.
-    // https://github.com/flutter/flutter/issues/102535.
-    element.setAttribute('href', '#');
+    // TODO(mdebbar): Fill in the real link once the framework sends entire uri.
+    // https://github.com/flutter/flutter/issues/150263.
     element.style.display = 'block';
     return element;
   }
