@@ -23,14 +23,11 @@ part of dart._engine;
 /// viewport "scrollTop" may take positive values.
 class Scrollable extends PrimaryRoleManager {
   Scrollable(SemanticsObject semanticsObject)
-      : super.withBasics(PrimaryRole.scrollable, semanticsObject) {
-    _scrollOverflowElement.style
-      ..position = 'absolute'
-      ..transformOrigin = '0 0 0'
-      // Ignore pointer events since this is a dummy element.
-      ..pointerEvents = 'none';
-    append(_scrollOverflowElement);
-  }
+      : super.withBasics(
+          PrimaryRole.scrollable,
+          semanticsObject,
+          preferredLabelRepresentation: LabelRepresentation.ariaLabel,
+        );
 
   /// Disables browser-driven scrolling in the presence of pointer events.
   GestureModeCallback? _gestureModeListener;
@@ -92,6 +89,20 @@ class Scrollable extends PrimaryRoleManager {
     }
   }
 
+  @override
+  void initState() {
+    // Scrolling is controlled by setting overflow-y/overflow-x to 'scroll`. The
+    // default overflow = "visible" needs to be unset.
+    semanticsObject.element.style.overflow = '';
+
+    _scrollOverflowElement.style
+      ..position = 'absolute'
+      ..transformOrigin = '0 0 0'
+      // Ignore pointer events since this is a dummy element.
+      ..pointerEvents = 'none';
+    append(_scrollOverflowElement);
+  }
+
   @override
   void update() {
     super.update();
