@@ -40,7 +40,7 @@ String placeholderMessage = 'Enable accessibility';
 /// See [DesktopSemanticsEnabler], [MobileSemanticsEnabler].
 class SemanticsHelper {
   SemanticsEnabler _semanticsEnabler =
-      isDesktop ? DesktopSemanticsEnabler() : MobileSemanticsEnabler();
+      ui_web.browser.isDesktop ? DesktopSemanticsEnabler() : MobileSemanticsEnabler();
 
   
   set semanticsEnabler(SemanticsEnabler semanticsEnabler) {
@@ -135,7 +135,7 @@ class DesktopSemanticsEnabler extends SemanticsEnabler {
       return true;
     }
 
-    if (EngineSemanticsOwner.instance.semanticsEnabled) {
+    if (EngineSemantics.instance.semanticsEnabled) {
       // Semantics already enabled, forward to framework as normal.
       return true;
     }
@@ -164,7 +164,7 @@ class DesktopSemanticsEnabler extends SemanticsEnabler {
       return true;
     }
 
-    EngineSemanticsOwner.instance.semanticsEnabled = true;
+    EngineSemantics.instance.semanticsEnabled = true;
     dispose();
     return false;
   }
@@ -254,7 +254,7 @@ class MobileSemanticsEnabler extends SemanticsEnabler {
 
     if (_schedulePlaceholderRemoval) {
       // The event type can also be click for VoiceOver.
-      final bool removeNow = browserEngine != BrowserEngine.webkit ||
+      final bool removeNow = ui_web.browser.browserEngine != ui_web.BrowserEngine.webkit ||
           event.type == 'touchend' ||
           event.type == 'pointerup' ||
           event.type == 'click';
@@ -264,7 +264,7 @@ class MobileSemanticsEnabler extends SemanticsEnabler {
       return true;
     }
 
-    if (EngineSemanticsOwner.instance.semanticsEnabled) {
+    if (EngineSemantics.instance.semanticsEnabled) {
       // Semantics already enabled, forward to framework as normal.
       return true;
     }
@@ -354,7 +354,7 @@ class MobileSemanticsEnabler extends SemanticsEnabler {
       _schedulePlaceholderRemoval = true;
       semanticsActivationTimer = Timer(_periodToConsumeEvents, () {
         dispose();
-        EngineSemanticsOwner.instance.semanticsEnabled = true;
+        EngineSemantics.instance.semanticsEnabled = true;
       });
       return false;
     }
