@@ -427,7 +427,7 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
     this.maxLines,
     this.fontFamily,
     this.fontSize,
-    this.height,
+    double? height,
     ui.TextHeightBehavior? textHeightBehavior,
     this.fontWeight,
     this.fontStyle,
@@ -436,7 +436,8 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
     this.locale,
   })  : _textHeightBehavior = textHeightBehavior,
         // TODO(mdebbar): add support for strut style., b/128317744
-        _strutStyle = strutStyle as EngineStrutStyle?;
+        _strutStyle = strutStyle as EngineStrutStyle?,
+        height = height == ui.kTextHeightNone ? null : height;
 
   final ui.TextAlign? textAlign;
   final ui.TextDirection? textDirection;
@@ -492,6 +493,7 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
         other.fontSize == fontSize &&
         other.height == height &&
         other._textHeightBehavior == _textHeightBehavior &&
+        other._strutStyle == _strutStyle &&
         other.ellipsis == ellipsis &&
         other.locale == locale;
   }
@@ -499,17 +501,19 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
   @override
   int get hashCode {
     return Object.hash(
-        textAlign,
-        textDirection,
-        fontWeight,
-        fontStyle,
-        maxLines,
-        fontFamily,
-        fontSize,
-        height,
-        _textHeightBehavior,
-        ellipsis,
-        locale);
+      textAlign,
+      textDirection,
+      fontWeight,
+      fontStyle,
+      maxLines,
+      fontFamily,
+      fontSize,
+      height,
+      _textHeightBehavior,
+      _strutStyle,
+      ellipsis,
+      locale,
+    );
   }
 
   @override
@@ -528,6 +532,7 @@ class EngineParagraphStyle implements ui.ParagraphStyle {
           'fontFamily: ${fontFamily ?? "unspecified"}, '
           'fontSize: ${fontSize != null ? fontSize.toStringAsFixed(1) : "unspecified"}, '
           'height: ${height != null ? "${height.toStringAsFixed(1)}x" : "unspecified"}, '
+          'strutStyle: ${_strutStyle ?? "unspecified"}, '
           'ellipsis: ${ellipsis != null ? '"$ellipsis"' : "unspecified"}, '
           'locale: ${locale ?? "unspecified"}'
           ')';
@@ -767,6 +772,7 @@ class EngineTextStyle implements ui.TextStyle {
           'letterSpacing: ${letterSpacing != null ? "${letterSpacing}x" : "unspecified"}, '
           'wordSpacing: ${wordSpacing != null ? "${wordSpacing}x" : "unspecified"}, '
           'height: ${height != null ? "${height.toStringAsFixed(1)}x" : "unspecified"}, '
+          'leadingDistribution: ${leadingDistribution ?? "unspecified"}, '
           'locale: ${locale ?? "unspecified"}, '
           'background: ${background ?? "unspecified"}, '
           'foreground: ${foreground ?? "unspecified"}, '
@@ -795,7 +801,7 @@ class EngineStrutStyle implements ui.StrutStyle {
   })  : _fontFamily = fontFamily,
         _fontFamilyFallback = fontFamilyFallback,
         _fontSize = fontSize,
-        _height = height,
+        _height = height == ui.kTextHeightNone ? null : height,
         _leadingDistribution = leadingDistribution,
         _leading = leading,
         _fontWeight = fontWeight,
@@ -833,9 +839,11 @@ class EngineStrutStyle implements ui.StrutStyle {
   }
 
   @override
-  int get hashCode => Object.hash(
+  int get hashCode {
+    final List<String>? fontFamilyFallback = _fontFamilyFallback;
+    return Object.hash(
         _fontFamily,
-        _fontFamilyFallback,
+        fontFamilyFallback != null ? Object.hashAll(fontFamilyFallback) : null,
         _fontSize,
         _height,
         _leading,
@@ -844,6 +852,7 @@ class EngineStrutStyle implements ui.StrutStyle {
         _fontStyle,
         _forceStrutHeight,
       );
+  }
 }
 
 /// Holds information for a placeholder in a paragraph.
@@ -989,7 +998,7 @@ void applyTextStyleToElement({
       final String? textDecoration =
           _textDecorationToCssString(style.decoration, style.decorationStyle);
       if (textDecoration != null) {
-        if (browserEngine == BrowserEngine.webkit) {
+        if (ui_web.browser.browserEngine == ui_web.BrowserEngine.webkit) {
           setElementStyle(element, '-webkit-text-decoration', textDecoration);
         } else {
           cssStyle.textDecoration = textDecoration;
