@@ -140,23 +140,18 @@ class SkwasmRenderer implements Renderer {
   }
 
   @override
-  Future<ui.Codec> instantiateImageCodecFromUrl(Uri uri, {WebOnlyImageCodecChunkCallback? chunkCallback}) {
+  Future<ui.Codec> instantiateImageCodecFromUrl(Uri uri, {ui_web.ImageCodecChunkCallback? chunkCallback}) {
     throw UnimplementedError('Skwasm not implemented on this platform.');
   }
 
   @override
-  void renderScene(ui.Scene scene) {
+  Future<void> renderScene(ui.Scene scene, ui.FlutterView view) {
     throw UnimplementedError('Skwasm not implemented on this platform.');
   }
 
   @override
   String get rendererTag => throw UnimplementedError('Skwasm not implemented on this platform.');
 
-  @override
-  void reset(FlutterViewEmbedder embedder) {
-    throw UnimplementedError('Skwasm not implemented on this platform.');
-  }
-
   @override
   void clearFragmentProgramCache() => _programs.clear();
 
@@ -184,4 +179,14 @@ class SkwasmRenderer implements Renderer {
     required double baseline,
     required int lineNumber
   }) => throw UnimplementedError('Skwasm not implemented on this platform.');
+
+  @override
+  ui.Image createImageFromImageBitmap(DomImageBitmap imageSource) {
+    throw UnimplementedError('Skwasm not implemented on this platform.');
+  }
+
+  @override
+  ui.Image createImageFromTextureSource(JSAny object,  { required int width, required int height, required bool transferOwnership }) {
+    throw Exception('Skwasm not implemented on this platform.');
+  }
 }
