@@ -8,11 +8,12 @@ part of dart.async;
 // Controller for creating and adding events to a stream.
 // -------------------------------------------------------------------
 
-/// Type of a stream controller's `onListen`, `onPause` and `onResume` callbacks.
-typedef void ControllerCallback();
+/// Type of a stream controller's `onListen`, `onPause` and `onResume`
+/// callbacks.
+typedef ControllerCallback = void Function();
 
 /// Type of stream controller `onCancel` callbacks.
-typedef FutureOr<void> ControllerCancelCallback();
+typedef ControllerCancelCallback = FutureOr<void> Function();
 
 /// A controller with the stream it controls.
 ///
@@ -66,7 +67,7 @@ typedef FutureOr<void> ControllerCancelCallback();
 /// await streamController.close();
 /// isClosed = streamController.isClosed; // true
 /// ```
-abstract class StreamController<T> implements StreamSink<T> {
+abstract interface class StreamController<T> implements StreamSink<T> {
   /// The stream that this controller is controlling.
   Stream<T> get stream;
 
@@ -133,8 +134,8 @@ abstract class StreamController<T> implements StreamSink<T> {
   /// listeners at the time when [add], [addError] or [close] is called.
   /// It is not allowed to call `add`, `addError`, or `close` before a previous
   /// call has returned. The controller does not have any internal queue of
-  /// events, and if there are no listeners at the time the event is added,
-  /// it will just be dropped, or, if it is an error, be reported as uncaught.
+  /// events, and if there are no listeners at the time the event or error is
+  /// added, it will just be dropped.
   ///
   /// Each listener subscription is handled independently,
   /// and if one pauses, only the pausing listener is affected.
@@ -255,9 +256,13 @@ abstract class StreamController<T> implements StreamSink<T> {
   /// This happens either when the done event has been sent,
   /// or when the subscriber on a single-subscription stream is canceled.
   ///
-  /// A broadcast stream controller will send the done event
-  /// even if listeners are paused, so some broadcast events may not have been
-  /// received yet when the returned future completes.
+  /// A stream controller will not complete the returned future until all
+  /// listeners present when the done event is sent have stopped listening.
+  /// A listener will stop listening if it is cancelled, or if it has handled
+  /// the done event.
+  /// A paused listener will not process the done even until it is resumed, so
+  /// completion of the returned Future will be delayed until all paused
+  /// listeners have been resumed or cancelled.
   ///
   /// If no one listens to a non-broadcast stream,
   /// or the listener pauses and never resumes,
@@ -270,9 +275,13 @@ abstract class StreamController<T> implements StreamSink<T> {
   /// This happens either when the done event has been sent, or if the
   /// subscriber on a single-subscription stream is canceled.
   ///
-  /// A broadcast stream controller will send the done event
-  /// even if listeners are paused, so some broadcast events may not have been
-  /// received yet when the returned future completes.
+  /// A stream controller will not complete the returned future until all
+  /// listeners present when the done event is sent have stopped listening.
+  /// A listener will stop listening if it is cancelled, or if it has handled
+  /// the done event.
+  /// A paused listener will not process the done even until it is resumed, so
+  /// completion of the returned Future will be delayed until all paused
+  /// listeners have been resumed or cancelled.
   ///
   /// If there is no listener on a non-broadcast stream,
   /// or the listener pauses and never resumes,
@@ -368,7 +377,8 @@ abstract class StreamController<T> implements StreamSink<T> {
 /// another event is in progress may cause the second event to be delayed
 /// and not be delivered synchronously, and until that event is delivered,
 /// the controller will not act synchronously.
-abstract class SynchronousStreamController<T> implements StreamController<T> {
+abstract interface class SynchronousStreamController<T>
+    implements StreamController<T> {
   /// Adds event to the controller's stream.
   ///
   /// As [StreamController.add], but must not be called while an event is
@@ -523,7 +533,7 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
       return _varData as dynamic;
     }
     _StreamControllerAddStreamState<T> state = _varData as dynamic;
-    return state.varData;
+    return state._varData;
   }
 
   // Returns the pending events, and creates the object if necessary.
@@ -537,9 +547,9 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
       return events as dynamic;
     }
     _StreamControllerAddStreamState<T> state = _varData as dynamic;
-    Object? events = state.varData;
+    Object? events = state._varData;
     if (events == null) {
-      state.varData = events = _PendingEvents<T>();
+      state._varData = events = _PendingEvents<T>();
     }
     return events as dynamic;
   }
@@ -552,7 +562,7 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
     Object? varData = _varData;
     if (_isAddingStream) {
       _StreamControllerAddStreamState<Object?> streamState = varData as dynamic;
-      varData = streamState.varData;
+      varData = streamState._varData;
     }
     return varData as dynamic;
   }
@@ -663,7 +673,7 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
     // End of addStream stream.
     assert(_isAddingStream);
     _StreamControllerAddStreamState<T> addState = _varData as dynamic;
-    _varData = addState.varData;
+    _varData = addState._varData;
     _state &= ~_STATE_ADDSTREAM;
     addState.complete();
   }
@@ -682,7 +692,7 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
     _state |= _STATE_SUBSCRIBED;
     if (_isAddingStream) {
       _StreamControllerAddStreamState<T> addState = _varData as dynamic;
-      addState.varData = subscription;
+      addState._varData = subscription;
       addState.resume();
     } else {
       _varData = subscription;
@@ -701,8 +711,8 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
     // If either of addStream's cancel or `onCancel` returns a future,
     // we wait for it before continuing.
     // Any error during this process ends up in the returned future.
-    // If more errors happen, we act as if it happens inside nested try/finallys
-    // or whenComplete calls, and only the last error ends up in the
+    // If more errors happen, we act as if it happens inside nested try/finally
+    // blocks or whenComplete calls, and only the last error ends up in the
     // returned future.
     Future<void>? result;
     if (_isAddingStream) {
@@ -768,7 +778,7 @@ abstract class _StreamController<T> implements _StreamControllerBase<T> {
   }
 }
 
-abstract class _SyncStreamControllerDispatch<T>
+mixin _SyncStreamControllerDispatch<T>
     implements _StreamController<T>, SynchronousStreamController<T> {
   void _sendData(T data) {
     _subscription._add(data);
@@ -783,8 +793,7 @@ abstract class _SyncStreamControllerDispatch<T>
   }
 }
 
-abstract class _AsyncStreamControllerDispatch<T>
-    implements _StreamController<T> {
+mixin _AsyncStreamControllerDispatch<T> implements _StreamController<T> {
   void _sendData(T data) {
     _subscription._addPending(_DelayedData<T>(data));
   }
@@ -805,6 +814,7 @@ abstract class _AsyncStreamControllerDispatch<T>
 class _AsyncStreamController<T> = _StreamController<T>
     with _AsyncStreamControllerDispatch<T>;
 
+@pragma("vm:entry-point")
 class _SyncStreamController<T> = _StreamController<T>
     with _SyncStreamControllerDispatch<T>;
 
@@ -881,6 +891,7 @@ class _StreamSinkWrapper<T> implements StreamSink<T> {
 /// Object containing the state used to handle [StreamController.addStream].
 class _AddStreamState<T> {
   // [_Future] returned by call to addStream.
+  @pragma('vm:entry-point')
   final _Future addStreamFuture;
 
   // Subscription on stream argument to addStream.
@@ -935,10 +946,11 @@ class _StreamControllerAddStreamState<T> extends _AddStreamState<T> {
   // The subscription or pending data of a _StreamController.
   // Stored here because we reuse the `_varData` field  in the _StreamController
   // to store this state object.
-  var varData;
+  @pragma('vm:entry-point')
+  var _varData;
 
-  _StreamControllerAddStreamState(_StreamController<T> controller, this.varData,
-      Stream<T> source, bool cancelOnError)
+  _StreamControllerAddStreamState(_StreamController<T> controller,
+      this._varData, Stream<T> source, bool cancelOnError)
       : super(controller, source, cancelOnError) {
     if (controller.isPaused) {
       addSubscription.pause();
