@@ -9,7 +9,6 @@
 ///
 /// *NOTE*: The `dart:isolate` library is currently only supported by the
 /// [Dart Native](https://dart.dev/overview#platform) platform.
-
 ///
 /// To use this library in your code:
 /// ```dart
@@ -69,12 +68,12 @@ class IsolateSpawnException implements Exception {
 /// The `Isolate` object provided by a spawn operation will have the
 /// control port and capabilities needed to control the isolate.
 /// New isolate objects can be created without some of these capabilities
-/// if necessary, using the [Isolate.Isolate] constructor.
+/// if necessary, using the [Isolate.new] constructor.
 ///
 /// An `Isolate` object cannot be sent over a `SendPort`, but the control port
 /// and capabilities can be sent, and can be used to create a new functioning
 /// `Isolate` object in the receiving port's isolate.
-class Isolate {
+final class Isolate {
   /// Argument to `ping` and `kill`: Ask for immediate action.
   static const int immediate = 0;
 
@@ -150,6 +149,161 @@ class Isolate {
   /// inspect the isolate and see uncaught errors or when it terminates.
   Isolate(this.controlPort, {this.pauseCapability, this.terminateCapability});
 
+  /// Runs [computation] in a new isolate and returns the result.
+  ///
+  /// ```dart
+  /// int slowFib(int n) =>
+  ///     n <= 1 ? 1 : slowFib(n - 1) + slowFib(n - 2);
+  ///
+  /// // Compute without blocking current isolate.
+  /// var fib40 = await Isolate.run(() => slowFib(40));
+  /// ```
+  ///
+  /// If [computation] is asynchronous (returns a `Future<R>`) then
+  /// that future is awaited in the new isolate, completing the entire
+  /// asynchronous computation, before returning the result.
+  ///
+  /// ```dart
+  /// int slowFib(int n) =>
+  ///     n <= 1 ? 1 : slowFib(n - 1) + slowFib(n - 2);
+  /// Stream<int> fibStream() async* {
+  ///   for (var i = 0;; i++) yield slowFib(i);
+  /// }
+  ///
+  /// // Returns `Future<int>`.
+  /// var fib40 = await Isolate.run(() => fibStream().elementAt(40));
+  /// ```
+  ///
+  /// If [computation] throws, the isolate is terminated and this
+  /// function throws the same error.
+  ///
+  /// ```dart import:convert
+  /// Future<int> eventualError() async {
+  ///   await Future.delayed(const Duration(seconds: 1));
+  ///   throw StateError("In a bad state!");
+  /// }
+  ///
+  /// try {
+  ///   await Isolate.run(eventualError);
+  /// } on StateError catch (e, s) {
+  ///   print(e.message); // In a bad state!
+  ///   print(LineSplitter.split("$s").first); // Contains "eventualError"
+  /// }
+  /// ```
+  /// Any uncaught asynchronous errors will terminate the computation as well,
+  /// but will be reported as a [RemoteError] because [addErrorListener]
+  /// does not provide the original error object.
+  ///
+  /// The result is sent using [exit], which means it's sent to this
+  /// isolate without copying.
+  ///
+  /// The [computation] function and its result (or error) must be
+  /// sendable between isolates. Objects that cannot be sent include open
+  /// files and sockets (see [SendPort.send] for details).
+  ///
+  /// If [computation] is a closure then it may implicitly send unexpected
+  /// state to the isolate due to limitations in the Dart implementation. This
+  /// can cause performance issues, increased memory usage
+  /// (see http://dartbug.com/36983) or, if the state includes objects that
+  /// can't be spent between isolates, a runtime failure.
+  ///
+  /// ```dart import:convert import:io
+  ///
+  /// void serializeAndWrite(File f, Object o) async {
+  ///   final openFile = await f.open(mode: FileMode.append);
+  ///   Future writeNew() async {
+  ///     // Will fail with:
+  ///     // "Invalid argument(s): Illegal argument in isolate message"
+  ///     // because `openFile` is captured.
+  ///     final encoded = await Isolate.run(() => jsonEncode(o));
+  ///     await openFile.writeString(encoded);
+  ///     await openFile.flush();
+  ///     await openFile.close();
+  ///   }
+  ///
+  ///   if (await openFile.position() == 0) {
+  ///     await writeNew();
+  ///   }
+  /// }
+  /// ```
+  ///
+  /// In such cases, you can create a new function to call [Isolate.run] that
+  /// takes all of the required state as arguments.
+  ///
+  /// ```dart import:convert import:io
+  ///
+  /// void serializeAndWrite(File f, Object o) async {
+  ///   final openFile = await f.open(mode: FileMode.append);
+  ///   Future writeNew() async {
+  ///     Future<String> encode(o) => Isolate.run(() => jsonEncode(o));
+  ///     final encoded = await encode(o);
+  ///     await openFile.writeString(encoded);
+  ///     await openFile.flush();
+  ///     await openFile.close();
+  ///   }
+  ///
+  ///   if (await openFile.position() == 0) {
+  ///     await writeNew();
+  ///   }
+  /// }
+  /// ```
+  ///
+  /// The [debugName] is only used to name the new isolate for debugging.
+  @Since("2.19")
+  static Future<R> run<R>(FutureOr<R> computation(), {String? debugName}) {
+    var result = Completer<R>();
+    var resultPort = RawReceivePort();
+    resultPort.handler = (response) {
+      resultPort.close();
+      if (response == null) {
+        // onExit handler message, isolate terminated without sending result.
+        result.completeError(
+            RemoteError("Computation ended without result", ""),
+            StackTrace.empty);
+        return;
+      }
+      var list = response as List<Object?>;
+      if (list.length == 2) {
+        var remoteError = list[0];
+        var remoteStack = list[1];
+        if (remoteStack is StackTrace) {
+          // Typed error.
+          result.completeError(remoteError!, remoteStack);
+        } else {
+          // onError handler message, uncaught async error.
+          // Both values are strings, so calling `toString` is efficient.
+          var error =
+              RemoteError(remoteError.toString(), remoteStack.toString());
+          result.completeError(error, error.stackTrace);
+        }
+      } else {
+        assert(list.length == 1);
+        result.complete(list[0] as R);
+      }
+    };
+    try {
+      Isolate.spawn(_RemoteRunner._remoteExecute,
+              _RemoteRunner<R>(computation, resultPort.sendPort),
+              onError: resultPort.sendPort,
+              onExit: resultPort.sendPort,
+              errorsAreFatal: true,
+              debugName: debugName)
+          .then<void>((_) {}, onError: (error, stack) {
+        // Sending the computation failed asynchronously.
+        // Do not expect a response, report the error asynchronously.
+        resultPort.close();
+        result.completeError(error, stack);
+      });
+    } on Object {
+      // Sending the computation failed synchronously.
+      // This is not expected to happen, but if it does,
+      // the synchronous error is respected and rethrown synchronously.
+      resultPort.close();
+      rethrow;
+    }
+    return result.future;
+  }
+
   /// An [Isolate] object representing the current isolate.
   ///
   /// The current isolate for code using [current]
@@ -164,20 +318,84 @@ class Isolate {
   /// is a sure way to hang your program.
   external static Isolate get current;
 
-  /// The location of the package configuration of the current isolate, if any.
+  /// The location of the package configuration file of the current isolate.
   ///
-  /// If the isolate has not been setup for package resolution,
-  /// this location is `null`,
-  /// otherwise it is a URI referencing the package config file.
+  /// If the isolate was spawned without specifying its package configuration
+  /// file then the returned value is `null`.
+  ///
+  /// Otherwise, the returned value is an absolute URI specifying the location
+  /// of isolate's package configuration file.
+  ///
+  /// The package configuration file is usually named `package_config.json`,
+  /// and you can use [`package:package_config`](https://pub.dev/documentation/package_config/latest/)
+  /// to read and parse it.
   external static Future<Uri?> get packageConfig;
 
-  /// Maps a `package:` URI to a non-package Uri.
+  /// The location of the package configuration file of the current isolate.
   ///
-  /// If there is no valid mapping from the `package:` URI in the current
-  /// isolate, then this call returns `null`. Non-`package:` URIs are
-  /// returned unmodified.
+  /// If the isolate was spawned without specifying its package configuration
+  /// file then the returned value is `null`.
+  ///
+  /// Otherwise, the returned value is an absolute URI specifying the location
+  /// of isolate's package configuration file.
+  ///
+  /// The package configuration file is usually named `package_config.json`,
+  /// and you can use [`package:package_config`](https://pub.dev/documentation/package_config/latest/)
+  /// to read and parse it.
+  @Since('3.2')
+  external static Uri? get packageConfigSync;
+
+  /// Resolves a `package:` URI to its actual location.
+  ///
+  /// Returns the actual location of the file or directory specified by the
+  /// [packageUri] `package:` URI.
+  ///
+  /// If the [packageUri] is not a `package:` URI, it's returned as-is.
+  ///
+  /// Returns `null` if [packageUri] is a `package:` URI, but either
+  /// the current package configuration does not have a configuration
+  /// for the package name of the URI, or
+  /// the URI is not valid (doesn't start with `package:valid_package_name/`),
+  ///
+  /// A `package:` URI is resolved to its actual location based on
+  /// a package resolution configuration (see [packageConfig])
+  /// which specifies how to find the actual location of the file or directory
+  /// that the `package:` URI points to.
+  ///
+  /// The actual location corresponding to a `package:` URI is always a
+  /// non-`package:` URI, typically a `file:` or possibly `http:` URI.
+  ///
+  /// A program may be run in a way where source files are not available,
+  /// and if so, the returned URI may not correspond to the actual file or
+  /// directory or be `null`.
   external static Future<Uri?> resolvePackageUri(Uri packageUri);
 
+  /// Resolves a `package:` URI to its actual location.
+  ///
+  /// Returns the actual location of the file or directory specified by the
+  /// [packageUri] `package:` URI.
+  ///
+  /// If the [packageUri] is not a `package:` URI, it's returned as-is.
+  ///
+  /// Returns `null` if [packageUri] is a `package:` URI, but either
+  /// the current package configuration does not have a configuration
+  /// for the package name of the URI, or
+  /// the URI is not valid (doesn't start with `package:valid_package_name/`),
+  ///
+  /// A `package:` URI is resolved to its actual location based on
+  /// a package resolution configuration (see [packageConfig])
+  /// which specifies how to find the actual location of the file or directory
+  /// that the `package:` URI points to.
+  ///
+  /// The actual location corresponding to a `package:` URI is always a
+  /// non-`package:` URI, typically a `file:` or possibly `http:` URI.
+  ///
+  /// A program may be run in a way where source files are not available,
+  /// and if so, the returned URI may not correspond to the actual file or
+  /// directory or be `null`.
+  @Since('3.2')
+  external static Uri? resolvePackageUriSync(Uri packageUri);
+
   /// Creates and spawns an isolate that shares the same code as the current
   /// isolate.
   ///
@@ -186,18 +404,22 @@ class Isolate {
   /// The entry-point function is invoked in the new isolate with [message]
   /// as the only argument.
   ///
-  /// The function must be a top-level function or a static method
-  /// that can be called with a single argument,
-  /// that is, a compile-time constant function value
-  /// which accepts at least one positional parameter
-  /// and has at most one required positional parameter.
+  /// The [entryPoint] function must be able to be called with a single
+  /// argument, that is, a function which accepts at least one positional
+  /// parameter and has at most one required positional parameter.
   /// The function may accept any number of optional parameters,
-  /// as long as it *can* be called with just a single argument.
-  /// The function must not be the value of a function expression
-  /// or an instance method tear-off.
+  /// as long as it *can* be called with just a single argument. If
+  /// [entryPoint] is a closure then it may implicitly send unexpected state
+  /// to the isolate due to limitations in the Dart implementation. This can
+  /// cause performance issues, increased memory usage
+  /// (see http://dartbug.com/36983) or, if the state includes objects that
+  /// can't be spent between isolates, a runtime failure. See [run] for an
+  /// example.
   ///
-  /// Usually the initial [message] contains a [SendPort] so
-  /// that the spawner and spawnee can communicate with each other.
+  /// [message] must be sendable between isolates. Objects that cannot be sent
+  /// include open files and sockets (see [SendPort.send] for details). Usually
+  /// the initial [message] contains a [SendPort] so that the spawner and
+  /// spawnee can communicate with each other.
   ///
   /// If the [paused] parameter is set to `true`,
   /// the isolate will start up in a paused state,
@@ -306,9 +528,7 @@ class Isolate {
   /// Returns a future that will complete with an [Isolate] instance if the
   /// spawning succeeded. It will complete with an error otherwise.
   external static Future<Isolate> spawnUri(
-      Uri uri,
-      List<String> args,
-      var message,
+      Uri uri, List<String> args, var message,
       {bool paused = false,
       SendPort? onExit,
       SendPort? onError,
@@ -316,11 +536,10 @@ class Isolate {
       bool? checked,
       Map<String, String>? environment,
       @Deprecated('The packages/ dir is not supported in Dart 2')
-          Uri? packageRoot,
+      Uri? packageRoot,
       Uri? packageConfig,
       bool automaticPackageResolution = false,
-      @Since("2.3")
-          String? debugName});
+      @Since("2.3") String? debugName});
 
   /// Requests the isolate to pause.
   ///
@@ -389,9 +608,9 @@ class Isolate {
   /// no exit message will be sent.
   ///
   /// The [response] object must follow the same restrictions as enforced by
-  /// [SendPort.send].
-  /// It is recommended to only use simple values that can be sent to all
-  /// isolates, like `null`, booleans, numbers or strings.
+  /// [SendPort.send] when sending to an isolate from another isolate group;
+  /// only simple values that can be sent to all isolates, like `null`,
+  /// booleans, numbers or strings, are allowed.
   ///
   /// Since isolates run concurrently, it's possible for it to exit before the
   /// exit listener is established, and in that case no response will be
@@ -468,9 +687,9 @@ class Isolate {
   /// Requests that the isolate send [response] on the [responsePort].
   ///
   /// The [response] object must follow the same restrictions as enforced by
-  /// [SendPort.send].
-  /// It is recommended to only use simple values that can be sent to all
-  /// isolates, like `null`, booleans, numbers or strings.
+  /// [SendPort.send] when sending to an isolate from another isolate group;
+  /// only simple values that can be sent to all isolates, like `null`,
+  /// booleans, numbers or strings, are allowed.
   ///
   /// If the isolate is alive, it will eventually send `response`
   /// (defaulting to `null`) on the response port.
@@ -599,28 +818,38 @@ class Isolate {
 ///
 /// [SendPort]s can be transmitted to other isolates, and they preserve equality
 /// when sent.
-abstract class SendPort implements Capability {
+@pragma("vm:entry-point")
+abstract interface class SendPort implements Capability {
   /// Sends an asynchronous [message] through this send port, to its
   /// corresponding [ReceivePort].
   ///
-  /// The transitive object graph of [message] can contain the following
-  /// objects:
-  ///   - [Null]
-  ///   - [bool]
-  ///   - [int]
-  ///   - [double]
-  ///   - [String]
-  ///   - [List] or [Map] (whose elements are any of these)
-  ///   - [TransferableTypedData]
-  ///   - [SendPort]
-  ///   - [Capability]
+  /// If the sending and receiving isolates do not share the same code
+  /// (an isolate created using [Isolate.spawnUri] does not share the code
+  /// of the isolate that spawned it), the transitive object graph of [message]
+  /// can **only** contain the following kinds of objects:
+  ///
+  ///   - `null`
+  ///   - `true` and `false`
+  ///   - Instances of [int], [double], [String]
+  ///   - Instances created through list, map and set literals
+  ///   - Instances created by constructors of:
+  ///     - [List], [Map], [LinkedHashMap], [Set] and [LinkedHashSet]
+  ///     - [TransferableTypedData]
+  ///     - [Capability]
+  ///   - [SendPort] instances from [ReceivePort.sendPort] or
+  ///     [RawReceivePort.sendPort] where the receive ports are created
+  ///     using those classes' constructors.
+  ///   - Instances of [Type] representing one of the types mentioned above,
+  ///     `Object`, `dynamic`, `void` and `Never` as well as nullable variants
+  ///     of all these types. For generic types type arguments must be sendable
+  ///     types for the whole type to be sendable.
   ///
   /// If the sender and receiver isolate share the same code (e.g. isolates
   /// created via [Isolate.spawn]), the transitive object graph of [message] can
   /// contain any object, with the following exceptions:
   ///
   ///   - Objects with native resources (subclasses of e.g.
-  ///     `NativeFieldWrapperClass1`). A [Socket] object for example referrs
+  ///     `NativeFieldWrapperClass1`). A [Socket] object for example refers
   ///     internally to objects that have native resources attached and can
   ///     therefore not be sent.
   ///   - [ReceivePort]
@@ -632,6 +861,10 @@ abstract class SendPort implements Capability {
   ///   - [UserTag]
   ///   - `MirrorReference`
   ///
+  /// Instances of classes that either themselves are marked with
+  /// `@pragma('vm:isolate-unsendable')`, extend or implement such classes
+  /// cannot be sent through the ports.
+  ///
   /// Apart from those exceptions any object can be sent. Objects that are
   /// identified as immutable (e.g. strings) will be shared whereas all other
   /// objects will be copied.
@@ -669,7 +902,7 @@ abstract class SendPort implements Capability {
 /// to a broadcast stream.
 ///
 /// A [ReceivePort] may have many [SendPort]s.
-abstract class ReceivePort implements Stream<dynamic> {
+abstract interface class ReceivePort implements Stream<dynamic> {
   /// Opens a long-lived port for receiving messages.
   ///
   /// A [ReceivePort] is a non-broadcast stream. This means that it buffers
@@ -726,7 +959,7 @@ abstract class ReceivePort implements Stream<dynamic> {
 /// message is received, otherwise the message is lost.
 ///
 /// Messages can be sent to this port using [sendPort].
-abstract class RawReceivePort {
+abstract interface class RawReceivePort {
   /// Opens a long-lived port for receiving messages.
   ///
   /// A [RawReceivePort] is low level and does not work with [Zone]s. It
@@ -766,13 +999,21 @@ abstract class RawReceivePort {
 
   /// Returns a [SendPort] that sends messages to this raw receive port.
   SendPort get sendPort;
+
+  /// Whether this [RawReceivePort] keeps its [Isolate] alive.
+  ///
+  /// By default, receive ports keep the [Isolate] that created them alive until
+  /// [close] is called. If [keepIsolateAlive] is set to `false`, the isolate
+  /// may close while the port is still open. The port is closed when the
+  /// isolate closes, and further messages sent by the [sendPort] are ignored.
+  abstract bool keepIsolateAlive;
 }
 
 /// Description of an error from another isolate.
 ///
 /// This error has the same `toString()` and `stackTrace.toString()` behavior
 /// as the original error, but has no other features of the original error.
-class RemoteError implements Error {
+final class RemoteError implements Error {
   final String _description;
   final StackTrace stackTrace;
   RemoteError(String description, String stackDescription)
@@ -792,7 +1033,7 @@ class RemoteError implements Error {
 /// When sent this way, the local transferable can no longer be materialized,
 /// and the received object is now the only way to materialize the data.
 @Since("2.3.2")
-abstract class TransferableTypedData {
+abstract final class TransferableTypedData {
   /// Creates a new [TransferableTypedData] containing the bytes of [list].
   ///
   /// It must be possible to create a single [Uint8List] containing the
@@ -807,3 +1048,62 @@ abstract class TransferableTypedData {
   /// transferable bytes, even if the calls occur in different isolates.
   ByteBuffer materialize();
 }
+
+/// Parameter object used by [Isolate.run].
+///
+/// The [_remoteExecute] function is run in a new isolate with a
+/// [_RemoteRunner] object as argument.
+final class _RemoteRunner<R> {
+  /// User computation to run.
+  final FutureOr<R> Function() computation;
+
+  /// Port to send isolate computation result on.
+  ///
+  /// Only one object is ever sent on this port.
+  /// If the value is `null`, it is sent by the isolate's "on-exit" handler
+  /// when the isolate terminates without otherwise sending value.
+  /// If the value is a list with one element,
+  /// then it is the result value of the computation.
+  /// Otherwise it is a list with two elements representing an error.
+  /// If the error is sent by the isolate's "on-error" uncaught error handler,
+  /// then the list contains two strings. This also terminates the isolate.
+  /// If sent manually by this class, after capturing the error,
+  /// the list contains one non-`null` [Object] and one [StackTrace].
+  final SendPort resultPort;
+
+  _RemoteRunner(this.computation, this.resultPort);
+
+  /// Run in a new isolate to get the result of [computation].
+  ///
+  /// The result is sent back on [resultPort] as a single-element list.
+  /// A two-element list sent on the same port is an error result.
+  /// When sent by this function, it's always an object and a [StackTrace].
+  /// (The same port listens on uncaught errors from the isolate, which
+  /// sends two-element lists containing [String]s instead).
+  static void _remoteExecute(_RemoteRunner<Object?> runner) {
+    runner._run();
+  }
+
+  void _run() async {
+    R result;
+    try {
+      var potentiallyAsyncResult = computation();
+      if (potentiallyAsyncResult is Future<R>) {
+        result = await potentiallyAsyncResult;
+      } else {
+        result = potentiallyAsyncResult;
+      }
+    } catch (e, s) {
+      // If sending fails, the error becomes an uncaught error.
+      Isolate.exit(resultPort, _list2(e, s));
+    }
+    Isolate.exit(resultPort, _list1(result));
+  }
+
+  /// Helper function to create a one-element non-growable list.
+  static List<Object?> _list1(Object? value) => List.filled(1, value);
+
+  /// Helper function to create a two-element non-growable list.
+  static List<Object?> _list2(Object? value1, Object? value2) =>
+      List.filled(2, value1)..[1] = value2;
+}
