@@ -10,7 +10,6 @@ import 'test_utils.dart';
 void main() {
   testWidgets('text button can run', (WidgetTester tester) async {
     await pumpsUseCase(tester, TextButtonUseCase());
-    expect(find.text('Text button'), findsOneWidget);
-    expect(find.text('Text button disabled'), findsOneWidget);
+    expect(find.text('Action'), findsExactly(2));
   });
 }
