@@ -11,12 +11,15 @@ const String kPictureCacheRouteName = '/picture_cache';
 const String kPictureCacheComplexityScoringRouteName = '/picture_cache_complexity_scoring';
 const String kLargeImageChangerRouteName = '/large_image_changer';
 const String kLargeImagesRouteName = '/large_images';
+const String kPathTessellationRouteName = '/path_tessellation';
 const String kTextRouteName = '/text';
+const String kVeryLongPictureScrollingRouteName = '/very_long_picture_scrolling';
 const String kFullscreenTextRouteName = '/fullscreen_text';
 const String kAnimatedPlaceholderRouteName = '/animated_placeholder';
 const String kClipperCacheRouteName = '/clipper_cache';
 const String kColorFilterAndFadeRouteName = '/color_filter_and_fade';
 const String kColorFilterCacheRouteName = '/color_filter_cache';
+const String kColorFilterWithUnstableChildName = '/color_filter_with_unstable_child';
 const String kFadingChildAnimationRouteName = '/fading_child_animation';
 const String kImageFilteredTransformAnimationRouteName = '/imagefiltered_transform_animation';
 const String kMultiWidgetConstructionRouteName = '/multi_widget_construction';
@@ -30,7 +33,15 @@ const String kAnimatedImageRouteName = '/animated_image';
 const String kOpacityPeepholeRouteName = '/opacity_peephole';
 const String kGradientPerfRouteName = '/gradient_perf';
 const String kAnimatedComplexOpacityPerfRouteName = '/animated_complex_opacity';
+const String kAnimatedComplexImageFilteredPerfRouteName = '/animated_complex_image_filtered';
 const String kListTextLayoutRouteName = '/list_text_layout';
+const String kAnimatedBlurBackdropFilter = '/animated_blur_backdrop_filter';
+const String kSlidersRouteName = '/sliders';
+const String kDrawPointsPageRougeName = '/draw_points';
+const String kDrawVerticesPageRouteName = '/draw_vertices';
+const String kDrawAtlasPageRouteName = '/draw_atlas';
+const String kAnimatedAdvancedBlend = '/animated_advanced_blend';
+const String kRRectBlurRouteName = '/rrect_blur';
 
 const String kOpacityPeepholeOneRectRouteName = '$kOpacityPeepholeRouteName/one_big_rect';
 const String kOpacityPeepholeColumnOfOpacityRouteName = '$kOpacityPeepholeRouteName/column_of_opacity';
