@@ -31,13 +31,11 @@ class BenchBuildMaterialCheckbox extends WidgetBuildRecorder {
   }
 
   Row _buildRow() {
-    if (_isChecked == null) {
-      _isChecked = true;
-    } else if (_isChecked!) {
-      _isChecked = false;
-    } else {
-      _isChecked = null;
-    }
+    _isChecked = switch (_isChecked) {
+      null  => true,
+      true  => false,
+      false => null,
+    };
 
     return Row(
       children: List<Widget>.generate(10, (int i) {
