@@ -218,26 +218,14 @@ class StartContext extends Context {
         engine = EngineRepository(
           checkouts,
           initialRef: 'upstream/$candidateBranch',
-          upstreamRemote: Remote(
-            name: RemoteName.upstream,
-            url: engineUpstream,
-          ),
-          mirrorRemote: Remote(
-            name: RemoteName.mirror,
-            url: engineMirror,
-          ),
+          upstreamRemote: Remote.upstream(engineUpstream),
+          mirrorRemote: Remote.mirror(engineMirror),
         ),
         framework = FrameworkRepository(
           checkouts,
           initialRef: 'upstream/$candidateBranch',
-          upstreamRemote: Remote(
-            name: RemoteName.upstream,
-            url: frameworkUpstream,
-          ),
-          mirrorRemote: Remote(
-            name: RemoteName.mirror,
-            url: frameworkMirror,
-          ),
+          upstreamRemote: Remote.upstream(frameworkUpstream),
+          mirrorRemote: Remote.mirror(frameworkMirror),
         );
 
   final String candidateBranch;
@@ -267,16 +255,14 @@ class StartContext extends Context {
     if (atBranchPoint) {
       return ReleaseType.BETA_INITIAL;
     }
-
-    if (releaseChannel == 'stable') {
-      if (lastVersion.type == VersionType.stable) {
-        return ReleaseType.STABLE_HOTFIX;
-      } else {
-        return ReleaseType.STABLE_INITIAL;
-      }
+    if (releaseChannel != 'stable') {
+      return ReleaseType.BETA_HOTFIX;
     }
 
-    return ReleaseType.BETA_HOTFIX;
+    return switch (lastVersion.type) {
+      VersionType.stable => ReleaseType.STABLE_HOTFIX,
+      VersionType.development || VersionType.gitDescribe || VersionType.latest => ReleaseType.STABLE_INITIAL,
+    };
   }
 
   Future<void> run() async {
@@ -310,16 +296,24 @@ class StartContext extends Context {
     }
 
     final String engineHead = await engine.reverseParse('HEAD');
-    state.engine = pb.Repository(
-      candidateBranch: candidateBranch,
-      workingBranch: workingBranchName,
-      startingGitHead: engineHead,
-      currentGitHead: engineHead,
-      checkoutPath: (await engine.checkoutDirectory).path,
-      dartRevision: dartRevision,
-      upstream: pb.Remote(name: 'upstream', url: engine.upstreamRemote.url),
-      mirror: pb.Remote(name: 'mirror', url: engine.mirrorRemote!.url),
+    state.engine = (pb.Repository.create()
+      ..candidateBranch = candidateBranch
+      ..workingBranch = workingBranchName
+      ..startingGitHead = engineHead
+      ..currentGitHead = engineHead
+      ..checkoutPath = (await engine.checkoutDirectory).path
+      ..upstream = (pb.Remote.create()
+        ..name = 'upstream'
+        ..url = engine.upstreamRemote.url
+      )
+      ..mirror = (pb.Remote.create()
+        ..name = 'mirror'
+        ..url = engine.mirrorRemote!.url
+      )
     );
+    if (dartRevision != null && dartRevision!.isNotEmpty) {
+      state.engine.dartRevision = dartRevision!;
+    }
 
     await framework.newBranch(workingBranchName);
 
@@ -362,14 +356,20 @@ class StartContext extends Context {
 
     state.releaseVersion = nextVersion.toString();
 
-    state.framework = pb.Repository(
-      candidateBranch: candidateBranch,
-      workingBranch: workingBranchName,
-      startingGitHead: frameworkHead,
-      currentGitHead: frameworkHead,
-      checkoutPath: (await framework.checkoutDirectory).path,
-      upstream: pb.Remote(name: 'upstream', url: framework.upstreamRemote.url),
-      mirror: pb.Remote(name: 'mirror', url: framework.mirrorRemote!.url),
+    state.framework = (pb.Repository.create()
+      ..candidateBranch = candidateBranch
+      ..workingBranch = workingBranchName
+      ..startingGitHead = frameworkHead
+      ..currentGitHead = frameworkHead
+      ..checkoutPath = (await framework.checkoutDirectory).path
+      ..upstream = (pb.Remote.create()
+        ..name = 'upstream'
+        ..url = framework.upstreamRemote.url
+      )
+      ..mirror = (pb.Remote.create()
+        ..name = 'mirror'
+        ..url = framework.mirrorRemote!.url
+      )
     );
 
     state.currentPhase = ReleasePhase.APPLY_ENGINE_CHERRYPICKS;
@@ -385,23 +385,12 @@ class StartContext extends Context {
 
   /// Determine this release's version number from the [lastVersion] and the [incrementLetter].
   Version calculateNextVersion(Version lastVersion, ReleaseType releaseType) {
-    late final Version nextVersion;
-    switch (releaseType) {
-      case ReleaseType.STABLE_INITIAL:
-        nextVersion = Version(
-          x: lastVersion.x,
-          y: lastVersion.y,
-          z: 0,
-          type: VersionType.stable,
-        );
-      case ReleaseType.STABLE_HOTFIX:
-        nextVersion = Version.increment(lastVersion, 'z');
-      case ReleaseType.BETA_INITIAL:
-        nextVersion = Version.fromCandidateBranch(candidateBranch);
-      case ReleaseType.BETA_HOTFIX:
-        nextVersion = Version.increment(lastVersion, 'n');
-    }
-    return nextVersion;
+    return switch (releaseType) {
+      ReleaseType.STABLE_INITIAL   => Version(x: lastVersion.x, y: lastVersion.y, z: 0, type: VersionType.stable),
+      ReleaseType.STABLE_HOTFIX    => Version.increment(lastVersion, 'z'),
+      ReleaseType.BETA_INITIAL     => Version.fromCandidateBranch(candidateBranch),
+      ReleaseType.BETA_HOTFIX || _ => Version.increment(lastVersion, 'n'),
+    };
   }
 
   /// Ensures the branch point [candidateBranch] and `master` has a version tag.
