@@ -31,161 +31,6 @@ void main() {
       stdio = TestStdio();
     });
 
-    test('canCherryPick returns true if git cherry-pick returns 0', () async {
-      processManager.addCommands(<FakeCommand>[
-        FakeCommand(command: <String>[
-          'git',
-          'clone',
-          '--origin',
-          'upstream',
-          '--',
-          FrameworkRepository.defaultUpstream,
-          fileSystem.path
-              .join(rootDir, 'flutter_conductor_checkouts', 'framework'),
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'checkout',
-          FrameworkRepository.defaultBranch,
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'rev-parse',
-          'HEAD',
-        ], stdout: revision),
-        const FakeCommand(command: <String>[
-          'git',
-          'status',
-          '--porcelain',
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'cherry-pick',
-          '--no-commit',
-          revision,
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'reset',
-          'HEAD',
-          '--hard',
-        ]),
-      ]);
-      final Checkouts checkouts = Checkouts(
-        fileSystem: fileSystem,
-        parentDirectory: fileSystem.directory(rootDir),
-        platform: platform,
-        processManager: processManager,
-        stdio: stdio,
-      );
-      final Repository repository = FrameworkRepository(checkouts);
-      expect(await repository.canCherryPick(revision), true);
-    });
-
-    test('canCherryPick returns false if git cherry-pick returns non-zero', () async {
-      const String commit = 'abc123';
-
-      processManager.addCommands(<FakeCommand>[
-        FakeCommand(command: <String>[
-          'git',
-          'clone',
-          '--origin',
-          'upstream',
-          '--',
-          FrameworkRepository.defaultUpstream,
-          fileSystem.path
-              .join(rootDir, 'flutter_conductor_checkouts', 'framework'),
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'checkout',
-          FrameworkRepository.defaultBranch,
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'rev-parse',
-          'HEAD',
-        ], stdout: commit),
-        const FakeCommand(command: <String>[
-          'git',
-          'status',
-          '--porcelain',
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'cherry-pick',
-          '--no-commit',
-          commit,
-        ], exitCode: 1),
-        const FakeCommand(command: <String>[
-          'git',
-          'diff',
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'reset',
-          'HEAD',
-          '--hard',
-        ]),
-      ]);
-      final Checkouts checkouts = Checkouts(
-        fileSystem: fileSystem,
-        parentDirectory: fileSystem.directory(rootDir),
-        platform: platform,
-        processManager: processManager,
-        stdio: stdio,
-      );
-      final Repository repository = FrameworkRepository(checkouts);
-      expect(await repository.canCherryPick(commit), false);
-    });
-
-    test('cherryPick() applies the commit', () async {
-      const String commit = 'abc123';
-
-      processManager.addCommands(<FakeCommand>[
-        FakeCommand(command: <String>[
-          'git',
-          'clone',
-          '--origin',
-          'upstream',
-          '--',
-          FrameworkRepository.defaultUpstream,
-          fileSystem.path
-              .join(rootDir, 'flutter_conductor_checkouts', 'framework'),
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'checkout',
-          FrameworkRepository.defaultBranch,
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'rev-parse',
-          'HEAD',
-        ], stdout: commit),
-        const FakeCommand(command: <String>[
-          'git',
-          'status',
-          '--porcelain',
-        ]),
-        const FakeCommand(command: <String>[
-          'git',
-          'cherry-pick',
-          commit,
-        ]),
-      ]);
-      final Checkouts checkouts = Checkouts(
-        fileSystem: fileSystem,
-        parentDirectory: fileSystem.directory(rootDir),
-        platform: platform,
-        processManager: processManager,
-        stdio: stdio,
-      );
-      final Repository repository = FrameworkRepository(checkouts);
-      await repository.cherryPick(commit);
-      expect(processManager.hasRemainingExpectations, false);
-    });
-
     test('updateDartRevision() updates the DEPS file', () async {
       const String previousDartRevision = '171876a4e6cf56ee6da1f97d203926bd7afda7ef';
       const String nextDartRevision = 'f6c91128be6b77aef8351e1e3a9d07c85bc2e46e';
@@ -393,15 +238,6 @@ vars = {
       expect(didUpdate, false);
     });
 
-    test('CiYaml(file) will throw if file does not exist', () {
-      final File file = fileSystem.file('/non/existent/file.txt');
-
-      expect(
-        () => CiYaml(file),
-        throwsExceptionWith('Could not find the .ci.yaml file at /non/existent/file.txt'),
-      );
-    });
-
     test('framework repo set as localUpstream ensures requiredLocalBranches exist locally', () async {
       const String commit = 'deadbeef';
       const String candidateBranch = 'flutter-1.2-candidate.3';
@@ -418,7 +254,7 @@ vars = {
         ]),
         FakeCommand(
           command: const <String>['git', 'checkout', candidateBranch, '--'],
-          onRun: () => createdCandidateBranch = true,
+          onRun: (_) => createdCandidateBranch = true,
         ),
         const FakeCommand(
           command: <String>['git', 'checkout', 'stable', '--'],
@@ -473,7 +309,7 @@ vars = {
         ]),
         FakeCommand(
           command: const <String>['git', 'checkout', candidateBranch, '--'],
-          onRun: () => createdCandidateBranch = true,
+          onRun: (_) => createdCandidateBranch = true,
         ),
         const FakeCommand(
           command: <String>['git', 'checkout', EngineRepository.defaultBranch],
