@@ -252,63 +252,83 @@ Future<void> main() async {
         section('gradlew on build script with error');
         await project.introduceError();
         ProcessResult result = await inDirectory(project.rootPath, () {
-          return executeFlutter('build', options: <String>[
-            'apk',
-            '--release',
-          ]);
+          return executeFlutter(
+            'build',
+            options: <String>[
+              'apk',
+              '--release',
+            ],
+            canFail: true,
+          );
         });
 
-        if (result.exitCode == 0)
+        if (result.exitCode == 0) {
           throw failure(
               'Gradle did not exit with error as expected', result);
+        }
         String output = '${result.stdout}\n${result.stderr}';
         if (output.contains('GradleException') ||
             output.contains('Failed to notify') ||
-            output.contains('at org.gradle'))
+            output.contains('at org.gradle')) {
           throw failure(
               'Gradle output should not contain stacktrace', result);
-        if (!output.contains('Build failed'))
+        }
+        if (!output.contains('Build failed')) {
           throw failure(
               'Gradle output should contain a readable error message',
               result);
+        }
 
         section('flutter build apk on build script with error');
         await project.introduceError();
         result = await inDirectory(project.rootPath, () {
-          return executeFlutter('build', options: <String>[
-            'apk',
-            '--release',
-          ]);
+          return executeFlutter(
+            'build',
+            options: <String>[
+              'apk',
+              '--release',
+            ],
+            canFail: true,
+          );
         });
-        if (result.exitCode == 0)
+        if (result.exitCode == 0) {
           throw failure(
               'flutter build apk should fail when Gradle does', result);
+        }
         output = '${result.stdout}\n${result.stderr}';
-        if (!output.contains('Build failed'))
+        if (!output.contains('Build failed')) {
           throw failure(
               'flutter build apk output should contain a readable Gradle error message',
               result);
-        if (hasMultipleOccurrences(output, 'Build failed'))
+        }
+        if (hasMultipleOccurrences(output, 'Build failed')) {
           throw failure(
               'flutter build apk should not invoke Gradle repeatedly on error',
               result);
+        }
       });
 
       await runProjectTest((FlutterProject project) async {
         section('gradlew assembleDebug forwards stderr');
         await project.introducePubspecError();
         final ProcessResult result = await inDirectory(project.rootPath, () {
-          return executeFlutter('build', options: <String>[
-            'apk',
-            '--release',
-          ]);
+          return executeFlutter(
+            'build',
+            options: <String>[
+              'apk',
+              '--release',
+            ],
+            canFail: true,
+          );
         });
-        if (result.exitCode == 0)
+        if (result.exitCode == 0) {
           throw failure(
               'Gradle did not exit with error as expected', result);
+        }
         final String output = '${result.stdout}\n${result.stderr}';
-        if (!output.contains('No file or variants found for asset: lib/gallery/example_code.dart.'))
+        if (!output.contains('No file or variants found for asset: lib/gallery/example_code.dart.')) {
           throw failure(output, result);
+        }
       });
 
       return TaskResult.success(null);
