@@ -32,13 +32,11 @@ class ExtendedStandardMessageCodec extends StandardMessageCodec {
 
   @override
   dynamic readValueOfType(int type, ReadBuffer buffer) {
-    switch (type) {
-    case _dateTime:
-      return DateTime.fromMillisecondsSinceEpoch(buffer.getInt64());
-    case _pair:
-      return Pair(readValue(buffer), readValue(buffer));
-    default: return super.readValueOfType(type, buffer);
-    }
+    return switch (type) {
+      _dateTime => DateTime.fromMillisecondsSinceEpoch(buffer.getInt64()),
+      _pair => Pair(readValue(buffer), readValue(buffer)),
+      _ => super.readValueOfType(type, buffer),
+    };
   }
 }
 
