@@ -184,21 +184,18 @@ class _ListDemoState extends State<ListDemo> {
   }
 
   Widget buildListTile(BuildContext context, String item) {
-    Widget? secondary;
-    if (_itemType == _MaterialListType.twoLine) {
-      secondary = const Text('Additional item information.');
-    } else if (_itemType == _MaterialListType.threeLine) {
-      secondary = const Text(
-        'Even more additional list item information appears on line three.',
-      );
-    }
+    final String? subtitle = switch (_itemType) {
+      _MaterialListType.oneLine || _MaterialListType.oneLineWithAvatar || null => null,
+      _MaterialListType.twoLine   => 'Additional item information.',
+      _MaterialListType.threeLine => 'Even more additional list item information appears on line three.',
+    };
     return MergeSemantics(
       child: ListTile(
         isThreeLine: _itemType == _MaterialListType.threeLine,
         dense: _dense,
         leading: _showAvatars != null ? ExcludeSemantics(child: CircleAvatar(child: Text(item))) : null,
         title: Text('This item represents $item.'),
-        subtitle: secondary,
+        subtitle: subtitle != null ? Text(subtitle) : null,
         trailing: _showIcons != null ? Icon(Icons.info, color: Theme.of(context).disabledColor) : null,
       ),
     );
@@ -207,18 +204,12 @@ class _ListDemoState extends State<ListDemo> {
   @override
   Widget build(BuildContext context) {
     final String layoutText = _dense != null ? ' \u2013 Dense' : '';
-    String? itemTypeText;
-    switch (_itemType) {
-      case _MaterialListType.oneLine:
-      case _MaterialListType.oneLineWithAvatar:
-        itemTypeText = 'Single-line';
-      case _MaterialListType.twoLine:
-        itemTypeText = 'Two-line';
-      case _MaterialListType.threeLine:
-        itemTypeText = 'Three-line';
-      case null:
-        break;
-    }
+    final String? itemTypeText = switch (_itemType) {
+      _MaterialListType.oneLine || _MaterialListType.oneLineWithAvatar => 'Single-line',
+      _MaterialListType.twoLine => 'Two-line',
+      _MaterialListType.threeLine => 'Three-line',
+      null => null,
+    };
 
     Iterable<Widget> listTiles = items.map<Widget>((String item) => buildListTile(context, item));
     if (_showDividers != null) {
