@@ -23,8 +23,9 @@ const double _kRecipePageMaxWidth = 500.0;
 final Set<Recipe?> _favoriteRecipes = <Recipe?>{};
 
 final ThemeData _kTheme = ThemeData(
+  appBarTheme: const AppBarTheme(foregroundColor: Colors.white, backgroundColor: Colors.teal),
   brightness: Brightness.light,
-  primarySwatch: Colors.teal,
+  floatingActionButtonTheme: const FloatingActionButtonThemeData(foregroundColor: Colors.white),
 );
 
 class PestoHome extends StatelessWidget {
@@ -418,10 +419,11 @@ class _RecipePageState extends State<RecipePage> {
 
   void _toggleFavorite() {
     setState(() {
-      if (_favoriteRecipes.contains(widget.recipe))
+      if (_favoriteRecipes.contains(widget.recipe)) {
         _favoriteRecipes.remove(widget.recipe);
-      else
+      } else {
         _favoriteRecipes.add(widget.recipe);
+      }
     });
   }
 }
