@@ -23,7 +23,7 @@ ThemeData _buildDarkTheme() {
     secondary: secondaryColor,
     onPrimary: Colors.white,
     error: const Color(0xFFB00020),
-    background: const Color(0xFF202124),
+    surface: const Color(0xFF202124),
   );
   final ThemeData base = ThemeData(
     useMaterial3: false,
