@@ -4,10 +4,12 @@
 
 // @dart = 2.12
 
-import 'dart:html' as html;
+import 'dart:js_interop';
+
+import 'package:web/web.dart' as web;
 
 // Verify that web applications can be run in sound mode.
-void main() {
+void main() async {
   const bool isWeak = <int?>[] is List<int>;
   String output;
   if (isWeak) {
@@ -15,10 +17,12 @@ void main() {
   } else {
     output = '--- TEST SUCCEEDED ---';
   }
+  await web.window.fetch(
+    '/test-result'.toJS,
+    web.RequestInit(
+      method: 'POST',
+      body: output.toJS,
+    )
+  ).toDart;
   print(output);
-  html.HttpRequest.request(
-    '/test-result',
-    method: 'POST',
-    sendData: output,
-  );
 }
