@@ -2,25 +2,82 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// This script generates `android/build.gradle` for each directory specify in the stdin.
+// For each directory specified in the stdin, this script generates:
+//     1. The top-level build.gradle (android/build.gradle).
+//     2. The top level settings.gradle (android/settings.gradle).
+//     3. The gradle wrapper file (android/gradle/wrapper/gradle-wrapper.properties).
 // Then it generate the lockfiles for each Gradle project.
-// To regenerate these files, run `find . -type d -name 'android' | dart dev/tools/bin/generate_gradle_lockfiles.dart`
+// To regenerate these files, run `find . -type d -name 'android' | dart dev/tools/bin/generate_gradle_lockfiles.dart`.
 
+import 'dart:collection';
 import 'dart:io';
+
+import 'package:args/args.dart';
 import 'package:file/file.dart';
 import 'package:file/local.dart';
 import 'package:path/path.dart' as path;
+import 'package:yaml/yaml.dart';
 
 void main(List<String> arguments) {
-  print(
-    "Usage: find . -type d -name 'android' | dart dev/tools/bin/generate_gradle_lockfiles.dart\n"
-    'If you would rather enter the files manually, just run `dart dev/tools/bin/generate_gradle_lockfiles.dart`,\n'
-    "enter the absolute paths to the app's android directory, then press CTRL-D.\n"
-  );
+  const String usageMessage = "Usage: find . -type d -name 'android' | dart dev/tools/bin/generate_gradle_lockfiles.dart\n"
+      'If you would rather enter the files manually, just run `dart dev/tools/bin/generate_gradle_lockfiles.dart`,\n'
+      "enter the absolute paths to the app's android directory, then press CTRL-D.\n"
+      "If you don't wish to re-generate the settings.gradle, build.gradle, and gradle-wrapper.properties files,\n"
+      'add the flag `--no-gradle-generation`.\n'
+      'This tool automatically excludes a set of android subdirectories, defined at dev/tools/bin/config/lockfile_exclusion.yaml.\n'
+      'To disable this behavior, run with `--no-exclusion`.\n';
+
+  final ArgParser argParser = ArgParser()
+    ..addFlag(
+      'gradle-generation',
+      help: 'Re-generate gradle files in each processed directory.',
+      defaultsTo: true,
+    )..addFlag(
+      'exclusion',
+      help: 'Run the script using the config file at ./configs/lockfile_exclusion.yaml to skip the specified subdirectories.',
+      defaultsTo: true,
+    );
+
+  ArgResults args;
+  try {
+    args = argParser.parse(arguments);
+  } on FormatException catch (error) {
+    stderr.writeln('${error.message}\n');
+    stderr.writeln(usageMessage);
+    exit(1);
+  }
+
+  print(usageMessage);
+
+  /// Re-generate gradle files in each processed directory.
+  final bool gradleGeneration = (args['gradle-generation'] as bool?) ?? true;
+
+  // Skip android subdirectories specified in the ./config/lockfile_exclusion.yaml file.
+  final bool useExclusion = (args['exclusion'] as bool?) ?? true;
 
   const FileSystem fileSystem = LocalFileSystem();
   final List<String> androidDirectories = getFilesFromStdin();
 
+  final File exclusionFile = fileSystem
+      .currentDirectory.childDirectory('dev').childDirectory('tools').childDirectory('bin')
+      .childDirectory('config')
+      .childFile('lockfile_exclusion.yaml');
+
+  // Load the exclusion set, or make an empty exclusion set.
+  final Set<String> exclusionSet;
+  if (useExclusion) {
+    exclusionSet = HashSet<String>.from(
+        (loadYaml(exclusionFile.readAsStringSync()) as YamlList)
+            .toList()
+            .cast<String>()
+    );
+    print('Loaded exclusion file from ${exclusionFile.path}.');
+  } else {
+    exclusionSet = <String>{};
+    print('Running without exclusion.');
+  }
+
+
   for (final String androidDirectoryPath in androidDirectories) {
     final Directory androidDirectory = fileSystem.directory(path.normalize(androidDirectoryPath));
 
@@ -28,15 +85,37 @@ void main(List<String> arguments) {
       throw '$androidDirectory does not exist';
     }
 
-    final File rootBuildGradle = androidDirectory.childFile('build.gradle');
-    if (!rootBuildGradle.existsSync()) {
-      print('${rootBuildGradle.path} does not exist - skipping');
+    if (exclusionSet.contains(androidDirectory.path)) {
+      print('${androidDirectory.path} is included in the exclusion config file at ${exclusionFile.path} - skipping');
       continue;
     }
 
-    final File settingsGradle = androidDirectory.childFile('settings.gradle');
-    if (!settingsGradle.existsSync()) {
-      print('${settingsGradle.path} does not exist - skipping');
+    late File rootBuildGradle;
+    if (androidDirectory.childFile('build.gradle').existsSync()) {
+      rootBuildGradle = androidDirectory.childFile('build.gradle');
+    } else if (androidDirectory.childFile('build.gradle.kts').existsSync()) {
+      rootBuildGradle = androidDirectory.childFile('build.gradle.kts');
+    } else {
+      print('${androidDirectory.childFile('build.gradle').path}(.kts) does not exist - skipping');
+      continue;
+    }
+
+    late File settingsGradle;
+    if (androidDirectory.childFile('settings.gradle').existsSync()) {
+      settingsGradle = androidDirectory.childFile('settings.gradle');
+    } else if (androidDirectory.childFile('settings.gradle.kts').existsSync()) {
+      settingsGradle = androidDirectory.childFile('settings.gradle.kts');
+    } else {
+      print('${androidDirectory.childFile('settings.gradle').path}(.kts) does not exist - skipping');
+      continue;
+    }
+
+    final File wrapperGradle = androidDirectory
+        .childDirectory('gradle')
+        .childDirectory('wrapper')
+        .childFile('gradle-wrapper.properties');
+    if (!wrapperGradle.existsSync()) {
+      print('${wrapperGradle.path} does not exist - skipping');
       continue;
     }
 
@@ -76,8 +155,11 @@ void main(List<String> arguments) {
       // noop
     }
 
-    rootBuildGradle.writeAsStringSync(rootGradleFileContent);
-    settingsGradle.writeAsStringSync(settingGradleFile);
+    if (gradleGeneration) {
+      rootBuildGradle.writeAsStringSync(rootGradleFileContent);
+      settingsGradle.writeAsStringSync(settingGradleFile);
+      wrapperGradle.writeAsStringSync(wrapperGradleFileContent);
+    }
 
     final String appDirectory = androidDirectory.parent.absolute.path;
 
@@ -88,7 +170,7 @@ void main(List<String> arguments) {
     final File gradleWrapper = androidDirectory.childFile('gradlew');
     // Generate Gradle wrapper if it doesn't exist.
     if (!gradleWrapper.existsSync()) {
-      Process.runSync(
+      exec(
         'flutter',
         <String>['build', 'apk', '--config-only'],
         workingDirectory: appDirectory,
@@ -139,23 +221,6 @@ const String rootGradleFileContent = r'''
 // To update all the build.gradle files in the Flutter repo,
 // See dev/tools/bin/generate_gradle_lockfiles.dart.
 
-buildscript {
-    ext.kotlin_version = '1.7.10'
-    repositories {
-        google()
-        mavenCentral()
-    }
-
-    dependencies {
-        classpath 'com.android.tools.build:gradle:7.3.0'
-        classpath "org.jetbrains.kotlin:kotlin-gradle-plugin:$kotlin_version"
-    }
-
-    configurations.classpath {
-        resolutionStrategy.activateDependencyLocking()
-    }
-}
-
 allprojects {
     repositories {
         google()
@@ -205,12 +270,33 @@ pluginManagement {
 
     includeBuild("${settings.ext.flutterSdkPath}/packages/flutter_tools/gradle")
 
-    plugins {
-        id "dev.flutter.flutter-gradle-plugin" version "1.0.0" apply false
+    repositories {
+        google()
+        mavenCentral()
+        gradlePluginPortal()
     }
 }
 
-include ":app"
+buildscript {
+    dependencyLocking {
+        lockFile = file("${rootProject.projectDir}/buildscript-gradle.lockfile")
+        lockAllConfigurations()
+    }
+}
 
-apply from: "${settings.ext.flutterSdkPath}/packages/flutter_tools/gradle/app_plugin_loader.gradle"
+plugins {
+    id "dev.flutter.flutter-plugin-loader" version "1.0.0"
+    id "com.android.application" version "8.1.0" apply false
+    id "org.jetbrains.kotlin.android" version "1.7.10" apply false
+}
+
+include ":app"
+''';
+
+const String wrapperGradleFileContent = r'''
+distributionBase=GRADLE_USER_HOME
+distributionPath=wrapper/dists
+zipStoreBase=GRADLE_USER_HOME
+zipStorePath=wrapper/dists
+distributionUrl=https\://services.gradle.org/distributions/gradle-8.3-all.zip
 ''';
