@@ -21,19 +21,19 @@ class _${blockName}DefaultsM3 extends FloatingActionButtonThemeData {
         highlightElevation: ${elevation("md.comp.fab.primary.pressed.container")},
         enableFeedback: true,
         sizeConstraints: const BoxConstraints.tightFor(
-          width: ${tokens["md.comp.fab.primary.container.width"]},
-          height: ${tokens["md.comp.fab.primary.container.height"]},
+          width: ${getToken("md.comp.fab.primary.container.width")},
+          height: ${getToken("md.comp.fab.primary.container.height")},
         ),
         smallSizeConstraints: const BoxConstraints.tightFor(
-          width: ${tokens["md.comp.fab.primary.small.container.width"]},
-          height: ${tokens["md.comp.fab.primary.small.container.height"]},
+          width: ${getToken("md.comp.fab.primary.small.container.width")},
+          height: ${getToken("md.comp.fab.primary.small.container.height")},
         ),
         largeSizeConstraints: const BoxConstraints.tightFor(
-          width: ${tokens["md.comp.fab.primary.large.container.width"]},
-          height: ${tokens["md.comp.fab.primary.large.container.height"]},
+          width: ${getToken("md.comp.fab.primary.large.container.width")},
+          height: ${getToken("md.comp.fab.primary.large.container.height")},
         ),
         extendedSizeConstraints: const BoxConstraints.tightFor(
-          height: ${tokens["md.comp.extended-fab.primary.container.height"]},
+          height: ${getToken("md.comp.extended-fab.primary.container.height")},
         ),
         extendedIconLabelSpacing: 8.0,
       );
@@ -54,26 +54,22 @@ class _${blockName}DefaultsM3 extends FloatingActionButtonThemeData {
 
   @override
   ShapeBorder? get shape {
-    switch (type) {
-      case _FloatingActionButtonType.regular:
-       return ${shape("md.comp.fab.primary.container")};
-      case _FloatingActionButtonType.small:
-       return ${shape("md.comp.fab.primary.small.container")};
-      case _FloatingActionButtonType.large:
-       return ${shape("md.comp.fab.primary.large.container")};
-      case _FloatingActionButtonType.extended:
-       return ${shape("md.comp.extended-fab.primary.container")};
-     }
+    return switch (type) {
+      _FloatingActionButtonType.regular  => ${shape("md.comp.fab.primary.container")},
+      _FloatingActionButtonType.small    => ${shape("md.comp.fab.primary.small.container")},
+      _FloatingActionButtonType.large    => ${shape("md.comp.fab.primary.large.container")},
+      _FloatingActionButtonType.extended => ${shape("md.comp.extended-fab.primary.container")},
+    };
   }
 
   @override
   double? get iconSize {
-    switch (type) {
-      case _FloatingActionButtonType.regular: return ${tokens["md.comp.fab.primary.icon.size"]};
-      case _FloatingActionButtonType.small: return  ${tokens["md.comp.fab.primary.small.icon.size"]};
-      case _FloatingActionButtonType.large: return ${tokens["md.comp.fab.primary.large.icon.size"]};
-      case _FloatingActionButtonType.extended: return ${tokens["md.comp.extended-fab.primary.icon.size"]};
-    }
+    return switch (type) {
+      _FloatingActionButtonType.regular  => ${getToken("md.comp.fab.primary.icon.size")},
+      _FloatingActionButtonType.small    => ${getToken("md.comp.fab.primary.small.icon.size")},
+      _FloatingActionButtonType.large    => ${getToken("md.comp.fab.primary.large.icon.size")},
+      _FloatingActionButtonType.extended => ${getToken("md.comp.extended-fab.primary.icon.size")},
+    };
   }
 
   @override EdgeInsetsGeometry? get extendedPadding => EdgeInsetsDirectional.only(start: hasChild && _isExtended ? 16.0 : 20.0, end: 20.0);
