@@ -2,31 +2,29 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for InputDecoration
-
 import 'package:flutter/material.dart';
 
-void main() => runApp(const MyApp());
+/// Flutter code sample for [InputDecoration].
 
-class MyApp extends StatelessWidget {
-  const MyApp({super.key});
+void main() => runApp(const InputDecorationExampleApp());
 
-  static const String _title = 'Flutter Code Sample';
+class InputDecorationExampleApp extends StatelessWidget {
+  const InputDecorationExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
     return MaterialApp(
-      title: _title,
+      theme: ThemeData(useMaterial3: true),
       home: Scaffold(
-        appBar: AppBar(title: const Text(_title)),
-        body: const MyStatelessWidget(),
+        appBar: AppBar(title: const Text('InputDecoration Sample')),
+        body: const InputDecorationExample(),
       ),
     );
   }
 }
 
-class MyStatelessWidget extends StatelessWidget {
-  const MyStatelessWidget({super.key});
+class InputDecorationExample extends StatelessWidget {
+  const InputDecorationExample({super.key});
 
   @override
   Widget build(BuildContext context) {
