@@ -2,11 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for FontFeature.FontFeature.numerators
-import 'dart:ui';
-
 import 'package:flutter/widgets.dart';
 
+/// Flutter code sample for [FontFeature.FontFeature.numerators].
+
 void main() => runApp(const ExampleApp());
 
 class ExampleApp extends StatelessWidget {
@@ -27,7 +26,8 @@ class ExampleWidget extends StatelessWidget {
 
   @override
   Widget build(BuildContext context) {
-    // The Piazzolla font can be downloaded from Google Fonts (https://www.google.com/fonts).
+    // The Piazzolla font can be downloaded from Google Fonts
+    // (https://www.google.com/fonts).
     return const Text(
       'Fractions: 1/2 2/3 3/4 4/5',
       style: TextStyle(
