@@ -2,11 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for FontFeature.FontFeature.oldstyleFigures
-import 'dart:ui';
-
 import 'package:flutter/widgets.dart';
 
+/// Flutter code sample for [FontFeature.FontFeature.oldstyleFigures].
+
 void main() => runApp(const ExampleApp());
 
 class ExampleApp extends StatelessWidget {
@@ -26,7 +25,8 @@ class ExampleWidget extends StatelessWidget {
 
   @override
   Widget build(BuildContext context) {
-    // The Piazzolla font can be downloaded from Google Fonts (https://www.google.com/fonts).
+    // The Piazzolla font can be downloaded from Google Fonts
+    // (https://www.google.com/fonts).
     return const Text(
       'Call 311-555-2368 now!',
       style: TextStyle(
