@@ -11,13 +11,19 @@ void main() => runApp(const AnimatedSizeExampleApp());
 class AnimatedSizeExampleApp extends StatelessWidget {
   const AnimatedSizeExampleApp({super.key});
 
+  static const Duration duration = Duration(seconds: 1);
+  static const Curve curve = Curves.easeIn;
+
   @override
   Widget build(BuildContext context) {
     return MaterialApp(
       home: Scaffold(
         appBar: AppBar(title: const Text('AnimatedSize Sample')),
         body: const Center(
-          child: AnimatedSizeExample(),
+          child: AnimatedSizeExample(
+            duration: duration,
+            curve: curve,
+          ),
         ),
       ),
     );
@@ -25,33 +31,42 @@ class AnimatedSizeExampleApp extends StatelessWidget {
 }
 
 class AnimatedSizeExample extends StatefulWidget {
-  const AnimatedSizeExample({super.key});
+  const AnimatedSizeExample({
+    required this.duration,
+    required this.curve,
+    super.key,
+  });
+
+  final Duration duration;
+
+  final Curve curve;
 
   @override
   State<AnimatedSizeExample> createState() => _AnimatedSizeExampleState();
 }
 
 class _AnimatedSizeExampleState extends State<AnimatedSizeExample> {
-  double _size = 50.0;
-  bool _large = false;
-
-  void _updateSize() {
-    setState(() {
-      _size = _large ? 250.0 : 100.0;
-      _large = !_large;
-    });
-  }
+  bool _isSelected = false;
 
   @override
   Widget build(BuildContext context) {
     return GestureDetector(
-      onTap: () => _updateSize(),
+      onTap: () {
+        setState(() {
+          _isSelected = !_isSelected;
+        });
+      },
       child: ColoredBox(
         color: Colors.amberAccent,
         child: AnimatedSize(
-          curve: Curves.easeIn,
-          duration: const Duration(seconds: 1),
-          child: FlutterLogo(size: _size),
+          duration: widget.duration,
+          curve: widget.curve,
+          child: SizedBox.square(
+            dimension: _isSelected ? 250.0 : 100.0,
+            child: const Center(
+              child: FlutterLogo(size: 75.0),
+            ),
+          ),
         ),
       ),
     );
