@@ -2,61 +2,53 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for Focus
-
 import 'package:flutter/material.dart';
 import 'package:flutter/services.dart';
 
-void main() => runApp(const MyApp());
+/// Flutter code sample for [Focus].
 
-class MyApp extends StatelessWidget {
-  const MyApp({super.key});
+void main() => runApp(const FocusExampleApp());
 
-  static const String _title = 'Flutter Code Sample';
+class FocusExampleApp extends StatelessWidget {
+  const FocusExampleApp({super.key});
 
   @override
   Widget build(BuildContext context) {
     return MaterialApp(
-      title: _title,
       home: Scaffold(
-        appBar: AppBar(title: const Text(_title)),
-        body: const MyStatefulWidget(),
+        appBar: AppBar(title: const Text('Focus Sample')),
+        body: const FocusExample(),
       ),
     );
   }
 }
 
-class MyStatefulWidget extends StatefulWidget {
-  const MyStatefulWidget({super.key});
+class FocusExample extends StatefulWidget {
+  const FocusExample({super.key});
 
   @override
-  State<MyStatefulWidget> createState() => _MyStatefulWidgetState();
+  State<FocusExample> createState() => _FocusExampleState();
 }
 
-class _MyStatefulWidgetState extends State<MyStatefulWidget> {
+class _FocusExampleState extends State<FocusExample> {
   Color _color = Colors.white;
 
-  KeyEventResult _handleKeyPress(FocusNode node, RawKeyEvent event) {
-    if (event is RawKeyDownEvent) {
+  KeyEventResult _handleKeyPress(FocusNode node, KeyEvent event) {
+    if (event is KeyDownEvent) {
       debugPrint('Focus node ${node.debugLabel} got key event: ${event.logicalKey}');
-      if (event.logicalKey == LogicalKeyboardKey.keyR) {
-        debugPrint('Changing color to red.');
-        setState(() {
-          _color = Colors.red;
-        });
-        return KeyEventResult.handled;
-      } else if (event.logicalKey == LogicalKeyboardKey.keyG) {
-        debugPrint('Changing color to green.');
-        setState(() {
-          _color = Colors.green;
-        });
-        return KeyEventResult.handled;
-      } else if (event.logicalKey == LogicalKeyboardKey.keyB) {
-        debugPrint('Changing color to blue.');
-        setState(() {
-          _color = Colors.blue;
-        });
-        return KeyEventResult.handled;
+      switch (event.logicalKey) {
+        case LogicalKeyboardKey.keyR:
+          debugPrint('Changing color to red.');
+          setState(() { _color = Colors.red; });
+          return KeyEventResult.handled;
+        case LogicalKeyboardKey.keyG:
+          debugPrint('Changing color to green.');
+          setState(() { _color = Colors.green; });
+          return KeyEventResult.handled;
+        case LogicalKeyboardKey.keyB:
+          debugPrint('Changing color to blue.');
+          setState(() { _color = Colors.blue; });
+          return KeyEventResult.handled;
       }
     }
     return KeyEventResult.ignored;
@@ -69,9 +61,9 @@ class _MyStatefulWidgetState extends State<MyStatefulWidget> {
       debugLabel: 'Scope',
       autofocus: true,
       child: DefaultTextStyle(
-        style: textTheme.headline4!,
+        style: textTheme.headlineMedium!,
         child: Focus(
-          onKey: _handleKeyPress,
+          onKeyEvent: _handleKeyPress,
           debugLabel: 'Button',
           child: Builder(
             builder: (BuildContext context) {
@@ -91,9 +83,7 @@ class _MyStatefulWidgetState extends State<MyStatefulWidget> {
                     height: 100,
                     alignment: Alignment.center,
                     color: hasFocus ? _color : Colors.white,
-                    child: Text(hasFocus
-                        ? "I'm in color! Press R,G,B!"
-                        : 'Press to focus'),
+                    child: Text(hasFocus ? "I'm in color! Press R,G,B!" : 'Press to focus'),
                   ),
                 ),
               );
