@@ -2,11 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// Flutter code sample for InteractiveViewer.builder
-
 import 'package:flutter/material.dart';
 import 'package:vector_math/vector_math_64.dart' show Quad, Vector3;
 
+/// Flutter code sample for [InteractiveViewer.builder].
+
 void main() => runApp(const IVBuilderExampleApp());
 
 class IVBuilderExampleApp extends StatelessWidget {
@@ -36,7 +36,8 @@ class _IVBuilderExampleState extends State<_IVBuilderExample> {
   static const double _cellWidth = 160.0;
   static const double _cellHeight = 80.0;
 
-  // Returns the axis aligned bounding box for the given Quad, which might not be axis aligned.
+  // Returns the axis aligned bounding box for the given Quad, which might not
+  // be axis aligned.
   Rect axisAlignedBoundingBox(Quad quad) {
     double xMin = quad.point0.x;
     double xMax = quad.point0.x;
@@ -79,9 +80,7 @@ class _IVBuilderExampleState extends State<_IVBuilderExample> {
                   return Container(
                     height: _cellHeight,
                     width: _cellWidth,
-                    color: row % 2 + column % 2 == 1
-                        ? Colors.white
-                        : Colors.grey.withOpacity(0.1),
+                    color: row % 2 + column % 2 == 1 ? Colors.white : Colors.grey.withOpacity(0.1),
                     child: Align(
                       child: Text('$row x $column'),
                     ),
@@ -96,8 +95,7 @@ class _IVBuilderExampleState extends State<_IVBuilderExample> {
   }
 }
 
-typedef _CellBuilder = Widget Function(
-    BuildContext context, int row, int column);
+typedef _CellBuilder = Widget Function(BuildContext context, int row, int column);
 
 class _TableBuilder extends StatelessWidget {
   const _TableBuilder({
@@ -124,7 +122,7 @@ class _TableBuilder extends StatelessWidget {
     return SizedBox(
       // Stack needs constraints, even though we then Clip.none outside of them.
       // InteractiveViewer.builder always sets constrained to false, giving infinite constraints to the child.
-      // See: https://master-api.flutter.dev/flutter/widgets/InteractiveViewer/constrained.html
+      // See: https://api.flutter.dev/flutter/widgets/InteractiveViewer/constrained.html
       width: 1,
       height: 1,
       child: Stack(
