@@ -2,18 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'package:flutter/material.dart';
 import 'package:flutter_api_samples/material/dropdown/dropdown_button.0.dart' as example;
 import 'package:flutter_test/flutter_test.dart';
 
 void main() {
   testWidgets('Select an item from DropdownButton', (WidgetTester tester) async {
     await tester.pumpWidget(
-      const MaterialApp(
-        home: Scaffold(
-          body: example.DropdownButtonApp(),
-        ),
-      ),
+      const example.DropdownButtonApp(),
     );
 
     expect(find.text('One'), findsOneWidget);
