@@ -11,7 +11,7 @@ void main() {
     const String tooltipText = 'I am a Tooltip';
 
     await tester.pumpWidget(
-      const example.MyApp(),
+      const example.TooltipExampleApp(),
     );
 
     // Tooltip is not visible before tapping the button.
@@ -20,7 +20,8 @@ void main() {
     await tester.tap(find.byType(FloatingActionButton));
     await tester.pump(const Duration(milliseconds: 10));
     expect(find.text(tooltipText), findsOneWidget);
-    // Wait for the tooltip to disappear.
+    // Tap on the tooltip and wait for the tooltip to disappear.
+    await tester.tap(find.byTooltip(tooltipText));
     await tester.pump(const Duration(seconds: 1));
     expect(find.text(tooltipText), findsNothing);
   });
