@@ -10,7 +10,7 @@ import 'package:flutter_test/flutter_test.dart';
 void main() {
   testWidgets('MouseRegion detects mouse hover', (WidgetTester tester) async {
     await tester.pumpWidget(
-      const MaterialApp(home: example.MouseRegionApp()),
+      const example.MouseRegionApp(),
     );
 
     Container container = tester.widget<Container>(find.byType(Container));
