@@ -6,7 +6,9 @@ import 'dart:async';
 import 'dart:io';
 import 'dart:ui';
 
-import 'package:camera/camera.dart';
+import 'package:camera_avfoundation/camera_avfoundation.dart';
+import 'package:camera_example/camera_controller.dart';
+import 'package:camera_platform_interface/camera_platform_interface.dart';
 import 'package:flutter/painting.dart';
 import 'package:flutter_test/flutter_test.dart';
 import 'package:integration_test/integration_test.dart';
@@ -19,6 +21,7 @@ void main() {
   IntegrationTestWidgetsFlutterBinding.ensureInitialized();
 
   setUpAll(() async {
+    CameraPlatform.instance = AVFoundationCamera();
     final Directory extDir = await getTemporaryDirectory();
     testDir = await Directory('${extDir.path}/test').create(recursive: true);
   });
@@ -29,10 +32,8 @@ void main() {
 
   final Map<ResolutionPreset, Size> presetExpectedSizes =
       <ResolutionPreset, Size>{
-    ResolutionPreset.low:
-        Platform.isAndroid ? const Size(240, 320) : const Size(288, 352),
-    ResolutionPreset.medium:
-        Platform.isAndroid ? const Size(480, 720) : const Size(480, 640),
+    ResolutionPreset.low: const Size(288, 352),
+    ResolutionPreset.medium: const Size(480, 640),
     ResolutionPreset.high: const Size(720, 1280),
     ResolutionPreset.veryHigh: const Size(1080, 1920),
     ResolutionPreset.ultraHigh: const Size(2160, 3840),
@@ -71,7 +72,8 @@ void main() {
 
   testWidgets('Capture specific image resolutions',
       (WidgetTester tester) async {
-    final List<CameraDescription> cameras = await availableCameras();
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
     if (cameras.isEmpty) {
       return;
     }
@@ -117,36 +119,34 @@ void main() {
         expectedSize, Size(video.height, video.width));
   }
 
-  testWidgets(
-    'Capture specific video resolutions',
-    (WidgetTester tester) async {
-      final List<CameraDescription> cameras = await availableCameras();
-      if (cameras.isEmpty) {
-        return;
+  testWidgets('Capture specific video resolutions',
+      (WidgetTester tester) async {
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
+    if (cameras.isEmpty) {
+      return;
+    }
+    for (final CameraDescription cameraDescription in cameras) {
+      bool previousPresetExactlySupported = true;
+      for (final MapEntry<ResolutionPreset, Size> preset
+          in presetExpectedSizes.entries) {
+        final CameraController controller =
+            CameraController(cameraDescription, preset.key);
+        await controller.initialize();
+        await controller.prepareForVideoRecording();
+        final bool presetExactlySupported =
+            await testCaptureVideoResolution(controller, preset.key);
+        assert(!(!previousPresetExactlySupported && presetExactlySupported),
+            'The camera took higher resolution pictures at a lower resolution.');
+        previousPresetExactlySupported = presetExactlySupported;
+        await controller.dispose();
       }
-      for (final CameraDescription cameraDescription in cameras) {
-        bool previousPresetExactlySupported = true;
-        for (final MapEntry<ResolutionPreset, Size> preset
-            in presetExpectedSizes.entries) {
-          final CameraController controller =
-              CameraController(cameraDescription, preset.key);
-          await controller.initialize();
-          await controller.prepareForVideoRecording();
-          final bool presetExactlySupported =
-              await testCaptureVideoResolution(controller, preset.key);
-          assert(!(!previousPresetExactlySupported && presetExactlySupported),
-              'The camera took higher resolution pictures at a lower resolution.');
-          previousPresetExactlySupported = presetExactlySupported;
-          await controller.dispose();
-        }
-      }
-    },
-    // TODO(egarciad): Fix https://github.com/flutter/flutter/issues/93686.
-    skip: true,
-  );
+    }
+  });
 
   testWidgets('Pause and resume video recording', (WidgetTester tester) async {
-    final List<CameraDescription> cameras = await availableCameras();
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
     if (cameras.isEmpty) {
       return;
     }
@@ -196,73 +196,11 @@ void main() {
     await videoController.dispose();
 
     expect(duration, lessThan(recordingTime - timePaused));
-  }, skip: !Platform.isAndroid);
-
-  testWidgets(
-    'Android image streaming',
-    (WidgetTester tester) async {
-      final List<CameraDescription> cameras = await availableCameras();
-      if (cameras.isEmpty) {
-        return;
-      }
-
-      final CameraController controller = CameraController(
-        cameras[0],
-        ResolutionPreset.low,
-        enableAudio: false,
-      );
-
-      await controller.initialize();
-      bool isDetecting = false;
-
-      await controller.startImageStream((CameraImage image) {
-        if (isDetecting) {
-          return;
-        }
-
-        isDetecting = true;
-
-        expectLater(image, isNotNull).whenComplete(() => isDetecting = false);
-      });
-
-      expect(controller.value.isStreamingImages, true);
-
-      sleep(const Duration(milliseconds: 500));
-
-      await controller.stopImageStream();
-      await controller.dispose();
-    },
-    skip: !Platform.isAndroid,
-  );
-
-  /// Start streaming with specifying the ImageFormatGroup.
-  Future<CameraImage> startStreaming(List<CameraDescription> cameras,
-      ImageFormatGroup? imageFormatGroup) async {
-    final CameraController controller = CameraController(
-      cameras.first,
-      ResolutionPreset.low,
-      enableAudio: false,
-      imageFormatGroup: imageFormatGroup,
-    );
-
-    await controller.initialize();
-    final Completer<CameraImage> completer = Completer<CameraImage>();
-
-    await controller.startImageStream((CameraImage image) {
-      if (!completer.isCompleted) {
-        Future<void>(() async {
-          await controller.stopImageStream();
-          await controller.dispose();
-        }).then((Object? value) {
-          completer.complete(image);
-        });
-      }
-    });
-    return completer.future;
-  }
+  });
 
   testWidgets('Set description while recording', (WidgetTester tester) async {
-    final List<CameraDescription> cameras = await availableCameras();
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
     if (cameras.length < 2) {
       return;
     }
@@ -283,7 +221,8 @@ void main() {
   });
 
   testWidgets('Set description', (WidgetTester tester) async {
-    final List<CameraDescription> cameras = await availableCameras();
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
     if (cameras.length < 2) {
       return;
     }
@@ -300,15 +239,42 @@ void main() {
     expect(controller.description, cameras[1]);
   });
 
+  /// Start streaming with specifying the ImageFormatGroup.
+  Future<CameraImageData> startStreaming(List<CameraDescription> cameras,
+      ImageFormatGroup? imageFormatGroup) async {
+    final CameraController controller = CameraController(
+      cameras.first,
+      ResolutionPreset.low,
+      enableAudio: false,
+      imageFormatGroup: imageFormatGroup,
+    );
+
+    await controller.initialize();
+    final Completer<CameraImageData> completer = Completer<CameraImageData>();
+
+    await controller.startImageStream((CameraImageData image) {
+      if (!completer.isCompleted) {
+        Future<void>(() async {
+          await controller.stopImageStream();
+          await controller.dispose();
+        }).then((Object? value) {
+          completer.complete(image);
+        });
+      }
+    });
+    return completer.future;
+  }
+
   testWidgets(
-    'iOS image streaming with imageFormatGroup',
+    'image streaming with imageFormatGroup',
     (WidgetTester tester) async {
-      final List<CameraDescription> cameras = await availableCameras();
+      final List<CameraDescription> cameras =
+          await CameraPlatform.instance.availableCameras();
       if (cameras.isEmpty) {
         return;
       }
 
-      CameraImage image = await startStreaming(cameras, null);
+      CameraImageData image = await startStreaming(cameras, null);
       expect(image, isNotNull);
       expect(image.format.group, ImageFormatGroup.bgra8888);
       expect(image.planes.length, 1);
@@ -323,6 +289,178 @@ void main() {
       expect(image.format.group, ImageFormatGroup.bgra8888);
       expect(image.planes.length, 1);
     },
-    skip: !Platform.isIOS,
   );
+
+  testWidgets('Recording with video streaming', (WidgetTester tester) async {
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
+    if (cameras.isEmpty) {
+      return;
+    }
+
+    final CameraController controller = CameraController(
+      cameras[0],
+      ResolutionPreset.low,
+      enableAudio: false,
+    );
+
+    await controller.initialize();
+    await controller.prepareForVideoRecording();
+    final Completer<CameraImageData> completer = Completer<CameraImageData>();
+    await controller.startVideoRecording(
+        streamCallback: (CameraImageData image) {
+      if (!completer.isCompleted) {
+        completer.complete(image);
+      }
+    });
+    sleep(const Duration(milliseconds: 500));
+    await controller.stopVideoRecording();
+    await controller.dispose();
+
+    expect(await completer.future, isNotNull);
+  });
+
+  // Test fileFormat is respected when taking a picture.
+  testWidgets('Capture specific image output formats',
+      (WidgetTester tester) async {
+    final List<CameraDescription> cameras =
+        await CameraPlatform.instance.availableCameras();
+    if (cameras.isEmpty) {
+      return;
+    }
+    for (final CameraDescription cameraDescription in cameras) {
+      for (final ImageFileFormat fileFormat in ImageFileFormat.values) {
+        final CameraController controller =
+            CameraController(cameraDescription, ResolutionPreset.low);
+        await controller.initialize();
+        await controller.setImageFileFormat(fileFormat);
+        final XFile file = await controller.takePicture();
+        await controller.dispose();
+        expect(file.path.endsWith(fileFormat.name), true);
+      }
+    }
+  });
+
+  group('Camera settings', () {
+    testWidgets('Control FPS', (WidgetTester tester) async {
+      final List<CameraDescription> cameras =
+          await CameraPlatform.instance.availableCameras();
+      if (cameras.isEmpty) {
+        return;
+      }
+
+      final List<int> lengths = <int>[];
+      for (final int fps in <int>[10, 30]) {
+        final CameraController controller = CameraController.withSettings(
+          cameras.first,
+          mediaSettings: MediaSettings(
+            resolutionPreset: ResolutionPreset.medium,
+            fps: fps,
+          ),
+        );
+        await controller.initialize();
+        await controller.prepareForVideoRecording();
+
+        // Take Video
+        await controller.startVideoRecording();
+        sleep(const Duration(milliseconds: 500));
+        final XFile file = await controller.stopVideoRecording();
+
+        // Load video size
+        final File videoFile = File(file.path);
+
+        lengths.add(await videoFile.length());
+
+        await controller.dispose();
+      }
+
+      for (int n = 0; n < lengths.length - 1; n++) {
+        expect(lengths[n], lessThan(lengths[n + 1]),
+            reason: 'incrementing fps should increment file size');
+      }
+    });
+
+    testWidgets('Control video bitrate', (WidgetTester tester) async {
+      final List<CameraDescription> cameras =
+          await CameraPlatform.instance.availableCameras();
+      if (cameras.isEmpty) {
+        return;
+      }
+
+      const int kiloBits = 1024;
+      final List<int> lengths = <int>[];
+      for (final int videoBitrate in <int>[100 * kiloBits, 1000 * kiloBits]) {
+        final CameraController controller = CameraController.withSettings(
+          cameras.first,
+          mediaSettings: MediaSettings(
+            resolutionPreset: ResolutionPreset.medium,
+            videoBitrate: videoBitrate,
+          ),
+        );
+        await controller.initialize();
+        await controller.prepareForVideoRecording();
+
+        // Take Video
+        await controller.startVideoRecording();
+        sleep(const Duration(milliseconds: 500));
+        final XFile file = await controller.stopVideoRecording();
+
+        // Load video size
+        final File videoFile = File(file.path);
+
+        lengths.add(await videoFile.length());
+
+        await controller.dispose();
+      }
+
+      for (int n = 0; n < lengths.length - 1; n++) {
+        expect(lengths[n], lessThan(lengths[n + 1]),
+            reason: 'incrementing video bitrate should increment file size');
+      }
+    });
+
+    testWidgets('Control audio bitrate', (WidgetTester tester) async {
+      final List<CameraDescription> cameras =
+          await CameraPlatform.instance.availableCameras();
+      if (cameras.isEmpty) {
+        return;
+      }
+
+      final List<int> lengths = <int>[];
+
+      const int kiloBits = 1024;
+      for (final int audioBitrate in <int>[32 * kiloBits, 64 * kiloBits]) {
+        final CameraController controller = CameraController.withSettings(
+          cameras.first,
+          mediaSettings: MediaSettings(
+              resolutionPreset: ResolutionPreset.low,
+              fps: 5,
+              videoBitrate: 32000,
+              audioBitrate: audioBitrate,
+              enableAudio: true),
+        );
+        await controller.initialize();
+        await controller.prepareForVideoRecording();
+
+        // Take Video
+        await controller.startVideoRecording();
+        sleep(const Duration(milliseconds: 1000));
+        final XFile file = await controller.stopVideoRecording();
+
+        // Load video metadata
+        final File videoFile = File(file.path);
+
+        final int length = await videoFile.length();
+
+        lengths.add(length);
+
+        await controller.dispose();
+      }
+
+      for (int n = 0; n < lengths.length - 1; n++) {
+        expect(lengths[n], lessThan(lengths[n + 1]),
+            reason: 'incrementing audio bitrate should increment file size');
+      }
+    });
+  });
 }
