@@ -14,8 +14,11 @@ import 'package:win32/win32.dart';
 
 // For simplicity, the sample uses a constant magnification factor
 const MAGFACTOR = 2.0;
-const RESTOREDWINDOWSTYLES =
-    WS_SIZEBOX | WS_SYSMENU | WS_CLIPCHILDREN | WS_CAPTION | WS_MAXIMIZEBOX;
+const RESTOREDWINDOWSTYLES = WINDOW_STYLE.WS_SIZEBOX |
+    WINDOW_STYLE.WS_SYSMENU |
+    WINDOW_STYLE.WS_CLIPCHILDREN |
+    WINDOW_STYLE.WS_CAPTION |
+    WINDOW_STYLE.WS_MAXIMIZEBOX;
 
 const timerInterval = 16;
 final windowClassName = TEXT('MagnifierWindow');
@@ -32,8 +35,8 @@ void main() => initApp(winMain);
 
 /// Entry point for the application
 void winMain(int hInstance, List<String> args, int nCmdShow) {
-  final lpfnWndProc = NativeCallable<WindowProc>.isolateLocal(hostWndProc,
-      exceptionalReturn: 0);
+  final lpfnWndProc =
+      NativeCallable<WNDPROC>.isolateLocal(hostWndProc, exceptionalReturn: 0);
 
   if (MagInitialize() == FALSE ||
       !setupMagnifier(hInstance, lpfnWndProc.nativeFunction)) {
@@ -43,7 +46,7 @@ void winMain(int hInstance, List<String> args, int nCmdShow) {
   ShowWindow(hwndHost, nCmdShow);
   UpdateWindow(hwndHost);
 
-  final lpTimerFunc = NativeCallable<TimerProc>.isolateLocal(updateMagWindow);
+  final lpTimerFunc = NativeCallable<TIMERPROC>.isolateLocal(updateMagWindow);
 
   // Create a timer to update the control
   final timerId =
@@ -69,7 +72,7 @@ void winMain(int hInstance, List<String> args, int nCmdShow) {
 int hostWndProc(int hWnd, int message, int wParam, int lParam) {
   switch (message) {
     case WM_KEYDOWN:
-      if (wParam == VK_ESCAPE) {
+      if (wParam == VIRTUAL_KEY.VK_ESCAPE) {
         if (isFullScreen) {
           goPartialScreen();
         }
@@ -109,32 +112,31 @@ int hostWndProc(int hWnd, int message, int wParam, int lParam) {
 /// Registers the window class for the window that contains the magnification
 /// control.
 int registerHostWindowClass(
-    int hInstance, Pointer<NativeFunction<WindowProc>> lpfnWndProc) {
+    int hInstance, Pointer<NativeFunction<WNDPROC>> lpfnWndProc) {
   final wcex = calloc<WNDCLASSEX>()
     ..ref.cbSize = sizeOf<WNDCLASSEX>()
-    ..ref.style = CS_HREDRAW | CS_VREDRAW
+    ..ref.style = WNDCLASS_STYLES.CS_HREDRAW | WNDCLASS_STYLES.CS_VREDRAW
     ..ref.lpfnWndProc = lpfnWndProc
     ..ref.hInstance = hInstance
     ..ref.hCursor = LoadCursor(NULL, IDC_ARROW)
-    ..ref.hbrBackground = COLOR_BTNFACE + 1
+    ..ref.hbrBackground = SYS_COLOR_INDEX.COLOR_BTNFACE + 1
     ..ref.lpszClassName = windowClassName;
 
   return RegisterClassEx(wcex);
 }
 
-bool setupMagnifier(
-    int hInst, Pointer<NativeFunction<WindowProc>> lpfnWndProc) {
+bool setupMagnifier(int hInst, Pointer<NativeFunction<WNDPROC>> lpfnWndProc) {
   // Set bounds of host window according to screen size
   hostWindowRect
     ..ref.top = 0
-    ..ref.bottom = GetSystemMetrics(SM_CYSCREEN) ~/ 4
+    ..ref.bottom = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYSCREEN) ~/ 4
     ..ref.left = 0
-    ..ref.right = GetSystemMetrics(SM_CXSCREEN);
+    ..ref.right = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CXSCREEN);
 
   // Create the host window
   registerHostWindowClass(hInst, lpfnWndProc);
   hwndHost = CreateWindowEx(
-      WS_EX_TOPMOST | WS_EX_LAYERED,
+      WINDOW_EX_STYLE.WS_EX_TOPMOST | WINDOW_EX_STYLE.WS_EX_LAYERED,
       windowClassName,
       windowTitle,
       RESTOREDWINDOWSTYLES,
@@ -150,14 +152,15 @@ bool setupMagnifier(
   if (hwndHost == FALSE) return false;
 
   // Make the window opaque
-  SetLayeredWindowAttributes(hwndHost, 0, 255, LWA_ALPHA);
+  SetLayeredWindowAttributes(
+      hwndHost, 0, 255, LAYERED_WINDOW_ATTRIBUTES_FLAGS.LWA_ALPHA);
 
   // Create a magnifier control that fills the client area
   GetClientRect(hwndHost, magWindowRect);
   hwndMag = CreateWindow(
       TEXT('Magnifier'),
       TEXT('MagnifierWindow'),
-      WS_CHILD | MS_SHOWMAGNIFIEDCURSOR | WS_VISIBLE,
+      WINDOW_STYLE.WS_CHILD | MS_SHOWMAGNIFIEDCURSOR | WINDOW_STYLE.WS_VISIBLE,
       magWindowRect.ref.left,
       magWindowRect.ref.top,
       magWindowRect.ref.right,
@@ -238,16 +241,20 @@ void updateMagWindow(int hwnd, int uMsg, Pointer<Uint32> idEvent, int dwTime) {
     if (sourceRect.left < 0) {
       sourceRect.left = 0;
     }
-    if (sourceRect.left > GetSystemMetrics(SM_CXSCREEN) - width) {
-      sourceRect.left = GetSystemMetrics(SM_CXSCREEN) - width;
+    if (sourceRect.left >
+        GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CXSCREEN) - width) {
+      sourceRect.left =
+          GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CXSCREEN) - width;
     }
     sourceRect.right = sourceRect.left + width;
 
     if (sourceRect.top < 0) {
       sourceRect.top = 0;
     }
-    if (sourceRect.top > GetSystemMetrics(SM_CYSCREEN) - height) {
-      sourceRect.top = GetSystemMetrics(SM_CYSCREEN) - height;
+    if (sourceRect.top >
+        GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYSCREEN) - height) {
+      sourceRect.top =
+          GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYSCREEN) - height;
     }
     sourceRect.bottom = sourceRect.top + height;
 
@@ -256,8 +263,16 @@ void updateMagWindow(int hwnd, int uMsg, Pointer<Uint32> idEvent, int dwTime) {
 
     // Reclaim topmost status, to prevent unmagnified menus from remaining in
     // view.
-    SetWindowPos(hwndHost, HWND_TOPMOST, 0, 0, 0, 0,
-        SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE);
+    SetWindowPos(
+        hwndHost,
+        HWND_TOPMOST,
+        0,
+        0,
+        0,
+        0,
+        SET_WINDOW_POS_FLAGS.SWP_NOACTIVATE |
+            SET_WINDOW_POS_FLAGS.SWP_NOMOVE |
+            SET_WINDOW_POS_FLAGS.SWP_NOSIZE);
 
     // Force redraw.
     InvalidateRect(hwndMag, nullptr, TRUE);
@@ -275,21 +290,26 @@ void goFullScreen() {
   // The window must be styled as layered for proper rendering.
   // It is styled as transparent so that it does not capture mouse clicks.
   SetWindowLongPtr(
-      hwndHost, GWL_EXSTYLE, WS_EX_TOPMOST | WS_EX_LAYERED | WS_EX_TRANSPARENT);
+      hwndHost,
+      WINDOW_LONG_PTR_INDEX.GWL_EXSTYLE,
+      WINDOW_EX_STYLE.WS_EX_TOPMOST |
+          WINDOW_EX_STYLE.WS_EX_LAYERED |
+          WINDOW_EX_STYLE.WS_EX_TRANSPARENT);
 
   // Give the window a system menu so it can be closed on the taskbar.
-  SetWindowLongPtr(hwndHost, GWL_STYLE, WS_CAPTION | WS_SYSMENU);
+  SetWindowLongPtr(hwndHost, WINDOW_LONG_PTR_INDEX.GWL_STYLE,
+      WINDOW_STYLE.WS_CAPTION | WINDOW_STYLE.WS_SYSMENU);
 
   // Calculate the span of the display area.
   final hDC = GetDC(NULL);
-  var xSpan = GetSystemMetrics(SM_CXSCREEN);
-  var ySpan = GetSystemMetrics(SM_CYSCREEN);
+  var xSpan = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CXSCREEN);
+  var ySpan = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYSCREEN);
   ReleaseDC(NULL, hDC);
 
   // Calculate the size of system elements.
-  final xBorder = GetSystemMetrics(SM_CXFRAME);
-  final yCaption = GetSystemMetrics(SM_CYCAPTION);
-  final yBorder = GetSystemMetrics(SM_CYFRAME);
+  final xBorder = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CXFRAME);
+  final yCaption = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYCAPTION);
+  final yBorder = GetSystemMetrics(SYSTEM_METRICS_INDEX.SM_CYFRAME);
 
   // Calculate the window origin and span for full-screen mode.
   final xOrigin = -xBorder;
@@ -297,16 +317,26 @@ void goFullScreen() {
   xSpan += 2 * xBorder;
   ySpan += 2 * yBorder + yCaption;
 
-  SetWindowPos(hwndHost, HWND_TOPMOST, xOrigin, yOrigin, xSpan, ySpan,
-      SWP_SHOWWINDOW | SWP_NOZORDER | SWP_NOACTIVATE);
+  SetWindowPos(
+      hwndHost,
+      HWND_TOPMOST,
+      xOrigin,
+      yOrigin,
+      xSpan,
+      ySpan,
+      SET_WINDOW_POS_FLAGS.SWP_SHOWWINDOW |
+          SET_WINDOW_POS_FLAGS.SWP_NOZORDER |
+          SET_WINDOW_POS_FLAGS.SWP_NOACTIVATE);
 }
 
 /// Makes the host window resizable and focusable.
 void goPartialScreen() {
   isFullScreen = false;
 
-  SetWindowLongPtr(hwndHost, GWL_EXSTYLE, WS_EX_TOPMOST | WS_EX_LAYERED);
-  SetWindowLongPtr(hwndHost, GWL_STYLE, RESTOREDWINDOWSTYLES);
+  SetWindowLongPtr(hwndHost, WINDOW_LONG_PTR_INDEX.GWL_EXSTYLE,
+      WINDOW_EX_STYLE.WS_EX_TOPMOST | WINDOW_EX_STYLE.WS_EX_LAYERED);
+  SetWindowLongPtr(
+      hwndHost, WINDOW_LONG_PTR_INDEX.GWL_STYLE, RESTOREDWINDOWSTYLES);
   SetWindowPos(
       hwndHost,
       HWND_TOPMOST,
@@ -314,5 +344,7 @@ void goPartialScreen() {
       hostWindowRect.ref.top,
       hostWindowRect.ref.right,
       hostWindowRect.ref.bottom,
-      SWP_SHOWWINDOW | SWP_NOZORDER | SWP_NOACTIVATE);
+      SET_WINDOW_POS_FLAGS.SWP_SHOWWINDOW |
+          SET_WINDOW_POS_FLAGS.SWP_NOZORDER |
+          SET_WINDOW_POS_FLAGS.SWP_NOACTIVATE);
 }
