@@ -11,7 +11,7 @@ import 'icons.dart';
 import 'localizations.dart';
 import 'text_field.dart';
 
-export 'package:flutter/services.dart' show SmartQuotesType, SmartDashesType;
+export 'package:flutter/services.dart' show SmartDashesType, SmartQuotesType;
 
 /// A [CupertinoTextField] that mimics the look and behavior of UIKit's
 /// `UISearchTextField`.
@@ -108,10 +108,11 @@ class CupertinoSearchTextField extends StatefulWidget {
     this.decoration,
     this.backgroundColor,
     this.borderRadius,
-    this.padding = const EdgeInsetsDirectional.fromSTEB(3.8, 8, 5, 8),
+    this.keyboardType = TextInputType.text,
+    this.padding = const EdgeInsetsDirectional.fromSTEB(5.5, 8, 5.5, 8),
     this.itemColor = CupertinoColors.secondaryLabel,
     this.itemSize = 20.0,
-    this.prefixInsets = const EdgeInsetsDirectional.fromSTEB(6, 0, 0, 4),
+    this.prefixInsets = const EdgeInsetsDirectional.fromSTEB(6, 0, 0, 3),
     this.prefixIcon = const Icon(CupertinoIcons.search),
     this.suffixInsets = const EdgeInsetsDirectional.fromSTEB(0, 0, 5, 2),
     this.suffixIcon = const Icon(CupertinoIcons.xmark_circle_fill),
@@ -121,18 +122,12 @@ class CupertinoSearchTextField extends StatefulWidget {
     this.focusNode,
     this.smartQuotesType,
     this.smartDashesType,
+    this.enableIMEPersonalizedLearning = true,
     this.autofocus = false,
     this.onTap,
     this.autocorrect = true,
     this.enabled,
-  })  : assert(padding != null),
-        assert(itemColor != null),
-        assert(itemSize != null),
-        assert(prefixInsets != null),
-        assert(suffixInsets != null),
-        assert(suffixIcon != null),
-        assert(suffixMode != null),
-        assert(
+  })  : assert(
           !((decoration != null) && (backgroundColor != null)),
           'Cannot provide both a background color and a decoration\n'
           'To provide both, use "decoration: BoxDecoration(color: '
@@ -195,53 +190,58 @@ class CupertinoSearchTextField extends StatefulWidget {
   // https://github.com/flutter/flutter/issues/13914.
   final BorderRadius? borderRadius;
 
+  /// The keyboard type for this search field.
+  ///
+  /// Defaults to [TextInputType.text].
+  final TextInputType? keyboardType;
+
   /// Sets the padding insets for the text and placeholder.
   ///
-  /// Cannot be null. Defaults to padding that replicates the
-  /// `UISearchTextField` look. The inset values were determined using the
-  /// comparison tool in https://github.com/flutter/platform_tests/.
+  /// Defaults to padding that replicates the `UISearchTextField` look. The
+  /// inset values were determined using the comparison tool in
+  /// https://github.com/flutter/platform_tests/.
   final EdgeInsetsGeometry padding;
 
   /// Sets the color for the suffix and prefix icons.
   ///
-  /// Cannot be null. Defaults to [CupertinoColors.secondaryLabel].
+  /// Defaults to [CupertinoColors.secondaryLabel].
   final Color itemColor;
 
   /// Sets the base icon size for the suffix and prefix icons.
   ///
-  /// Cannot be null. The size of the icon is scaled using the accessibility
-  /// font scale settings. Defaults to `20.0`.
+  /// The size of the icon is scaled using the accessibility font scale
+  /// settings. Defaults to `20.0`.
   final double itemSize;
 
   /// Sets the padding insets for the suffix.
   ///
-  /// Cannot be null. Defaults to padding that replicates the
-  /// `UISearchTextField` suffix look. The inset values were determined using
-  /// the comparison tool in https://github.com/flutter/platform_tests/.
+  /// Defaults to padding that replicates the `UISearchTextField` suffix look.
+  /// The inset values were determined using the comparison tool in
+  /// https://github.com/flutter/platform_tests/.
   final EdgeInsetsGeometry prefixInsets;
 
   /// Sets a prefix widget.
   ///
-  /// Cannot be null. Defaults to an [Icon] widget with the [CupertinoIcons.search] icon.
+  /// Defaults to an [Icon] widget with the [CupertinoIcons.search] icon.
   final Widget prefixIcon;
 
   /// Sets the padding insets for the prefix.
   ///
-  /// Cannot be null. Defaults to padding that replicates the
-  /// `UISearchTextField` prefix look. The inset values were determined using
-  /// the comparison tool in https://github.com/flutter/platform_tests/.
+  /// Defaults to padding that replicates the `UISearchTextField` prefix look.
+  /// The inset values were determined using the comparison tool in
+  /// https://github.com/flutter/platform_tests/.
   final EdgeInsetsGeometry suffixInsets;
 
   /// Sets the suffix widget's icon.
   ///
-  /// Cannot be null. Defaults to the X-Mark [CupertinoIcons.xmark_circle_fill].
-  /// "To change the functionality of the suffix icon, provide a custom
-  /// onSuffixTap callback and specify an intuitive suffixIcon.
+  /// Defaults to the X-Mark [CupertinoIcons.xmark_circle_fill]. "To change the
+  /// functionality of the suffix icon, provide a custom onSuffixTap callback
+  /// and specify an intuitive suffixIcon.
   final Icon suffixIcon;
 
   /// Dictates when the X-Mark (suffix) should be visible.
   ///
-  /// Cannot be null. Defaults to only on when editing.
+  /// Defaults to only on when editing.
   final OverlayVisibilityMode suffixMode;
 
   /// Sets the X-Mark (suffix) action.
@@ -312,6 +312,9 @@ class CupertinoSearchTextField extends StatefulWidget {
   ///  * <https://developer.apple.com/documentation/uikit/uitextinputtraits>
   final SmartDashesType? smartDashesType;
 
+  /// {@macro flutter.services.TextInputConfiguration.enableIMEPersonalizedLearning}
+  final bool enableIMEPersonalizedLearning;
+
   /// Disables the text field when false.
   ///
   /// Text fields in disabled states have a light grey background and don't
@@ -361,6 +364,14 @@ class _CupertinoSearchTextFieldState extends State<CupertinoSearchTextField>
     }
   }
 
+  @override
+  void dispose() {
+    super.dispose();
+    if (widget.controller == null) {
+      _controller?.dispose();
+    }
+  }
+
   void _registerController() {
     assert(_controller != null);
     registerForRestoration(_controller!, 'controller');
@@ -397,8 +408,7 @@ class _CupertinoSearchTextFieldState extends State<CupertinoSearchTextField>
 
     // The icon size will be scaled by a factor of the accessibility text scale,
     // to follow the behavior of `UISearchTextField`.
-    final double scaledIconSize =
-        MediaQuery.textScaleFactorOf(context) * widget.itemSize;
+    final double scaledIconSize = MediaQuery.textScalerOf(context).scale(widget.itemSize);
 
     // If decoration was not provided, create a decoration with the provided
     // background color and border radius.
@@ -440,8 +450,9 @@ class _CupertinoSearchTextFieldState extends State<CupertinoSearchTextField>
       style: widget.style,
       prefix: prefix,
       suffix: suffix,
+      keyboardType: widget.keyboardType,
       onTap: widget.onTap,
-      enabled: widget.enabled,
+      enabled: widget.enabled ?? true,
       suffixMode: widget.suffixMode,
       placeholder: placeholder,
       placeholderStyle: placeholderStyle,
@@ -453,6 +464,7 @@ class _CupertinoSearchTextFieldState extends State<CupertinoSearchTextField>
       autocorrect: widget.autocorrect,
       smartQuotesType: widget.smartQuotesType,
       smartDashesType: widget.smartDashesType,
+      enableIMEPersonalizedLearning: widget.enableIMEPersonalizedLearning,
       textInputAction: TextInputAction.search,
     );
   }
