@@ -2,6 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+/// @docImport 'package:flutter/services.dart';
+///
+/// @docImport 'page_scaffold.dart';
+/// @docImport 'route.dart';
+/// @docImport 'tab_view.dart';
+library;
+
 import 'package:flutter/widgets.dart';
 
 import 'bottom_tab_bar.dart';
@@ -32,8 +39,8 @@ class CupertinoTabController extends ChangeNotifier {
   /// Creates a [CupertinoTabController] to control the tab index of [CupertinoTabScaffold]
   /// and [CupertinoTabBar].
   ///
-  /// The [initialIndex] must not be null and defaults to 0. The value must be
-  /// greater than or equal to 0, and less than the total number of tabs.
+  /// The [initialIndex] defaults to 0. The value must be greater than or equal
+  /// to 0, and less than the total number of tabs.
   CupertinoTabController({ int initialIndex = 0 })
     : _index = initialIndex,
       assert(initialIndex >= 0);
@@ -123,8 +130,6 @@ class CupertinoTabController extends ChangeNotifier {
 ///  * [iOS human interface guidelines](https://developer.apple.com/design/human-interface-guidelines/ios/bars/tab-bars/).
 class CupertinoTabScaffold extends StatefulWidget {
   /// Creates a layout for applications with a tab bar at the bottom.
-  ///
-  /// The [tabBar] and [tabBuilder] arguments must not be null.
   CupertinoTabScaffold({
     super.key,
     required this.tabBar,
@@ -158,15 +163,9 @@ class CupertinoTabScaffold extends StatefulWidget {
   /// If translucent, the main content may slide behind it.
   /// Otherwise, the main content's bottom margin will be offset by its height.
   ///
-  /// By default [tabBar] has its text scale factor set to 1.0 and does not
-  /// respond to text scale factor changes from the operating system, to match
-  /// the native iOS behavior. To override this behavior, wrap each of the [tabBar]'s
-  /// items inside a [MediaQuery] with the desired [MediaQueryData.textScaleFactor]
-  /// value. The text scale factor value from the operating system can be retrieved
-  /// int many ways, such as querying [MediaQuery.textScaleFactorOf] against
-  /// [CupertinoApp]'s [BuildContext].
-  ///
-  /// Must not be null.
+  /// By default [tabBar] disables text scaling to match the native iOS behavior.
+  /// To override this behavior, wrap each of the [tabBar]'s items inside a
+  /// [MediaQuery] with the desired [TextScaler].
   final CupertinoTabBar tabBar;
 
   /// Controls the currently selected tab index of the [tabBar], as well as the
@@ -190,8 +189,6 @@ class CupertinoTabScaffold extends StatefulWidget {
   /// In that case, the child's [BuildContext]'s [MediaQuery] will have a
   /// bottom padding indicating the area of obstructing overlap from the
   /// [tabBar].
-  ///
-  /// Must not be null.
   final IndexedWidgetBuilder tabBuilder;
 
   /// The color of the widget that underlies the entire scaffold.
@@ -205,7 +202,7 @@ class CupertinoTabScaffold extends StatefulWidget {
   /// scaffold, the body can be resized to avoid overlapping the keyboard, which
   /// prevents widgets inside the body from being obscured by the keyboard.
   ///
-  /// Defaults to true and cannot be null.
+  /// Defaults to true.
   final bool resizeToAvoidBottomInset;
 
   /// Restoration ID to save and restore the state of the [CupertinoTabScaffold].
@@ -361,8 +358,7 @@ class _CupertinoTabScaffoldState extends State<CupertinoTabScaffold> with Restor
         children: <Widget>[
           // The main content being at the bottom is added to the stack first.
           content,
-          MediaQuery(
-            data: existingMediaQuery.copyWith(textScaleFactor: 1),
+          MediaQuery.withNoTextScaling(
             child: Align(
               alignment: Alignment.bottomCenter,
               // Override the tab bar's currentIndex to the current tab and hook in
@@ -519,8 +515,8 @@ class RestorableCupertinoTabController extends RestorableChangeNotifier<Cupertin
   /// Creates a [RestorableCupertinoTabController] to control the tab index of
   /// [CupertinoTabScaffold] and [CupertinoTabBar].
   ///
-  /// The `initialIndex` must not be null and defaults to 0. The value must be
-  /// greater than or equal to 0, and less than the total number of tabs.
+  /// The `initialIndex` defaults to zero. The value must be greater than or
+  /// equal to zero, and less than the total number of tabs.
   RestorableCupertinoTabController({ int initialIndex = 0 })
     : assert(initialIndex >= 0),
       _initialIndex = initialIndex;
