@@ -2,6 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+/// @docImport 'package:flutter/foundation.dart';
+/// @docImport 'package:flutter/material.dart';
+/// @docImport 'package:flutter/scheduler.dart';
+library;
+
 import 'dart:async';
 
 import 'package:meta/meta.dart' show visibleForTesting;
@@ -176,12 +181,10 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
               lineStart = currentPosition + 1;
               currentLineIndent += 1;
               state = _LicenseEntryWithLineBreaksParserState.beforeParagraph;
-              break;
             case '\t':
               lineStart = currentPosition + 1;
               currentLineIndent += 8;
               state = _LicenseEntryWithLineBreaksParserState.beforeParagraph;
-              break;
             case '\r':
             case '\n':
             case '\f':
@@ -197,7 +200,6 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
               currentParagraphIndentation = null;
               lineStart = currentPosition + 1;
               state = _LicenseEntryWithLineBreaksParserState.beforeParagraph;
-              break;
             case '[':
               // This is a bit of a hack for the LGPL 2.1, which does something like this:
               //
@@ -224,7 +226,6 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
               }
               state = _LicenseEntryWithLineBreaksParserState.inParagraph;
           }
-          break;
         case _LicenseEntryWithLineBreaksParserState.inParagraph:
           switch (text[currentPosition]) {
             case '\n':
@@ -233,7 +234,6 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
               currentLineIndent = 0;
               lineStart = currentPosition + 1;
               state = _LicenseEntryWithLineBreaksParserState.beforeParagraph;
-              break;
             case '\f':
               addLine();
               result.add(getParagraph());
@@ -242,11 +242,9 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
               currentParagraphIndentation = null;
               lineStart = currentPosition + 1;
               state = _LicenseEntryWithLineBreaksParserState.beforeParagraph;
-              break;
             default:
               state = _LicenseEntryWithLineBreaksParserState.inParagraph;
           }
-          break;
       }
       currentPosition += 1;
     }
@@ -255,11 +253,9 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
         if (lines.isNotEmpty) {
           result.add(getParagraph());
         }
-        break;
       case _LicenseEntryWithLineBreaksParserState.inParagraph:
         addLine();
         result.add(getParagraph());
-        break;
     }
     return result;
   }
@@ -291,11 +287,7 @@ class LicenseEntryWithLineBreaks extends LicenseEntry {
 ///    uses this API to select licenses to show.
 ///  * [AboutListTile], which is a widget that can be added to a [Drawer]. When
 ///    tapped it calls [showAboutDialog].
-class LicenseRegistry {
-  // This class is not meant to be instantiated or extended; this constructor
-  // prevents instantiation and extension.
-  LicenseRegistry._();
-
+abstract final class LicenseRegistry {
   static List<LicenseEntryCollector>? _collectors;
 
   /// Adds licenses to the registry.
