@@ -211,18 +211,12 @@ class _CalendarDatePickerState extends State<CalendarDatePicker> {
     _vibrate();
     setState(() {
       _mode = mode;
-      if (_selectedDate != null) {
-        if (_mode == DatePickerMode.day) {
-          SemanticsService.announce(
-            _localizations.formatMonthYear(_selectedDate!),
-            _textDirection,
-          );
-        } else {
-          SemanticsService.announce(
-            _localizations.formatYear(_selectedDate!),
-            _textDirection,
-          );
-        }
+      if (_selectedDate case final DateTime selected) {
+        final String message = switch (mode) {
+          DatePickerMode.day  => _localizations.formatMonthYear(selected),
+          DatePickerMode.year => _localizations.formatYear(selected),
+        };
+        SemanticsService.announce(message, _textDirection);
       }
     });
   }
@@ -265,6 +259,21 @@ class _CalendarDatePickerState extends State<CalendarDatePicker> {
     setState(() {
       _selectedDate = value;
       widget.onDateChanged(_selectedDate!);
+      switch (Theme.of(context).platform) {
+        case TargetPlatform.linux:
+        case TargetPlatform.macOS:
+        case TargetPlatform.windows:
+          final bool isToday = DateUtils.isSameDay(widget.currentDate, _selectedDate);
+          final String semanticLabelSuffix = isToday ? ', ${_localizations.currentDateLabel}' : '';
+          SemanticsService.announce(
+            '${_localizations.selectedDateLabel} ${_localizations.formatFullDate(_selectedDate!)}$semanticLabelSuffix',
+            _textDirection,
+          );
+        case TargetPlatform.android:
+        case TargetPlatform.iOS:
+        case TargetPlatform.fuchsia:
+          break;
+      }
     });
   }
 
@@ -316,10 +325,10 @@ class _CalendarDatePickerState extends State<CalendarDatePicker> {
         _DatePickerModeToggleButton(
           mode: _mode,
           title: _localizations.formatMonthYear(_currentDisplayedMonthDate),
-          onTitlePressed: () {
-            // Toggle the day/year mode.
-            _handleModeChanged(_mode == DatePickerMode.day ? DatePickerMode.year : DatePickerMode.day);
-          },
+          onTitlePressed: () => _handleModeChanged(switch (_mode) {
+            DatePickerMode.day => DatePickerMode.year,
+            DatePickerMode.year => DatePickerMode.day,
+          }),
         ),
       ],
     );
@@ -384,51 +393,54 @@ class _DatePickerModeToggleButtonState extends State<_DatePickerModeToggleButton
     final TextTheme textTheme = Theme.of(context).textTheme;
     final Color controlColor = colorScheme.onSurface.withOpacity(0.60);
 
-    return Container(
-      padding: const EdgeInsetsDirectional.only(start: 16, end: 4),
+    return SizedBox(
       height: _subHeaderHeight,
-      child: Row(
-        children: <Widget>[
-          Flexible(
-            child: Semantics(
-              label: MaterialLocalizations.of(context).selectYearSemanticsLabel,
-              excludeSemantics: true,
-              button: true,
-              child: SizedBox(
-                height: _subHeaderHeight,
-                child: InkWell(
-                  onTap: widget.onTitlePressed,
-                  child: Padding(
-                    padding: const EdgeInsets.symmetric(horizontal: 8),
-                    child: Row(
-                      children: <Widget>[
-                        Flexible(
-                          child: Text(
-                            widget.title,
-                            overflow: TextOverflow.ellipsis,
-                            style: textTheme.titleSmall?.copyWith(
+      child: Padding(
+        padding: const EdgeInsetsDirectional.only(start: 16, end: 4),
+        child: Row(
+          children: <Widget>[
+            Flexible(
+              child: Semantics(
+                label: MaterialLocalizations.of(context).selectYearSemanticsLabel,
+                excludeSemantics: true,
+                button: true,
+                container: true,
+                child: SizedBox(
+                  height: _subHeaderHeight,
+                  child: InkWell(
+                    onTap: widget.onTitlePressed,
+                    child: Padding(
+                      padding: const EdgeInsets.symmetric(horizontal: 8),
+                      child: Row(
+                        children: <Widget>[
+                          Flexible(
+                            child: Text(
+                              widget.title,
+                              overflow: TextOverflow.ellipsis,
+                              style: textTheme.titleSmall?.copyWith(
+                                color: controlColor,
+                              ),
+                            ),
+                          ),
+                          RotationTransition(
+                            turns: _controller,
+                            child: Icon(
+                              Icons.arrow_drop_down,
                               color: controlColor,
                             ),
                           ),
-                        ),
-                        RotationTransition(
-                          turns: _controller,
-                          child: Icon(
-                            Icons.arrow_drop_down,
-                            color: controlColor,
-                          ),
-                        ),
-                      ],
+                        ],
+                      ),
                     ),
                   ),
                 ),
               ),
             ),
-          ),
-          if (widget.mode == DatePickerMode.day)
-            // Give space for the prev/next month buttons that are underneath this row
-            const SizedBox(width: _monthNavButtonsWidth),
-        ],
+            if (widget.mode == DatePickerMode.day)
+              // Give space for the prev/next month buttons that are underneath this row
+              const SizedBox(width: _monthNavButtonsWidth),
+          ],
+        ),
       ),
     );
   }
@@ -745,25 +757,27 @@ class _MonthPickerState extends State<_MonthPicker> {
     return Semantics(
       child: Column(
         children: <Widget>[
-          Container(
-            padding: const EdgeInsetsDirectional.only(start: 16, end: 4),
+          SizedBox(
             height: _subHeaderHeight,
-            child: Row(
-              children: <Widget>[
-                const Spacer(),
-                IconButton(
-                  icon: const Icon(Icons.chevron_left),
-                  color: controlColor,
-                  tooltip: _isDisplayingFirstMonth ? null : _localizations.previousMonthTooltip,
-                  onPressed: _isDisplayingFirstMonth ? null : _handlePreviousMonth,
-                ),
-                IconButton(
-                  icon: const Icon(Icons.chevron_right),
-                  color: controlColor,
-                  tooltip: _isDisplayingLastMonth ? null : _localizations.nextMonthTooltip,
-                  onPressed: _isDisplayingLastMonth ? null : _handleNextMonth,
-                ),
-              ],
+            child: Padding(
+              padding: const EdgeInsetsDirectional.only(start: 16, end: 4),
+              child: Row(
+                children: <Widget>[
+                  const Spacer(),
+                  IconButton(
+                    icon: const Icon(Icons.chevron_left),
+                    color: controlColor,
+                    tooltip: _isDisplayingFirstMonth ? null : _localizations.previousMonthTooltip,
+                    onPressed: _isDisplayingFirstMonth ? null : _handlePreviousMonth,
+                  ),
+                  IconButton(
+                    icon: const Icon(Icons.chevron_right),
+                    color: controlColor,
+                    tooltip: _isDisplayingLastMonth ? null : _localizations.nextMonthTooltip,
+                    onPressed: _isDisplayingLastMonth ? null : _handleNextMonth,
+                  ),
+                ],
+              ),
             ),
           ),
           Expanded(
@@ -944,7 +958,7 @@ class _DayPickerState extends State<_DayPicker> {
     while (day < daysInMonth) {
       day++;
       if (day < 1) {
-        dayItems.add(Container());
+        dayItems.add(const SizedBox.shrink());
       } else {
         final DateTime dayToBuild = DateTime(year, month, day);
         final bool isDisabled =
@@ -1041,21 +1055,21 @@ class _DayState extends State<_Day> {
     final MaterialStateProperty<Color?> dayOverlayColor = MaterialStateProperty.resolveWith<Color?>(
       (Set<MaterialState> states) => effectiveValue((DatePickerThemeData? theme) => theme?.dayOverlayColor?.resolve(states)),
     );
-    final BoxDecoration decoration = widget.isToday
-      ? BoxDecoration(
+    final OutlinedBorder dayShape = resolve<OutlinedBorder?>((DatePickerThemeData? theme) => theme?.dayShape, states)!;
+    final ShapeDecoration decoration = widget.isToday
+      ? ShapeDecoration(
           color: dayBackgroundColor,
-          border: Border.fromBorderSide(
-            (datePickerTheme.todayBorder ?? defaults.todayBorder!)
-              .copyWith(color: dayForegroundColor)
+          shape: dayShape.copyWith(
+            side: (datePickerTheme.todayBorder ?? defaults.todayBorder!)
+              .copyWith(color: dayForegroundColor),
           ),
-          shape: BoxShape.circle,
         )
-      : BoxDecoration(
+      : ShapeDecoration(
           color: dayBackgroundColor,
-          shape: BoxShape.circle,
+          shape: dayShape,
         );
 
-    Widget dayWidget = Container(
+    Widget dayWidget = DecoratedBox(
       decoration: decoration,
       child: Center(
         child: Text(localizations.formatDecimal(widget.day.day), style: dayStyle?.apply(color: dayForegroundColor)),
@@ -1070,9 +1084,10 @@ class _DayState extends State<_Day> {
       dayWidget = InkResponse(
         focusNode: widget.focusNode,
         onTap: () => widget.onChanged(widget.day),
-        radius: _dayPickerRowHeight / 2 + 4,
         statesController: _statesController,
         overlayColor: dayOverlayColor,
+        customBorder: dayShape,
+        containedInkWell: true,
         child: Semantics(
           // We want the day of month to be spoken first irrespective of the
           // locale-specific preferences or TextDirection. This is because
@@ -1280,12 +1295,11 @@ class _YearPickerState extends State<YearPicker> {
         decoration: decoration,
         height: decorationHeight,
         width: decorationWidth,
-        child: Center(
-          child: Semantics(
-            selected: isSelected,
-            button: true,
-            child: Text(year.toString(), style: itemStyle),
-          ),
+        alignment: Alignment.center,
+        child: Semantics(
+          selected: isSelected,
+          button: true,
+          child: Text(year.toString(), style: itemStyle),
         ),
       ),
     );
