@@ -79,61 +79,24 @@ const Duration _kBaseSettleDuration = Duration(milliseconds: 246);
 /// are a little bit different, see the Material 3 spec at
 /// <https://m3.material.io/components/navigation-drawer/overview> for
 /// more details. While the [Drawer] widget can have only one child, the
-/// [NavigationDrawer] widget can have list of widgets, which typically contains
+/// [NavigationDrawer] widget can have a list of widgets, which typically contains
 /// [NavigationDrawerDestination] widgets and/or customized widgets like headlines
 /// and dividers.
 ///
-/// {@tool snippet}
+/// {@tool dartpad}
 /// This example shows how to create a [Scaffold] that contains an [AppBar] and
 /// a [Drawer]. A user taps the "menu" icon in the [AppBar] to open the
 /// [Drawer]. The [Drawer] displays four items: A header and three menu items.
 /// The [Drawer] displays the four items using a [ListView], which allows the
 /// user to scroll through the items if need be.
 ///
-/// ```dart
-/// Scaffold(
-///   appBar: AppBar(
-///     title: const Text('Drawer Demo'),
-///   ),
-///   drawer: Drawer(
-///     child: ListView(
-///       padding: EdgeInsets.zero,
-///       children: const <Widget>[
-///         DrawerHeader(
-///           decoration: BoxDecoration(
-///             color: Colors.blue,
-///           ),
-///           child: Text(
-///             'Drawer Header',
-///             style: TextStyle(
-///               color: Colors.white,
-///               fontSize: 24,
-///             ),
-///           ),
-///         ),
-///         ListTile(
-///           leading: Icon(Icons.message),
-///           title: Text('Messages'),
-///         ),
-///         ListTile(
-///           leading: Icon(Icons.account_circle),
-///           title: Text('Profile'),
-///         ),
-///         ListTile(
-///           leading: Icon(Icons.settings),
-///           title: Text('Settings'),
-///         ),
-///       ],
-///     ),
-///   ),
-/// )
-/// ```
+/// ** See code in examples/api/lib/material/drawer/drawer.0.dart **
 /// {@end-tool}
 ///
-/// {@tool snippet}
+/// {@tool dartpad}
 /// This example shows how to migrate the above [Drawer] to a [NavigationDrawer].
 ///
-/// See code in examples/api/lib/material/navigation_drawer/navigation_drawer.1.dart **
+/// ** See code in examples/api/lib/material/navigation_drawer/navigation_drawer.0.dart **
 /// {@end-tool}
 ///
 /// An open drawer may be closed with a swipe to close gesture, pressing the
@@ -212,13 +175,15 @@ class Drawer extends StatelessWidget {
   /// The color used as a surface tint overlay on the drawer's background color,
   /// which reflects the drawer's [elevation].
   ///
-  /// If [ThemeData.useMaterial3] is false property has no effect.
-  ///
-  /// If null and [ThemeData.useMaterial3] is true then [ThemeData]'s
-  /// [ColorScheme.surfaceTint] will be used.
+  /// This is not recommended for use. [Material 3 spec](https://m3.material.io/styles/color/the-color-system/color-roles)
+  /// introduced a set of tone-based surfaces and surface containers in its [ColorScheme],
+  /// which provide more flexibility. The intention is to eventually remove surface tint color from
+  /// the framework.
   ///
   /// To disable this feature, set [surfaceTintColor] to [Colors.transparent].
   ///
+  /// Defaults to [Colors.transparent].
+  ///
   /// See also:
   ///   * [Material.surfaceTintColor], which describes how the surface tint will
   ///     be applied to the background color of the drawer.
@@ -302,7 +267,7 @@ class Drawer extends StatelessWidget {
           shadowColor: shadowColor ?? drawerTheme.shadowColor ?? defaults.shadowColor,
           surfaceTintColor: surfaceTintColor ?? drawerTheme.surfaceTintColor ?? defaults.surfaceTintColor,
           shape: effectiveShape,
-          clipBehavior: effectiveShape != null ? (clipBehavior ?? Clip.hardEdge) : Clip.none,
+          clipBehavior: effectiveShape != null ? (clipBehavior ?? drawerTheme.clipBehavior ?? defaults.clipBehavior!) : Clip.none,
           child: child,
         ),
       ),
@@ -347,7 +312,7 @@ class DrawerController extends StatefulWidget {
   ///
   /// Rarely used directly.
   ///
-  /// The [child] argument must not be null and is typically a [Drawer].
+  /// The [child] argument is typically a [Drawer].
   const DrawerController({
     GlobalKey? key,
     required this.child,
@@ -508,6 +473,7 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
   void dispose() {
     _historyEntry?.remove();
     _controller.dispose();
+    _focusScopeNode.dispose();
     super.dispose();
   }
 
@@ -523,15 +489,8 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
     if (widget.scrimColor != oldWidget.scrimColor) {
       _scrimColorTween = _buildScrimColorTween();
     }
-    if (widget.isDrawerOpen != oldWidget.isDrawerOpen) {
-      switch (_controller.status) {
-        case AnimationStatus.completed:
-        case AnimationStatus.dismissed:
-          _controller.value = widget.isDrawerOpen ? 1.0 : 0.0;
-        case AnimationStatus.forward:
-        case AnimationStatus.reverse:
-          break;
-      }
+    if (widget.isDrawerOpen != oldWidget.isDrawerOpen && !_controller.isAnimating) {
+      _controller.value = widget.isDrawerOpen ? 1.0 : 0.0;
     }
   }
 
@@ -563,7 +522,6 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
         _historyEntry?.remove();
         _historyEntry = null;
       case AnimationStatus.dismissed:
-        break;
       case AnimationStatus.completed:
         break;
     }
@@ -596,28 +554,23 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
 
   double get _width {
     final RenderBox? box = _drawerKey.currentContext?.findRenderObject() as RenderBox?;
-    if (box != null) {
-      return box.size.width;
-    }
-    return _kWidth; // drawer not being shown currently
+    // return _kWidth if drawer not being shown currently
+    return box?.size.width ?? _kWidth;
   }
 
   bool _previouslyOpened = false;
 
+  int get _directionFactor {
+    return switch ((Directionality.of(context), widget.alignment)) {
+      (TextDirection.rtl, DrawerAlignment.start) => -1,
+      (TextDirection.rtl, DrawerAlignment.end)   =>  1,
+      (TextDirection.ltr, DrawerAlignment.start) =>  1,
+      (TextDirection.ltr, DrawerAlignment.end)   => -1,
+    };
+  }
+
   void _move(DragUpdateDetails details) {
-    double delta = details.primaryDelta! / _width;
-    switch (widget.alignment) {
-      case DrawerAlignment.start:
-        break;
-      case DrawerAlignment.end:
-        delta = -delta;
-    }
-    switch (Directionality.of(context)) {
-      case TextDirection.rtl:
-        _controller.value -= delta;
-      case TextDirection.ltr:
-        _controller.value += delta;
-    }
+    _controller.value += details.primaryDelta! / _width * _directionFactor;
 
     final bool opened = _controller.value > 0.5;
     if (opened != _previouslyOpened && widget.drawerCallback != null) {
@@ -630,22 +583,12 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
     if (_controller.isDismissed) {
       return;
     }
-    if (details.velocity.pixelsPerSecond.dx.abs() >= _kMinFlingVelocity) {
-      double visualVelocity = details.velocity.pixelsPerSecond.dx / _width;
-      switch (widget.alignment) {
-        case DrawerAlignment.start:
-          break;
-        case DrawerAlignment.end:
-          visualVelocity = -visualVelocity;
-      }
-      switch (Directionality.of(context)) {
-        case TextDirection.rtl:
-          _controller.fling(velocity: -visualVelocity);
-          widget.drawerCallback?.call(visualVelocity < 0.0);
-        case TextDirection.ltr:
-          _controller.fling(velocity: visualVelocity);
-          widget.drawerCallback?.call(visualVelocity > 0.0);
-      }
+    final double xVelocity = details.velocity.pixelsPerSecond.dx;
+    if (xVelocity.abs() >= _kMinFlingVelocity) {
+      final double visualVelocity = xVelocity / _width * _directionFactor;
+
+      _controller.fling(velocity: visualVelocity);
+      widget.drawerCallback?.call(visualVelocity > 0.0);
     } else if (_controller.value < 0.5) {
       close();
     } else {
@@ -680,52 +623,34 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
   }
 
   AlignmentDirectional get _drawerOuterAlignment {
-    switch (widget.alignment) {
-      case DrawerAlignment.start:
-        return AlignmentDirectional.centerStart;
-      case DrawerAlignment.end:
-        return AlignmentDirectional.centerEnd;
-    }
+    return switch (widget.alignment) {
+      DrawerAlignment.start => AlignmentDirectional.centerStart,
+      DrawerAlignment.end   => AlignmentDirectional.centerEnd,
+    };
   }
 
   AlignmentDirectional get _drawerInnerAlignment {
-    switch (widget.alignment) {
-      case DrawerAlignment.start:
-        return AlignmentDirectional.centerEnd;
-      case DrawerAlignment.end:
-        return AlignmentDirectional.centerStart;
-    }
+    return switch (widget.alignment) {
+      DrawerAlignment.start => AlignmentDirectional.centerEnd,
+      DrawerAlignment.end => AlignmentDirectional.centerStart,
+    };
   }
 
   Widget _buildDrawer(BuildContext context) {
-    final bool drawerIsStart = widget.alignment == DrawerAlignment.start;
-    final TextDirection textDirection = Directionality.of(context);
-    final bool isDesktop;
-    switch (Theme.of(context).platform) {
-      case TargetPlatform.android:
-      case TargetPlatform.iOS:
-      case TargetPlatform.fuchsia:
-        isDesktop = false;
-      case TargetPlatform.macOS:
-      case TargetPlatform.linux:
-      case TargetPlatform.windows:
-        isDesktop = true;
-    }
+    final bool isDesktop = switch (Theme.of(context).platform) {
+      TargetPlatform.android || TargetPlatform.iOS || TargetPlatform.fuchsia => false,
+      TargetPlatform.macOS || TargetPlatform.linux || TargetPlatform.windows => true,
+    };
 
-    double? dragAreaWidth = widget.edgeDragWidth;
-    if (widget.edgeDragWidth == null) {
-      final EdgeInsets padding = MediaQuery.paddingOf(context);
-      switch (textDirection) {
-        case TextDirection.ltr:
-          dragAreaWidth = _kEdgeDragWidth +
-            (drawerIsStart ? padding.left : padding.right);
-        case TextDirection.rtl:
-          dragAreaWidth = _kEdgeDragWidth +
-            (drawerIsStart ? padding.right : padding.left);
-      }
-    }
+    final double dragAreaWidth = widget.edgeDragWidth
+      ?? _kEdgeDragWidth + switch ((widget.alignment, Directionality.of(context))) {
+        (DrawerAlignment.start, TextDirection.ltr) => MediaQuery.paddingOf(context).left,
+        (DrawerAlignment.start, TextDirection.rtl) => MediaQuery.paddingOf(context).right,
+        (DrawerAlignment.end,   TextDirection.rtl) => MediaQuery.paddingOf(context).left,
+        (DrawerAlignment.end,   TextDirection.ltr) => MediaQuery.paddingOf(context).right,
+      };
 
-    if (_controller.status == AnimationStatus.dismissed) {
+    if (_controller.isDismissed) {
       if (widget.enableOpenDragGesture && !isDesktop) {
         return Align(
           alignment: _drawerOuterAlignment,
@@ -736,7 +661,7 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
             behavior: HitTestBehavior.translucent,
             excludeFromSemantics: true,
             dragStartBehavior: widget.dragStartBehavior,
-            child: Container(width: dragAreaWidth),
+            child: LimitedBox(maxHeight: 0.0, child: SizedBox(width: dragAreaWidth, height: double.infinity)),
           ),
         );
       } else {
@@ -755,6 +680,11 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
           platformHasBackButton = false;
       }
 
+      Widget drawerScrim = const LimitedBox(maxWidth: 0.0, maxHeight: 0.0, child: SizedBox.expand());
+      if (_scrimColorTween.evaluate(_controller) case final Color color) {
+        drawerScrim = ColoredBox(color: color, child: drawerScrim);
+      }
+
       final Widget child = _DrawerControllerScope(
         controller: widget,
         child: RepaintBoundary(
@@ -768,9 +698,7 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
                     onTap: close,
                     child: Semantics(
                       label: MaterialLocalizations.of(context).modalBarrierDismissLabel,
-                      child: Container( // The drawer's "scrim"
-                        color: _scrimColorTween.evaluate(_controller),
-                      ),
+                      child: drawerScrim,
                     ),
                   ),
                 ),
@@ -823,7 +751,10 @@ class DrawerControllerState extends State<DrawerController> with SingleTickerPro
 
 class _DrawerDefaultsM2 extends DrawerThemeData {
   const _DrawerDefaultsM2(this.context)
-      : super(elevation: 16.0);
+      : super(
+        elevation: 16.0,
+        clipBehavior: Clip.hardEdge,
+      );
 
   final BuildContext context;
 
@@ -841,16 +772,19 @@ class _DrawerDefaultsM2 extends DrawerThemeData {
 
 class _DrawerDefaultsM3 extends DrawerThemeData {
   _DrawerDefaultsM3(this.context)
-      : super(elevation: 1.0);
+      : super(
+          elevation: 1.0,
+          clipBehavior: Clip.hardEdge,
+        );
 
   final BuildContext context;
   late final TextDirection direction = Directionality.of(context);
 
   @override
-  Color? get backgroundColor => Theme.of(context).colorScheme.surface;
+  Color? get backgroundColor => Theme.of(context).colorScheme.surfaceContainerLow;
 
   @override
-  Color? get surfaceTintColor => Theme.of(context).colorScheme.surfaceTint;
+  Color? get surfaceTintColor => Colors.transparent;
 
   @override
   Color? get shadowColor => Colors.transparent;
