@@ -11,6 +11,7 @@ import 'package:flutter/widgets.dart';
 import 'button.dart';
 import 'color_scheme.dart';
 import 'floating_action_button_theme.dart';
+import 'material_state.dart';
 import 'scaffold.dart';
 import 'text_theme.dart';
 import 'theme.dart';
@@ -51,39 +52,28 @@ enum _FloatingActionButtonType {
 /// action button.
 ///
 /// {@tool dartpad}
-/// This example shows how to display a [FloatingActionButton] in a
-/// [Scaffold], with a pink [backgroundColor] and a thumbs up [Icon].
-///
-/// ![](https://flutter.github.io/assets-for-api-docs/assets/material/floating_action_button.png)
+/// This example shows a [FloatingActionButton] in its usual position within a
+/// [Scaffold]. Pressing the button cycles it through a few variations in its
+/// [foregroundColor], [backgroundColor], and [shape]. The button automatically
+/// animates its segue from one set of visual parameters to another.
 ///
 /// ** See code in examples/api/lib/material/floating_action_button/floating_action_button.0.dart **
 /// {@end-tool}
 ///
 /// {@tool dartpad}
-/// This example shows how to make an extended [FloatingActionButton] in a
-/// [Scaffold], with a  pink [backgroundColor], a thumbs up [Icon] and a
-/// [Text] label that reads "Approve".
-///
-/// ![](https://flutter.github.io/assets-for-api-docs/assets/material/floating_action_button_label.png)
+/// This sample shows all the variants of [FloatingActionButton] widget as
+/// described in: https://m3.material.io/components/floating-action-button/overview.
 ///
 /// ** See code in examples/api/lib/material/floating_action_button/floating_action_button.1.dart **
 /// {@end-tool}
 ///
-/// Material Design 3 introduced new types of floating action buttons.
 /// {@tool dartpad}
-/// This sample shows the creation of [FloatingActionButton] widget in the typical location in a Scaffold,
-/// as described in: https://m3.material.io/components/floating-action-button/overview
+/// This sample shows [FloatingActionButton] with additional color mappings as
+/// described in: https://m3.material.io/components/floating-action-button/overview.
 ///
 /// ** See code in examples/api/lib/material/floating_action_button/floating_action_button.2.dart **
 /// {@end-tool}
 ///
-/// {@tool dartpad}
-/// This sample shows the creation of all the variants of [FloatingActionButton] widget as
-/// described in: https://m3.material.io/components/floating-action-button/overview
-///
-/// ** See code in examples/api/lib/material/floating_action_button/floating_action_button.3.dart **
-/// {@end-tool}
-///
 /// See also:
 ///
 ///  * [Scaffold], in which floating action buttons typically live.
@@ -93,9 +83,8 @@ enum _FloatingActionButtonType {
 class FloatingActionButton extends StatelessWidget {
   /// Creates a circular floating action button.
   ///
-  /// The [mini] and [clipBehavior] arguments must not be null. Additionally,
-  /// [elevation], [highlightElevation], and [disabledElevation] (if specified)
-  /// must be non-negative.
+  /// The [elevation], [highlightElevation], and [disabledElevation] parameters,
+  /// if specified, must be non-negative.
   const FloatingActionButton({
     super.key,
     this.child,
@@ -126,10 +115,6 @@ class FloatingActionButton extends StatelessWidget {
        assert(hoverElevation == null || hoverElevation >= 0.0),
        assert(highlightElevation == null || highlightElevation >= 0.0),
        assert(disabledElevation == null || disabledElevation >= 0.0),
-       assert(mini != null),
-       assert(clipBehavior != null),
-       assert(isExtended != null),
-       assert(autofocus != null),
        _floatingActionButtonType = mini ? _FloatingActionButtonType.small : _FloatingActionButtonType.regular,
        _extendedLabel = null,
        extendedIconLabelSpacing = null,
@@ -141,10 +126,8 @@ class FloatingActionButton extends StatelessWidget {
   /// This constructor overrides the default size constraints of the floating
   /// action button.
   ///
-  /// The [clipBehavior] and [autofocus] arguments must not be null.
-  /// Additionally, [elevation], [focusElevation], [hoverElevation],
-  /// [highlightElevation], and [disabledElevation] (if specified) must be
-  /// non-negative.
+  /// The [elevation], [focusElevation], [hoverElevation], [highlightElevation],
+  /// and [disabledElevation] parameters, if specified, must be non-negative.
   const FloatingActionButton.small({
     super.key,
     this.child,
@@ -173,8 +156,6 @@ class FloatingActionButton extends StatelessWidget {
        assert(hoverElevation == null || hoverElevation >= 0.0),
        assert(highlightElevation == null || highlightElevation >= 0.0),
        assert(disabledElevation == null || disabledElevation >= 0.0),
-       assert(clipBehavior != null),
-       assert(autofocus != null),
        _floatingActionButtonType = _FloatingActionButtonType.small,
        mini = true,
        isExtended = false,
@@ -188,10 +169,8 @@ class FloatingActionButton extends StatelessWidget {
   /// This constructor overrides the default size constraints of the floating
   /// action button.
   ///
-  /// The [clipBehavior] and [autofocus] arguments must not be null.
-  /// Additionally, [elevation], [focusElevation], [hoverElevation],
-  /// [highlightElevation], and [disabledElevation] (if specified) must be
-  /// non-negative.
+  /// The [elevation], [focusElevation], [hoverElevation], [highlightElevation],
+  /// and [disabledElevation] parameters, if specified, must be non-negative.
   const FloatingActionButton.large({
     super.key,
     this.child,
@@ -220,8 +199,6 @@ class FloatingActionButton extends StatelessWidget {
        assert(hoverElevation == null || hoverElevation >= 0.0),
        assert(highlightElevation == null || highlightElevation >= 0.0),
        assert(disabledElevation == null || disabledElevation >= 0.0),
-       assert(clipBehavior != null),
-       assert(autofocus != null),
        _floatingActionButtonType = _FloatingActionButtonType.large,
        mini = false,
        isExtended = false,
@@ -233,9 +210,8 @@ class FloatingActionButton extends StatelessWidget {
   /// Creates a wider [StadiumBorder]-shaped floating action button with
   /// an optional [icon] and a [label].
   ///
-  /// The [label], [autofocus], and [clipBehavior] arguments must not be null.
-  /// Additionally, [elevation], [highlightElevation], and [disabledElevation]
-  /// (if specified) must be non-negative.
+  /// The [elevation], [highlightElevation], and [disabledElevation] parameters,
+  /// if specified, must be non-negative.
   ///
   /// See also:
   ///  * <https://m3.material.io/components/extended-fab>
@@ -272,9 +248,6 @@ class FloatingActionButton extends StatelessWidget {
        assert(hoverElevation == null || hoverElevation >= 0.0),
        assert(highlightElevation == null || highlightElevation >= 0.0),
        assert(disabledElevation == null || disabledElevation >= 0.0),
-       assert(isExtended != null),
-       assert(clipBehavior != null),
-       assert(autofocus != null),
        mini = false,
        _floatingActionButtonType = _FloatingActionButtonType.extended,
        child = icon,
@@ -293,36 +266,40 @@ class FloatingActionButton extends StatelessWidget {
 
   /// The default foreground color for icons and text within the button.
   ///
-  /// If this property is null, then the
-  /// [FloatingActionButtonThemeData.foregroundColor] of
-  /// [ThemeData.floatingActionButtonTheme] is used. If that property is also
-  /// null, then the [ColorScheme.onSecondary] color of [ThemeData.colorScheme]
-  /// is used.
+  /// If this property is null, then the [FloatingActionButtonThemeData.foregroundColor]
+  /// of [ThemeData.floatingActionButtonTheme] is used. If that property is also
+  /// null, then the [ColorScheme.onPrimaryContainer] color of [ThemeData.colorScheme]
+  /// is used. If [ThemeData.useMaterial3] is set to false, then the
+  /// [ColorScheme.onSecondary] color of [ThemeData.colorScheme] is used.
   final Color? foregroundColor;
 
   /// The button's background color.
   ///
-  /// If this property is null, then the
-  /// [FloatingActionButtonThemeData.backgroundColor] of
-  /// [ThemeData.floatingActionButtonTheme] is used. If that property is also
-  /// null, then the [Theme]'s [ColorScheme.secondary] color is used.
+  /// If this property is null, then the [FloatingActionButtonThemeData.backgroundColor]
+  /// of [ThemeData.floatingActionButtonTheme] is used. If that property is also
+  /// null, then the [ColorScheme.primaryContainer] color of [ThemeData.colorScheme]
+  /// is used. If [ThemeData.useMaterial3] is set to false, then the
+  /// [ColorScheme.secondary] color of [ThemeData.colorScheme] is used.
   final Color? backgroundColor;
 
   /// The color to use for filling the button when the button has input focus.
   ///
-  /// Defaults to [ThemeData.focusColor] for the current theme.
+  /// In Material3, defaults to [ColorScheme.onPrimaryContainer] with opacity 0.1.
+  /// In Material 2, it defaults to [ThemeData.focusColor] for the current theme.
   final Color? focusColor;
 
   /// The color to use for filling the button when the button has a pointer
   /// hovering over it.
   ///
-  /// Defaults to [ThemeData.hoverColor] for the current theme.
+  /// Defaults to [ThemeData.hoverColor] for the current theme in Material 2. In
+  /// Material 3, defaults to [ColorScheme.onPrimaryContainer] with opacity 0.08.
   final Color? hoverColor;
 
   /// The splash color for this [FloatingActionButton]'s [InkWell].
   ///
   /// If null, [FloatingActionButtonThemeData.splashColor] is used, if that is
-  /// null, [ThemeData.splashColor] is used.
+  /// null, [ThemeData.splashColor] is used in Material 2; [ColorScheme.onPrimaryContainer]
+  /// with opacity 0.1 is used in Material 3.
   final Color? splashColor;
 
   /// The tag to apply to the button's [Hero] widget.
@@ -439,7 +416,7 @@ class FloatingActionButton extends StatelessWidget {
 
   /// {@macro flutter.material.Material.clipBehavior}
   ///
-  /// Defaults to [Clip.none], and must not be null.
+  /// Defaults to [Clip.none].
   final Clip clipBehavior;
 
   /// True if this is an "extended" floating action button.
@@ -500,7 +477,7 @@ class FloatingActionButton extends StatelessWidget {
   /// The text style for an extended [FloatingActionButton]'s label.
   ///
   /// If null, [FloatingActionButtonThemeData.extendedTextStyle] is used. If
-  /// that is also null, then [TextTheme.button] with a letter spacing of 1.2
+  /// that is also null, then [TextTheme.labelLarge] with a letter spacing of 1.2
   /// is used.
   final TextStyle? extendedTextStyle;
 
@@ -566,16 +543,13 @@ class FloatingActionButton extends StatelessWidget {
       data: IconThemeData(size: iconSize),
       child: child!,
     ) : child;
-    switch(_floatingActionButtonType) {
+    switch (_floatingActionButtonType) {
       case _FloatingActionButtonType.regular:
         sizeConstraints = floatingActionButtonTheme.sizeConstraints ?? defaults.sizeConstraints!;
-        break;
       case _FloatingActionButtonType.small:
         sizeConstraints = floatingActionButtonTheme.smallSizeConstraints ?? defaults.smallSizeConstraints!;
-        break;
       case _FloatingActionButtonType.large:
         sizeConstraints = floatingActionButtonTheme.largeSizeConstraints ?? defaults.largeSizeConstraints!;
-        break;
       case _FloatingActionButtonType.extended:
         sizeConstraints = floatingActionButtonTheme.extendedSizeConstraints ?? defaults.extendedSizeConstraints!;
         final double iconLabelSpacing = extendedIconLabelSpacing ?? floatingActionButtonTheme.extendedIconLabelSpacing ?? 8.0;
@@ -598,12 +572,11 @@ class FloatingActionButton extends StatelessWidget {
             ),
           ),
         );
-        break;
     }
 
     Widget result = RawMaterialButton(
       onPressed: onPressed,
-      mouseCursor: mouseCursor,
+      mouseCursor: _EffectiveMouseCursor(mouseCursor, floatingActionButtonTheme.mouseCursor),
       elevation: elevation,
       focusElevation: focusElevation,
       hoverElevation: hoverElevation,
@@ -664,6 +637,26 @@ class FloatingActionButton extends StatelessWidget {
   }
 }
 
+// This MaterialStateProperty is passed along to RawMaterialButton which
+// resolves the property against MaterialState.pressed, MaterialState.hovered,
+// MaterialState.focused, MaterialState.disabled.
+class _EffectiveMouseCursor extends MaterialStateMouseCursor {
+  const _EffectiveMouseCursor(this.widgetCursor, this.themeCursor);
+
+  final MouseCursor? widgetCursor;
+  final MaterialStateProperty<MouseCursor?>? themeCursor;
+
+  @override
+  MouseCursor resolve(Set<MaterialState> states) {
+    return MaterialStateProperty.resolveAs<MouseCursor?>(widgetCursor, states)
+      ?? themeCursor?.resolve(states)
+      ?? MaterialStateMouseCursor.clickable.resolve(states);
+  }
+
+  @override
+  String get debugDescription => 'MaterialStateMouseCursor(FloatActionButton)';
+}
+
 // This widget's size matches its child's size unless its constraints
 // force it to be larger or smaller. The child is centered.
 //
@@ -773,7 +766,7 @@ class _FABDefaultsM2 extends FloatingActionButtonThemeData {
   @override double? get iconSize => _isLarge ? 36.0 : 24.0;
 
   @override EdgeInsetsGeometry? get extendedPadding => EdgeInsetsDirectional.only(start: hasChild && _isExtended ? 16.0 : 20.0, end: 20.0);
-  @override TextStyle? get extendedTextStyle => _theme.textTheme.button!.copyWith(letterSpacing: 1.2);
+  @override TextStyle? get extendedTextStyle => _theme.textTheme.labelLarge!.copyWith(letterSpacing: 1.2);
 }
 
 // BEGIN GENERATED TOKEN PROPERTIES - FAB
@@ -783,8 +776,6 @@ class _FABDefaultsM2 extends FloatingActionButtonThemeData {
 // Design token database by the script:
 //   dev/tools/gen_defaults/bin/gen_defaults.dart.
 
-// Token database version: v0_101
-
 class _FABDefaultsM3 extends FloatingActionButtonThemeData {
   _FABDefaultsM3(this.context, this.type, this.hasChild)
     : super(
@@ -821,32 +812,28 @@ class _FABDefaultsM3 extends FloatingActionButtonThemeData {
 
   @override Color? get foregroundColor => _colors.onPrimaryContainer;
   @override Color? get backgroundColor => _colors.primaryContainer;
-  @override Color? get splashColor => _colors.onPrimaryContainer.withOpacity(0.12);
-  @override Color? get focusColor => _colors.onPrimaryContainer.withOpacity(0.12);
+  @override Color? get splashColor => _colors.onPrimaryContainer.withOpacity(0.1);
+  @override Color? get focusColor => _colors.onPrimaryContainer.withOpacity(0.1);
   @override Color? get hoverColor => _colors.onPrimaryContainer.withOpacity(0.08);
 
   @override
   ShapeBorder? get shape {
-    switch (type) {
-      case _FloatingActionButtonType.regular:
-       return const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: Radius.circular(16.0), topRight: Radius.circular(16.0), bottomLeft: Radius.circular(16.0), bottomRight: Radius.circular(16.0)));
-      case _FloatingActionButtonType.small:
-       return const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: Radius.circular(12.0), topRight: Radius.circular(12.0), bottomLeft: Radius.circular(12.0), bottomRight: Radius.circular(12.0)));
-      case _FloatingActionButtonType.large:
-       return const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: Radius.circular(28.0), topRight: Radius.circular(28.0), bottomLeft: Radius.circular(28.0), bottomRight: Radius.circular(28.0)));
-      case _FloatingActionButtonType.extended:
-       return const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: Radius.circular(16.0), topRight: Radius.circular(16.0), bottomLeft: Radius.circular(16.0), bottomRight: Radius.circular(16.0)));
-     }
+    return switch (type) {
+      _FloatingActionButtonType.regular  => const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(16.0))),
+      _FloatingActionButtonType.small    => const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(12.0))),
+      _FloatingActionButtonType.large    => const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(28.0))),
+      _FloatingActionButtonType.extended => const RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(16.0))),
+    };
   }
 
   @override
   double? get iconSize {
-    switch (type) {
-      case _FloatingActionButtonType.regular: return 24.0;
-      case _FloatingActionButtonType.small: return  24.0;
-      case _FloatingActionButtonType.large: return 36.0;
-      case _FloatingActionButtonType.extended: return 24.0;
-    }
+    return switch (type) {
+      _FloatingActionButtonType.regular  => 24.0,
+      _FloatingActionButtonType.small    => 24.0,
+      _FloatingActionButtonType.large    => 36.0,
+      _FloatingActionButtonType.extended => 24.0,
+    };
   }
 
   @override EdgeInsetsGeometry? get extendedPadding => EdgeInsetsDirectional.only(start: hasChild && _isExtended ? 16.0 : 20.0, end: 20.0);
