@@ -266,7 +266,7 @@ class IconButton extends StatelessWidget {
   }) : assert(splashRadius == null || splashRadius > 0),
        _variant = _IconButtonVariant.filledTonal;
 
-  /// Create a filled tonal variant of IconButton.
+  /// Create an outlined variant of IconButton.
   ///
   /// Outlined icon buttons are medium-emphasis buttons. They’re useful when an
   /// icon button needs more emphasis than a standard icon button but less than
@@ -573,9 +573,15 @@ class IconButton extends StatelessWidget {
   /// [ButtonStyle.foregroundColor] value. Specify a value for [foregroundColor]
   /// to specify the color of the button's icons. The [hoverColor], [focusColor]
   /// and [highlightColor] colors are used to indicate the hover, focus,
-  /// and pressed states. Use [backgroundColor] for the button's background
-  /// fill color. Use [disabledForegroundColor] and [disabledBackgroundColor]
-  /// to specify the button's disabled icon and fill color.
+  /// and pressed states if [overlayColor] isn't specified.
+  ///
+  /// If [overlayColor] is specified and its value is [Colors.transparent]
+  /// then the pressed/focused/hovered highlights are effectively defeated.
+  /// Otherwise a [MaterialStateProperty] with the same opacities as the
+  /// default is created.
+  ///
+  /// Use [backgroundColor] for the button's background fill color. Use [disabledForegroundColor]
+  /// and [disabledBackgroundColor] to specify the button's disabled icon and fill color.
   ///
   /// Similarly, the [enabledMouseCursor] and [disabledMouseCursor]
   /// parameters are used to construct [ButtonStyle].mouseCursor.
@@ -611,6 +617,7 @@ class IconButton extends StatelessWidget {
     Color? highlightColor,
     Color? shadowColor,
     Color? surfaceTintColor,
+    Color? overlayColor,
     double? elevation,
     Size? minimumSize,
     Size? fixedSize,
@@ -629,20 +636,24 @@ class IconButton extends StatelessWidget {
     InteractiveInkFeatureFactory? splashFactory,
   }) {
     final MaterialStateProperty<Color?>? buttonBackgroundColor = (backgroundColor == null && disabledBackgroundColor == null)
-        ? null
-        : _IconButtonDefaultBackground(backgroundColor, disabledBackgroundColor);
+      ? null
+      : _IconButtonDefaultBackground(backgroundColor, disabledBackgroundColor);
     final MaterialStateProperty<Color?>? buttonForegroundColor = (foregroundColor == null && disabledForegroundColor == null)
+      ? null
+      : _IconButtonDefaultForeground(foregroundColor, disabledForegroundColor);
+    final MaterialStateProperty<Color?>? overlayColorProp = (foregroundColor == null &&
+      hoverColor == null && focusColor == null && highlightColor == null && overlayColor == null)
         ? null
-        : _IconButtonDefaultForeground(foregroundColor, disabledForegroundColor);
-    final MaterialStateProperty<Color?>? overlayColor = (foregroundColor == null && hoverColor == null && focusColor == null && highlightColor == null)
-        ? null
-        : _IconButtonDefaultOverlay(foregroundColor, focusColor, hoverColor, highlightColor);
+        : switch (overlayColor) {
+            (final Color overlayColor) when overlayColor.value == 0 => const MaterialStatePropertyAll<Color?>(Colors.transparent),
+            _ => _IconButtonDefaultOverlay(foregroundColor, focusColor, hoverColor, highlightColor, overlayColor),
+          };
     final MaterialStateProperty<MouseCursor?> mouseCursor = _IconButtonDefaultMouseCursor(enabledMouseCursor, disabledMouseCursor);
 
     return ButtonStyle(
       backgroundColor: buttonBackgroundColor,
       foregroundColor: buttonForegroundColor,
-      overlayColor: overlayColor,
+      overlayColor: overlayColorProp,
       shadowColor: ButtonStyleButton.allOrNull<Color>(shadowColor),
       surfaceTintColor: ButtonStyleButton.allOrNull<Color>(surfaceTintColor),
       elevation: ButtonStyleButton.allOrNull<double>(elevation),
@@ -795,7 +806,6 @@ class IconButton extends StatelessWidget {
   @override
   void debugFillProperties(DiagnosticPropertiesBuilder properties) {
     super.debugFillProperties(properties);
-    properties.add(DiagnosticsProperty<Widget>('icon', icon, showName: false));
     properties.add(StringProperty('tooltip', tooltip, defaultValue: null, quoted: false));
     properties.add(ObjectFlagProperty<VoidCallback>('onPressed', onPressed, ifNull: 'disabled'));
     properties.add(ColorProperty('color', color, defaultValue: null));
@@ -920,9 +930,9 @@ class _IconButtonM3 extends ButtonStyleButton {
   /// * `overlayColor`
   ///   * selected
   ///      * hovered - Theme.colorScheme.primary(0.08)
-  ///      * focused or pressed - Theme.colorScheme.primary(0.12)
-  ///   * hovered or focused - Theme.colorScheme.onSurfaceVariant(0.08)
-  ///   * pressed - Theme.colorScheme.onSurfaceVariant(0.12)
+  ///      * focused or pressed - Theme.colorScheme.primary(0.1)
+  ///   * hovered - Theme.colorScheme.onSurfaceVariant(0.08)
+  ///   * pressed or focused - Theme.colorScheme.onSurfaceVariant(0.1)
   ///   * others - null
   /// * `shadowColor` - null
   /// * `surfaceTintColor` - null
@@ -945,16 +955,12 @@ class _IconButtonM3 extends ButtonStyleButton {
   /// * `splashFactory` - Theme.splashFactory
   @override
   ButtonStyle defaultStyleOf(BuildContext context) {
-    switch (variant) {
-      case _IconButtonVariant.filled:
-        return _FilledIconButtonDefaultsM3(context, toggleable);
-      case _IconButtonVariant.filledTonal:
-        return _FilledTonalIconButtonDefaultsM3(context, toggleable);
-      case _IconButtonVariant.outlined:
-        return _OutlinedIconButtonDefaultsM3(context, toggleable);
-      case _IconButtonVariant.standard:
-        return _IconButtonDefaultsM3(context, toggleable);
-    }
+    return switch (variant) {
+      _IconButtonVariant.filled      => _FilledIconButtonDefaultsM3(context, toggleable),
+      _IconButtonVariant.filledTonal => _FilledTonalIconButtonDefaultsM3(context, toggleable),
+      _IconButtonVariant.outlined    => _OutlinedIconButtonDefaultsM3(context, toggleable),
+      _IconButtonVariant.standard    => _IconButtonDefaultsM3(context, toggleable),
+    };
   }
 
   /// Returns the [IconButtonThemeData.style] of the closest [IconButtonTheme] ancestor.
@@ -964,16 +970,14 @@ class _IconButtonM3 extends ButtonStyleButton {
   @override
   ButtonStyle? themeStyleOf(BuildContext context) {
     final IconThemeData iconTheme = IconTheme.of(context);
-    final bool isDark = Theme.of(context).brightness == Brightness.dark;
-
-    bool isIconThemeDefault(Color? color) {
-      if (isDark) {
-        return identical(color, kDefaultIconLightColor);
-      }
-      return identical(color, kDefaultIconDarkColor);
-    }
-    final bool isDefaultColor = isIconThemeDefault(iconTheme.color);
     final bool isDefaultSize = iconTheme.size == const IconThemeData.fallback().size;
+    final bool isDefaultColor = identical(
+      iconTheme.color,
+      switch (Theme.of(context).brightness) {
+        Brightness.light => kDefaultIconDarkColor,
+        Brightness.dark => kDefaultIconLightColor,
+      },
+    );
 
     final ButtonStyle iconThemeStyle = IconButton.styleFrom(
       foregroundColor: isDefaultColor ? null : iconTheme.color,
@@ -1028,34 +1032,41 @@ class _IconButtonDefaultForeground extends MaterialStateProperty<Color?> {
 
 @immutable
 class _IconButtonDefaultOverlay extends MaterialStateProperty<Color?> {
-  _IconButtonDefaultOverlay(this.foregroundColor, this.focusColor, this.hoverColor, this.highlightColor);
+  _IconButtonDefaultOverlay(
+    this.foregroundColor,
+    this.focusColor,
+    this.hoverColor,
+    this.highlightColor,
+    this.overlayColor,
+  );
 
   final Color? foregroundColor;
   final Color? focusColor;
   final Color? hoverColor;
   final Color? highlightColor;
+  final Color? overlayColor;
 
   @override
   Color? resolve(Set<MaterialState> states) {
     if (states.contains(MaterialState.selected)) {
       if (states.contains(MaterialState.pressed)) {
-        return highlightColor ?? foregroundColor?.withOpacity(0.12);
+        return highlightColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.1);
       }
       if (states.contains(MaterialState.hovered)) {
-        return hoverColor ?? foregroundColor?.withOpacity(0.08);
+        return hoverColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.08);
       }
       if (states.contains(MaterialState.focused)) {
-        return focusColor ?? foregroundColor?.withOpacity(0.12);
+        return focusColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.1);
       }
     }
     if (states.contains(MaterialState.pressed)) {
-      return highlightColor ?? foregroundColor?.withOpacity(0.12);
+      return highlightColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.1);
     }
     if (states.contains(MaterialState.hovered)) {
-      return hoverColor ?? foregroundColor?.withOpacity(0.08);
+      return hoverColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.08);
     }
     if (states.contains(MaterialState.focused)) {
-      return focusColor ?? foregroundColor?.withOpacity(0.08);
+      return focusColor ?? (overlayColor ?? foregroundColor)?.withOpacity(0.1);
     }
     return null;
   }
@@ -1124,23 +1135,23 @@ class _IconButtonDefaultsM3 extends ButtonStyle {
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
       if (states.contains(MaterialState.selected)) {
         if (states.contains(MaterialState.pressed)) {
-          return _colors.primary.withOpacity(0.12);
+          return _colors.primary.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.primary.withOpacity(0.08);
         }
         if (states.contains(MaterialState.focused)) {
-          return _colors.primary.withOpacity(0.12);
+          return _colors.primary.withOpacity(0.1);
         }
       }
       if (states.contains(MaterialState.pressed)) {
-        return _colors.onSurfaceVariant.withOpacity(0.12);
+        return _colors.onSurfaceVariant.withOpacity(0.1);
       }
       if (states.contains(MaterialState.hovered)) {
         return _colors.onSurfaceVariant.withOpacity(0.08);
       }
       if (states.contains(MaterialState.focused)) {
-        return _colors.onSurfaceVariant.withOpacity(0.12);
+        return _colors.onSurfaceVariant.withOpacity(0.1);
       }
       return Colors.transparent;
     });
@@ -1234,7 +1245,7 @@ class _FilledIconButtonDefaultsM3 extends ButtonStyle {
         return _colors.primary;
       }
       if (toggleable) { // toggleable but unselected case
-        return _colors.surfaceVariant;
+        return _colors.surfaceContainerHighest;
       }
       return _colors.primary;
     });
@@ -1259,34 +1270,34 @@ class _FilledIconButtonDefaultsM3 extends ButtonStyle {
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
       if (states.contains(MaterialState.selected)) {
         if (states.contains(MaterialState.pressed)) {
-          return _colors.onPrimary.withOpacity(0.12);
+          return _colors.onPrimary.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.onPrimary.withOpacity(0.08);
         }
         if (states.contains(MaterialState.focused)) {
-          return _colors.onPrimary.withOpacity(0.12);
+          return _colors.onPrimary.withOpacity(0.1);
         }
       }
       if (toggleable) { // toggleable but unselected case
         if (states.contains(MaterialState.pressed)) {
-          return _colors.primary.withOpacity(0.12);
+          return _colors.primary.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.primary.withOpacity(0.08);
         }
         if (states.contains(MaterialState.focused)) {
-          return _colors.primary.withOpacity(0.12);
+          return _colors.primary.withOpacity(0.1);
         }
       }
       if (states.contains(MaterialState.pressed)) {
-        return _colors.onPrimary.withOpacity(0.12);
+        return _colors.onPrimary.withOpacity(0.1);
       }
       if (states.contains(MaterialState.hovered)) {
         return _colors.onPrimary.withOpacity(0.08);
       }
       if (states.contains(MaterialState.focused)) {
-        return _colors.onPrimary.withOpacity(0.12);
+        return _colors.onPrimary.withOpacity(0.1);
       }
       return Colors.transparent;
     });
@@ -1380,7 +1391,7 @@ class _FilledTonalIconButtonDefaultsM3 extends ButtonStyle {
         return _colors.secondaryContainer;
       }
       if (toggleable) { // toggleable but unselected case
-        return _colors.surfaceVariant;
+        return _colors.surfaceContainerHighest;
       }
       return _colors.secondaryContainer;
     });
@@ -1405,34 +1416,34 @@ class _FilledTonalIconButtonDefaultsM3 extends ButtonStyle {
     MaterialStateProperty.resolveWith((Set<MaterialState> states) {
       if (states.contains(MaterialState.selected)) {
         if (states.contains(MaterialState.pressed)) {
-          return _colors.onSecondaryContainer.withOpacity(0.12);
+          return _colors.onSecondaryContainer.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.onSecondaryContainer.withOpacity(0.08);
         }
         if (states.contains(MaterialState.focused)) {
-          return _colors.onSecondaryContainer.withOpacity(0.12);
+          return _colors.onSecondaryContainer.withOpacity(0.1);
         }
       }
       if (toggleable) { // toggleable but unselected case
         if (states.contains(MaterialState.pressed)) {
-          return _colors.onSurfaceVariant.withOpacity(0.12);
+          return _colors.onSurfaceVariant.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.onSurfaceVariant.withOpacity(0.08);
         }
         if (states.contains(MaterialState.focused)) {
-          return _colors.onSurfaceVariant.withOpacity(0.12);
+          return _colors.onSurfaceVariant.withOpacity(0.1);
         }
       }
       if (states.contains(MaterialState.pressed)) {
-        return _colors.onSecondaryContainer.withOpacity(0.12);
+        return _colors.onSecondaryContainer.withOpacity(0.1);
       }
       if (states.contains(MaterialState.hovered)) {
         return _colors.onSecondaryContainer.withOpacity(0.08);
       }
       if (states.contains(MaterialState.focused)) {
-        return _colors.onSecondaryContainer.withOpacity(0.12);
+        return _colors.onSecondaryContainer.withOpacity(0.1);
       }
       return Colors.transparent;
     });
@@ -1547,7 +1558,7 @@ class _OutlinedIconButtonDefaultsM3 extends ButtonStyle {
   MaterialStateProperty<Color?>? get overlayColor =>    MaterialStateProperty.resolveWith((Set<MaterialState> states) {
       if (states.contains(MaterialState.selected)) {
         if (states.contains(MaterialState.pressed)) {
-          return _colors.onInverseSurface.withOpacity(0.12);
+          return _colors.onInverseSurface.withOpacity(0.1);
         }
         if (states.contains(MaterialState.hovered)) {
           return _colors.onInverseSurface.withOpacity(0.08);
@@ -1557,7 +1568,7 @@ class _OutlinedIconButtonDefaultsM3 extends ButtonStyle {
         }
       }
       if (states.contains(MaterialState.pressed)) {
-        return _colors.onSurface.withOpacity(0.12);
+        return _colors.onSurface.withOpacity(0.1);
       }
       if (states.contains(MaterialState.hovered)) {
         return _colors.onSurfaceVariant.withOpacity(0.08);
