@@ -10,6 +10,17 @@ import 'package:flutter/widgets.dart';
 import 'material_state.dart';
 import 'theme.dart';
 
+// Examples can assume:
+// late BuildContext context;
+
+/// Used to configure how the [PopupMenuButton] positions its popup menu.
+enum PopupMenuPosition {
+  /// Menu is positioned over the anchor.
+  over,
+  /// Menu is positioned under the anchor.
+  under,
+}
+
 /// Defines the visual properties of the routes used to display popup menus
 /// as well as [PopupMenuItem] and [PopupMenuDivider] widgets.
 ///
@@ -36,10 +47,17 @@ class PopupMenuThemeData with Diagnosticable {
   const PopupMenuThemeData({
     this.color,
     this.shape,
+    this.menuPadding,
     this.elevation,
+    this.shadowColor,
+    this.surfaceTintColor,
     this.textStyle,
+    this.labelTextStyle,
     this.enableFeedback,
     this.mouseCursor,
+    this.position,
+    this.iconColor,
+    this.iconSize,
   });
 
   /// The background color of the popup menu.
@@ -48,12 +66,27 @@ class PopupMenuThemeData with Diagnosticable {
   /// The shape of the popup menu.
   final ShapeBorder? shape;
 
+  /// If specified, the padding of the popup menu.
+  ///
+  /// If [PopupMenuButton.menuPadding] is provided, [menuPadding] is ignored.
+  final EdgeInsetsGeometry? menuPadding;
+
   /// The elevation of the popup menu.
   final double? elevation;
 
+  /// The color used to paint shadow below the popup menu.
+  final Color? shadowColor;
+
+  /// The color used as an overlay on [color] of the popup menu.
+  final Color? surfaceTintColor;
+
   /// The text style of items in the popup menu.
   final TextStyle? textStyle;
 
+  /// You can use this to specify a different style of the label
+  /// when the popup menu item is enabled and disabled.
+  final MaterialStateProperty<TextStyle?>? labelTextStyle;
+
   /// If specified, defines the feedback property for [PopupMenuButton].
   ///
   /// If [PopupMenuButton.enableFeedback] is provided, [enableFeedback] is ignored.
@@ -64,23 +97,49 @@ class PopupMenuThemeData with Diagnosticable {
   /// If specified, overrides the default value of [PopupMenuItem.mouseCursor].
   final MaterialStateProperty<MouseCursor?>? mouseCursor;
 
+  /// Whether the popup menu is positioned over or under the popup menu button.
+  ///
+  /// When not set, the position defaults to [PopupMenuPosition.over] which makes the
+  /// popup menu appear directly over the button that was used to create it.
+  final PopupMenuPosition? position;
+
+  /// The color of the icon in the popup menu button.
+  final Color? iconColor;
+
+  /// The size of the icon in the popup menu button.
+  final double? iconSize;
+
   /// Creates a copy of this object with the given fields replaced with the
   /// new values.
   PopupMenuThemeData copyWith({
     Color? color,
     ShapeBorder? shape,
+    EdgeInsetsGeometry? menuPadding,
     double? elevation,
+    Color? shadowColor,
+    Color? surfaceTintColor,
     TextStyle? textStyle,
+    MaterialStateProperty<TextStyle?>? labelTextStyle,
     bool? enableFeedback,
     MaterialStateProperty<MouseCursor?>? mouseCursor,
+    PopupMenuPosition? position,
+    Color? iconColor,
+    double? iconSize,
   }) {
     return PopupMenuThemeData(
       color: color ?? this.color,
       shape: shape ?? this.shape,
+      menuPadding: menuPadding ?? this.menuPadding,
       elevation: elevation ?? this.elevation,
+      shadowColor: shadowColor ?? this.shadowColor,
+      surfaceTintColor: surfaceTintColor ?? this.surfaceTintColor,
       textStyle: textStyle ?? this.textStyle,
+      labelTextStyle: labelTextStyle ?? this.labelTextStyle,
       enableFeedback: enableFeedback ?? this.enableFeedback,
       mouseCursor: mouseCursor ?? this.mouseCursor,
+      position: position ?? this.position,
+      iconColor: iconColor ?? this.iconColor,
+      iconSize: iconSize ?? this.iconSize,
     );
   }
 
@@ -90,17 +149,23 @@ class PopupMenuThemeData with Diagnosticable {
   ///
   /// {@macro dart.ui.shadow.lerp}
   static PopupMenuThemeData? lerp(PopupMenuThemeData? a, PopupMenuThemeData? b, double t) {
-    assert(t != null);
-    if (a == null && b == null) {
-      return null;
+    if (identical(a, b)) {
+      return a;
     }
     return PopupMenuThemeData(
       color: Color.lerp(a?.color, b?.color, t),
       shape: ShapeBorder.lerp(a?.shape, b?.shape, t),
+      menuPadding: EdgeInsetsGeometry.lerp(a?.menuPadding, b?.menuPadding, t),
       elevation: lerpDouble(a?.elevation, b?.elevation, t),
+      shadowColor: Color.lerp(a?.shadowColor, b?.shadowColor, t),
+      surfaceTintColor: Color.lerp(a?.surfaceTintColor, b?.surfaceTintColor, t),
       textStyle: TextStyle.lerp(a?.textStyle, b?.textStyle, t),
+      labelTextStyle: MaterialStateProperty.lerp<TextStyle?>(a?.labelTextStyle, b?.labelTextStyle, t, TextStyle.lerp),
       enableFeedback: t < 0.5 ? a?.enableFeedback : b?.enableFeedback,
       mouseCursor: t < 0.5 ? a?.mouseCursor : b?.mouseCursor,
+      position: t < 0.5 ? a?.position : b?.position,
+      iconColor: Color.lerp(a?.iconColor, b?.iconColor, t),
+      iconSize: lerpDouble(a?.iconSize, b?.iconSize, t),
     );
   }
 
@@ -108,10 +173,17 @@ class PopupMenuThemeData with Diagnosticable {
   int get hashCode => Object.hash(
     color,
     shape,
+    menuPadding,
     elevation,
+    shadowColor,
+    surfaceTintColor,
     textStyle,
+    labelTextStyle,
     enableFeedback,
     mouseCursor,
+    position,
+    iconColor,
+    iconSize,
   );
 
   @override
@@ -123,12 +195,19 @@ class PopupMenuThemeData with Diagnosticable {
       return false;
     }
     return other is PopupMenuThemeData
-        && other.elevation == elevation
         && other.color == color
         && other.shape == shape
+        && other.menuPadding == menuPadding
+        && other.elevation == elevation
+        && other.shadowColor == shadowColor
+        && other.surfaceTintColor == surfaceTintColor
         && other.textStyle == textStyle
+        && other.labelTextStyle == labelTextStyle
         && other.enableFeedback == enableFeedback
-        && other.mouseCursor == mouseCursor;
+        && other.mouseCursor == mouseCursor
+        && other.position == position
+        && other.iconColor == iconColor
+        && other.iconSize == iconSize;
   }
 
   @override
@@ -136,10 +215,17 @@ class PopupMenuThemeData with Diagnosticable {
     super.debugFillProperties(properties);
     properties.add(ColorProperty('color', color, defaultValue: null));
     properties.add(DiagnosticsProperty<ShapeBorder>('shape', shape, defaultValue: null));
+    properties.add(DiagnosticsProperty<EdgeInsetsGeometry>('menuPadding', menuPadding, defaultValue: null));
     properties.add(DoubleProperty('elevation', elevation, defaultValue: null));
+    properties.add(ColorProperty('shadowColor', shadowColor, defaultValue: null));
+    properties.add(ColorProperty('surfaceTintColor', surfaceTintColor, defaultValue: null));
     properties.add(DiagnosticsProperty<TextStyle>('text style', textStyle, defaultValue: null));
+    properties.add(DiagnosticsProperty<MaterialStateProperty<TextStyle?>>('labelTextStyle', labelTextStyle, defaultValue: null));
     properties.add(DiagnosticsProperty<bool>('enableFeedback', enableFeedback, defaultValue: null));
     properties.add(DiagnosticsProperty<MaterialStateProperty<MouseCursor?>>('mouseCursor', mouseCursor, defaultValue: null));
+    properties.add(EnumProperty<PopupMenuPosition?>('position', position, defaultValue: null));
+    properties.add(ColorProperty('iconColor', iconColor, defaultValue: null));
+    properties.add(DoubleProperty('iconSize', iconSize, defaultValue: null));
   }
 }
 
@@ -151,13 +237,11 @@ class PopupMenuThemeData with Diagnosticable {
 class PopupMenuTheme extends InheritedTheme {
   /// Creates a popup menu theme that controls the configurations for
   /// popup menus in its widget subtree.
-  ///
-  /// The data argument must not be null.
   const PopupMenuTheme({
     super.key,
     required this.data,
     required super.child,
-  }) : assert(data != null);
+  });
 
   /// The properties for descendant popup menu widgets.
   final PopupMenuThemeData data;
