@@ -2,6 +2,9 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+/// @docImport 'package:flutter/material.dart';
+library;
+
 import 'dart:math' as math;
 import 'dart:ui' show Color, lerpDouble;
 
@@ -31,34 +34,14 @@ Color _colorFromHue(
   double secondary,
   double match,
 ) {
-  double red;
-  double green;
-  double blue;
-  if (hue < 60.0) {
-    red = chroma;
-    green = secondary;
-    blue = 0.0;
-  } else if (hue < 120.0) {
-    red = secondary;
-    green = chroma;
-    blue = 0.0;
-  } else if (hue < 180.0) {
-    red = 0.0;
-    green = chroma;
-    blue = secondary;
-  } else if (hue < 240.0) {
-    red = 0.0;
-    green = secondary;
-    blue = chroma;
-  } else if (hue < 300.0) {
-    red = secondary;
-    green = 0.0;
-    blue = chroma;
-  } else {
-    red = chroma;
-    green = 0.0;
-    blue = secondary;
-  }
+  final (double red, double green, double blue) = switch (hue) {
+    <  60.0 => (chroma, secondary, 0.0),
+    < 120.0 => (secondary, chroma, 0.0),
+    < 180.0 => (0.0, chroma, secondary),
+    < 240.0 => (0.0, secondary, chroma),
+    < 300.0 => (secondary, 0.0, chroma),
+    _       => (chroma, 0.0, secondary),
+  };
   return Color.fromARGB((alpha * 0xFF).round(), ((red + match) * 0xFF).round(), ((green + match) * 0xFF).round(), ((blue + match) * 0xFF).round());
 }
 
@@ -421,7 +404,7 @@ class HSLColor {
 ///
 ///  * [MaterialColor] and [MaterialAccentColor], which define Material Design
 ///    primary and accent color swatches.
-///  * [material.Colors], which defines all of the standard Material Design
+///  * [Colors], which defines all of the standard Material Design
 ///    colors.
 @immutable
 class ColorSwatch<T> extends Color {
