@@ -2,6 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+/// @docImport 'package:flutter/rendering.dart';
+/// @docImport 'package:flutter/widgets.dart';
+///
+/// @docImport 'box_decoration.dart';
+/// @docImport 'image_resolution.dart';
+library;
+
 import 'dart:developer' as developer;
 import 'dart:math' as math;
 import 'dart:ui' as ui show FlutterView, Image;
@@ -40,9 +47,6 @@ enum ImageRepeat {
 @immutable
 class DecorationImage {
   /// Creates an image to show in a [BoxDecoration].
-  ///
-  /// The [image], [alignment], [repeat], and [matchTextDirection] arguments
-  /// must not be null.
   const DecorationImage({
     required this.image,
     this.onError,
@@ -54,7 +58,7 @@ class DecorationImage {
     this.matchTextDirection = false,
     this.scale = 1.0,
     this.opacity = 1.0,
-    this.filterQuality = FilterQuality.low,
+    this.filterQuality = FilterQuality.medium,
     this.invertColors = false,
     this.isAntiAlias = false,
   });
@@ -151,8 +155,7 @@ class DecorationImage {
 
   /// Used to set the filterQuality of the image.
   ///
-  /// Defaults to [FilterQuality.low] to scale the image, which corresponds to
-  /// bilinear interpolation.
+  /// Defaults to [FilterQuality.medium].
   final FilterQuality filterQuality;
 
   /// Whether the colors of the image are inverted when drawn.
@@ -173,11 +176,11 @@ class DecorationImage {
 
   /// Creates a [DecorationImagePainter] for this [DecorationImage].
   ///
-  /// The `onChanged` argument must not be null. It will be called whenever the
-  /// image needs to be repainted, e.g. because it is loading incrementally or
-  /// because it is animated.
+  /// The `onChanged` argument will be called whenever the image needs to be
+  /// repainted, e.g. because it is loading incrementally or because it is
+  /// animated.
   DecorationImagePainter createPainter(VoidCallback onChanged) {
-    return DecorationImagePainter._(this, onChanged);
+    return _DecorationImagePainter._(this, onChanged);
   }
 
   @override
@@ -236,8 +239,8 @@ class DecorationImage {
         '$repeat',
       if (matchTextDirection)
         'match text direction',
-      'scale $scale',
-      'opacity $opacity',
+      'scale ${scale.toStringAsFixed(1)}',
+      'opacity ${opacity.toStringAsFixed(1)}',
       '$filterQuality',
       if (invertColors)
         'invert colors',
@@ -246,6 +249,28 @@ class DecorationImage {
     ];
     return '${objectRuntimeType(this, 'DecorationImage')}(${properties.join(", ")})';
   }
+
+  /// Linearly interpolates between two [DecorationImage]s.
+  ///
+  /// The `t` argument represents position on the timeline, with 0.0 meaning
+  /// that the interpolation has not started, returning `a`, 1.0 meaning that
+  /// the interpolation has finished, returning `b`, and values in between
+  /// meaning that the interpolation is at the relevant point on the timeline
+  /// between `a` and `this`. The interpolation can be extrapolated beyond 0.0
+  /// and 1.0, so negative values and values greater than 1.0 are valid (and can
+  /// easily be generated by curves such as [Curves.elasticInOut]).
+  ///
+  /// Values for `t` are usually obtained from an [Animation<double>], such as
+  /// an [AnimationController].
+  static DecorationImage? lerp(DecorationImage? a, DecorationImage? b, double t) {
+    if (identical(a, b) || t == 0.0) {
+      return a;
+    }
+    if (t == 1.0) {
+      return b;
+    }
+    return _BlendedDecorationImage(a, b, t);
+  }
 }
 
 /// The painter for a [DecorationImage].
@@ -259,15 +284,7 @@ class DecorationImage {
 ///
 /// This object should be disposed using the [dispose] method when it is no
 /// longer needed.
-class DecorationImagePainter {
-  DecorationImagePainter._(this._details, this._onChanged);
-
-  final DecorationImage _details;
-  final VoidCallback _onChanged;
-
-  ImageStream? _imageStream;
-  ImageInfo? _image;
-
+abstract interface class DecorationImagePainter {
   /// Draw the image onto the given canvas.
   ///
   /// The image is drawn at the position and size given by the `rect` argument.
@@ -282,8 +299,44 @@ class DecorationImagePainter {
   /// because it had not yet been loaded the first time this method was called,
   /// then the `onChanged` callback passed to [DecorationImage.createPainter]
   /// will be called.
-  void paint(Canvas canvas, Rect rect, Path? clipPath, ImageConfiguration configuration) {
+  ///
+  /// The `blend` argument specifies the opacity that should be applied to the
+  /// image due to this image being blended with another. The `blendMode`
+  /// argument can be specified to override the [DecorationImagePainter]'s
+  /// default [BlendMode] behavior. It is usually set to [BlendMode.srcOver] if
+  /// this is the first or only image being blended, and [BlendMode.plus] if it
+  /// is being blended with an image below.
+  void paint(Canvas canvas, Rect rect, Path? clipPath, ImageConfiguration configuration, { double blend = 1.0, BlendMode blendMode = BlendMode.srcOver });
 
+  /// Releases the resources used by this painter.
+  ///
+  /// This should be called whenever the painter is no longer needed.
+  ///
+  /// After this method has been called, the object is no longer usable.
+  void dispose();
+}
+
+class _DecorationImagePainter implements DecorationImagePainter {
+  _DecorationImagePainter._(this._details, this._onChanged) {
+    // TODO(polina-c): stop duplicating code across disposables
+    // https://github.com/flutter/flutter/issues/137435
+    if (kFlutterMemoryAllocationsEnabled) {
+      FlutterMemoryAllocations.instance.dispatchObjectCreated(
+        library: 'package:flutter/painting.dart',
+        className: '$_DecorationImagePainter',
+        object: this,
+      );
+    }
+  }
+
+  final DecorationImage _details;
+  final VoidCallback _onChanged;
+
+  ImageStream? _imageStream;
+  ImageInfo? _image;
+
+  @override
+  void paint(Canvas canvas, Rect rect, Path? clipPath, ImageConfiguration configuration, { double blend = 1.0, BlendMode blendMode = BlendMode.srcOver }) {
     bool flipHorizontally = false;
     if (_details.matchTextDirection) {
       assert(() {
@@ -338,10 +391,11 @@ class DecorationImagePainter {
       centerSlice: _details.centerSlice,
       repeat: _details.repeat,
       flipHorizontally: flipHorizontally,
-      opacity: _details.opacity,
+      opacity: _details.opacity * blend,
       filterQuality: _details.filterQuality,
       invertColors: _details.invertColors,
       isAntiAlias: _details.isAntiAlias,
+      blendMode: blendMode,
     );
 
     if (clipPath != null) {
@@ -364,13 +418,11 @@ class DecorationImagePainter {
     }
   }
 
-  /// Releases the resources used by this painter.
-  ///
-  /// This should be called whenever the painter is no longer needed.
-  ///
-  /// After this method has been called, the object is no longer usable.
-  @mustCallSuper
+  @override
   void dispose() {
+    if (kFlutterMemoryAllocationsEnabled) {
+      FlutterMemoryAllocations.instance.dispatchObjectDisposed(object: this);
+    }
     _imageStream?.removeListener(ImageStreamListener(
       _handleImage,
       onError: _details.onError,
@@ -444,7 +496,7 @@ void debugFlushLastFrameImageSizeInfo() {
 ///    corners of the destination rectangle defined by applying `fit`. The
 ///    remaining five regions are drawn by stretching them to fit such that they
 ///    exactly cover the destination rectangle while maintaining their relative
-///    positions.
+///    positions. See also [Canvas.drawImageNine].
 ///
 ///  * `repeat`: If the image does not fill `rect`, whether and how the image
 ///    should be repeated to fill `rect`. By default, the image is not repeated.
@@ -462,12 +514,7 @@ void debugFlushLastFrameImageSizeInfo() {
 ///    smart invert on iOS.
 ///
 ///  * `filterQuality`: Use this to change the quality when scaling an image.
-///     Use the [FilterQuality.low] quality setting to scale the image, which corresponds to
-///     bilinear interpolation, rather than the default [FilterQuality.none] which corresponds
-///     to nearest-neighbor.
-///
-/// The `canvas`, `rect`, `image`, `scale`, `alignment`, `repeat`, `flipHorizontally` and `filterQuality`
-/// arguments must not be null.
+///     Defaults to [FilterQuality.medium].
 ///
 /// See also:
 ///
@@ -488,8 +535,9 @@ void paintImage({
   ImageRepeat repeat = ImageRepeat.noRepeat,
   bool flipHorizontally = false,
   bool invertColors = false,
-  FilterQuality filterQuality = FilterQuality.low,
+  FilterQuality filterQuality = FilterQuality.medium,
   bool isAntiAlias = false,
+  BlendMode blendMode = BlendMode.srcOver,
 }) {
   assert(
     image.debugGetOpenHandleStackTraces()?.isNotEmpty ?? true,
@@ -530,9 +578,10 @@ void paintImage({
   if (colorFilter != null) {
     paint.colorFilter = colorFilter;
   }
-  paint.color = Color.fromRGBO(0, 0, 0, opacity);
+  paint.color = Color.fromRGBO(0, 0, 0, clampDouble(opacity, 0.0, 1.0));
   paint.filterQuality = filterQuality;
   paint.invertColors = invertColors;
+  paint.blendMode = blendMode;
   final double halfWidthDelta = (outputSize.width - destinationSize.width) / 2.0;
   final double halfHeightDelta = (outputSize.height - destinationSize.height) / 2.0;
   final double dx = halfWidthDelta + (flipHorizontally ? -alignment.x : alignment.x) * halfWidthDelta;
@@ -543,6 +592,12 @@ void paintImage({
   // Set to true if we added a saveLayer to the canvas to invert/flip the image.
   bool invertedCanvas = false;
   // Output size and destination rect are fully calculated.
+
+  // Implement debug-mode and profile-mode features:
+  //  - cacheWidth/cacheHeight warning
+  //  - debugInvertOversizedImages
+  //  - debugOnPaintImage
+  //  - Flutter.ImageSizesForFrame events in timeline
   if (!kReleaseMode) {
     // We can use the devicePixelRatio of the views directly here (instead of
     // going through a MediaQuery) because if it changes, whatever is aware of
@@ -554,7 +609,6 @@ void paintImage({
       0.0,
       (double previousValue, ui.FlutterView view) => math.max(previousValue, view.devicePixelRatio),
     );
-
     final ImageSizeInfo sizeInfo = ImageSizeInfo(
       // Some ImageProvider implementations may not have given this.
       source: debugImageLabel ?? '<Unknown Image(${image.width}×${image.height})>',
@@ -599,7 +653,7 @@ void paintImage({
       return true;
     }());
     // Avoid emitting events that are the same as those emitted in the last frame.
-    if (!kReleaseMode && !_lastFrameImageSizeInfo.contains(sizeInfo)) {
+    if (!_lastFrameImageSizeInfo.contains(sizeInfo)) {
       final ImageSizeInfo? existingSizeInfo = _pendingImageSizeInfo[sizeInfo.source];
       if (existingSizeInfo == null || existingSizeInfo.displaySizeInBytes < sizeInfo.displaySizeInBytes) {
         _pendingImageSizeInfo[sizeInfo.source!] = sizeInfo;
@@ -618,7 +672,7 @@ void paintImage({
           },
         );
         _pendingImageSizeInfo = <String, ImageSizeInfo>{};
-      });
+      }, debugLabel: 'paintImage.recordImageSizes');
     }
   }
 
@@ -691,3 +745,114 @@ Iterable<Rect> _generateImageTileRects(Rect outputRect, Rect fundamentalRect, Im
 }
 
 Rect _scaleRect(Rect rect, double scale) => Rect.fromLTRB(rect.left * scale, rect.top * scale, rect.right * scale, rect.bottom * scale);
+
+// Implements DecorationImage.lerp when the image is different.
+//
+// This class just paints both decorations on top of each other, blended together.
+//
+// The Decoration properties are faked by just forwarded to the target image.
+class _BlendedDecorationImage implements DecorationImage {
+  const _BlendedDecorationImage(this.a, this.b, this.t) : assert(a != null || b != null);
+
+  final DecorationImage? a;
+  final DecorationImage? b;
+  final double t;
+
+  @override
+  ImageProvider get image => b?.image ?? a!.image;
+  @override
+  ImageErrorListener? get onError => b?.onError ?? a!.onError;
+  @override
+  ColorFilter? get colorFilter => b?.colorFilter ?? a!.colorFilter;
+  @override
+  BoxFit? get fit => b?.fit ?? a!.fit;
+  @override
+  AlignmentGeometry get alignment => b?.alignment ?? a!.alignment;
+  @override
+  Rect? get centerSlice => b?.centerSlice ?? a!.centerSlice;
+  @override
+  ImageRepeat get repeat => b?.repeat ?? a!.repeat;
+  @override
+  bool get matchTextDirection => b?.matchTextDirection ?? a!.matchTextDirection;
+  @override
+  double get scale => b?.scale ?? a!.scale;
+  @override
+  double get opacity => b?.opacity ?? a!.opacity;
+  @override
+  FilterQuality get filterQuality => b?.filterQuality ?? a!.filterQuality;
+  @override
+  bool get invertColors => b?.invertColors ?? a!.invertColors;
+  @override
+  bool get isAntiAlias => b?.isAntiAlias ?? a!.isAntiAlias;
+
+  @override
+  DecorationImagePainter createPainter(VoidCallback onChanged) {
+    return _BlendedDecorationImagePainter._(
+      a?.createPainter(onChanged),
+      b?.createPainter(onChanged),
+      t,
+    );
+  }
+
+  @override
+  bool operator ==(Object other) {
+    if (identical(this, other)) {
+      return true;
+    }
+    if (other.runtimeType != runtimeType) {
+      return false;
+    }
+    return other is _BlendedDecorationImage
+        && other.a == a
+        && other.b == b
+        && other.t == t;
+  }
+
+  @override
+  int get hashCode => Object.hash(a, b, t);
+
+  @override
+  String toString() {
+    return '${objectRuntimeType(this, '_BlendedDecorationImage')}($a, $b, $t)';
+  }
+}
+
+class _BlendedDecorationImagePainter implements DecorationImagePainter {
+  _BlendedDecorationImagePainter._(this.a, this.b, this.t) {
+    // TODO(polina-c): stop duplicating code across disposables
+    // https://github.com/flutter/flutter/issues/137435
+    if (kFlutterMemoryAllocationsEnabled) {
+      FlutterMemoryAllocations.instance.dispatchObjectCreated(
+        library: 'package:flutter/painting.dart',
+        className: '$_BlendedDecorationImagePainter',
+        object: this,
+      );
+    }
+  }
+
+  final DecorationImagePainter? a;
+  final DecorationImagePainter? b;
+  final double t;
+
+  @override
+  void paint(Canvas canvas, Rect rect, Path? clipPath, ImageConfiguration configuration, { double blend = 1.0, BlendMode blendMode = BlendMode.srcOver }) {
+    canvas.saveLayer(null, Paint());
+    a?.paint(canvas, rect, clipPath, configuration, blend: blend * (1.0 - t), blendMode: blendMode);
+    b?.paint(canvas, rect, clipPath, configuration, blend: blend * t, blendMode: a != null ? BlendMode.plus : blendMode);
+    canvas.restore();
+  }
+
+  @override
+  void dispose() {
+    if (kFlutterMemoryAllocationsEnabled) {
+      FlutterMemoryAllocations.instance.dispatchObjectDisposed(object: this);
+    }
+    a?.dispose();
+    b?.dispose();
+  }
+
+  @override
+  String toString() {
+    return '${objectRuntimeType(this, '_BlendedDecorationImagePainter')}($a, $b, $t)';
+  }
+}
