@@ -2,7 +2,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-import 'dart:typed_data';
+/// @docImport 'package:flutter/rendering.dart';
+library;
 
 import 'package:flutter/foundation.dart';
 import 'package:vector_math/vector_math_64.dart';
@@ -10,17 +11,12 @@ import 'package:vector_math/vector_math_64.dart';
 import 'basic_types.dart';
 
 /// Utility functions for working with matrices.
-class MatrixUtils {
-  // This class is not meant to be instantiated or extended; this constructor
-  // prevents instantiation and extension.
-  MatrixUtils._();
-
+abstract final class MatrixUtils {
   /// Returns the given [transform] matrix as an [Offset], if the matrix is
   /// nothing but a 2D translation.
   ///
   /// Otherwise, returns null.
   static Offset? getAsTranslation(Matrix4 transform) {
-    assert(transform != null);
     final Float64List values = transform.storage;
     // Values are stored in column-major order.
     if (values[0] == 1.0 && // col 1
@@ -47,7 +43,6 @@ class MatrixUtils {
   ///
   /// Otherwise, returns null.
   static double? getAsScale(Matrix4 transform) {
-    assert(transform != null);
     final Float64List values = transform.storage;
     // Values are stored in column-major order.
     if (values[1] == 0.0 && // col 1 (value 0 is the scale)
@@ -73,14 +68,16 @@ class MatrixUtils {
   /// Returns true if the given matrices are exactly equal, and false
   /// otherwise. Null values are assumed to be the identity matrix.
   static bool matrixEquals(Matrix4? a, Matrix4? b) {
-    if (identical(a, b))
+    if (identical(a, b)) {
       return true;
+    }
     assert(a != null || b != null);
-    if (a == null)
+    if (a == null) {
       return isIdentity(b!);
-    if (b == null)
+    }
+    if (b == null) {
       return isIdentity(a);
-    assert(a != null && b != null);
+    }
     return a.storage[0] == b.storage[0]
         && a.storage[1] == b.storage[1]
         && a.storage[2] == b.storage[2]
@@ -101,7 +98,6 @@ class MatrixUtils {
 
   /// Whether the given matrix is the identity matrix.
   static bool isIdentity(Matrix4 a) {
-    assert(a != null);
     return a.storage[0] == 1.0 // col 1
         && a.storage[1] == 0.0
         && a.storage[2] == 0.0
@@ -234,17 +230,17 @@ class MatrixUtils {
     // First, consider that a full point transform using the vector math
     // package involves expanding it out into a vector3 with a Z coordinate
     // of 0.0 and then performing 3 multiplies and 3 adds per coordinate:
-    // ```
-    // xt = x*m00 + y*m10 + z*m20 + m30;
-    // yt = x*m01 + y*m11 + z*m21 + m31;
-    // zt = x*m02 + y*m12 + z*m22 + m32;
-    // wt = x*m03 + y*m13 + z*m23 + m33;
-    // ```
+    //
+    //     xt = x*m00 + y*m10 + z*m20 + m30;
+    //     yt = x*m01 + y*m11 + z*m21 + m31;
+    //     zt = x*m02 + y*m12 + z*m22 + m32;
+    //     wt = x*m03 + y*m13 + z*m23 + m33;
+    //
     // Immediately we see that we can get rid of the 3rd column of multiplies
     // since we know that Z=0.0. We can also get rid of the 3rd row because
     // we ignore the resulting Z coordinate. Finally we can get rid of the
     // last row if we don't have a perspective transform since we can verify
-    // that the results are 1.0 for all points.  This gets us down to 16
+    // that the results are 1.0 for all points. This gets us down to 16
     // multiplies and 16 adds in the non-perspective case and 24 of each for
     // the perspective case. (Plus the 12 comparisons to turn them back into
     // a bounding box.)
@@ -281,7 +277,7 @@ class MatrixUtils {
     // continue to hold with respect to the non-normalized coordinates so
     // we can still save a lot of multiplications by computing the 4
     // non-normalized coordinates using relative additions before we normalize
-    // them and they lose their "pseudo-parallelogram" relationships.  We still
+    // them and they lose their "pseudo-parallelogram" relationships. We still
     // have to do the normalization divisions and min/max all 4 points to
     // get the resulting transformed bounding box, but we save a lot of
     // calculations over blindly transforming all 4 coordinates independently.
@@ -306,26 +302,26 @@ class MatrixUtils {
     // you combine both of them, the resulting "opposite corner" will
     // actually be between the limits they established by pushing the walls
     // away from each other, as below:
-    // ```
-    //         +---------(originx,originy)--------------+
-    //         |            -----^----                  |
-    //         |       -----          ----              |
-    //         |  -----                   ----          |
-    // (+hx,+hy)<                             ----      |
-    //         |  ----                            ----  |
-    //         |      ----                             >(+wx,+wy)
-    //         |          ----                   -----  |
-    //         |              ----          -----       |
-    //         |                  ---- -----            |
-    //         |                      v                 |
-    //         +---------------(+wx+hx,+wy+hy)----------+
-    // ```
+    //
+    //             +---------(originx,originy)--------------+
+    //             |            -----^----                  |
+    //             |       -----          ----              |
+    //             |  -----                   ----          |
+    //     (+hx,+hy)<                             ----      |
+    //             |  ----                            ----  |
+    //             |      ----                             >(+wx,+wy)
+    //             |          ----                   -----  |
+    //             |              ----          -----       |
+    //             |                  ---- -----            |
+    //             |                      v                 |
+    //             +---------------(+wx+hx,+wy+hy)----------+
+    //
     // In this diagram, consider that:
-    // ```
-    // wx would be a positive number
-    // hx would be a negative number
-    // wy and hy would both be positive numbers
-    // ```
+    //
+    //  * wx would be a positive number
+    //  * hx would be a negative number
+    //  * wy and hy would both be positive numbers
+    //
     // As a result, wx pushes out the right wall, hx pushes out the left wall,
     // and both wy and hy push down the bottom wall of the bounding box. The
     // wx,hx pair (of opposite signs) worked on opposite walls and the final
@@ -343,8 +339,8 @@ class MatrixUtils {
     // for a total of 8 multiplies, 8 adds, and 4 comparisons.
     //
     // An astute observer will note that we do need to do 2 subtractions at
-    // the top of the method to compute the width and height.  Add those to
-    // all of the relative solutions listed above.  The test for perspective
+    // the top of the method to compute the width and height. Add those to
+    // all of the relative solutions listed above. The test for perspective
     // also adds 3 compares to the affine case and up to 3 compares to the
     // perspective case (depending on which test fails, the rest are omitted).
     //
@@ -436,13 +432,13 @@ class MatrixUtils {
   /// The transformed rect is then projected back into the plane with z equals
   /// 0.0 before computing its bounding rect.
   static Rect inverseTransformRect(Matrix4 transform, Rect rect) {
-    assert(rect != null);
     // As exposed by `unrelated_type_equality_checks`, this assert was a no-op.
     // Fixing it introduces a bunch of runtime failures; for more context see:
     // https://github.com/flutter/flutter/pull/31568
     // assert(transform.determinant != 0.0);
-    if (isIdentity(transform))
+    if (isIdentity(transform)) {
       return rect;
+    }
     transform = Matrix4.copy(transform)..invert();
     return transformRect(transform, rect);
   }
@@ -453,8 +449,8 @@ class MatrixUtils {
   ///
   /// The `radius` simulates the radius of the cylinder the plane is being
   /// wrapped onto. If the transformation is applied to a 0-dimensional dot
-  /// instead of a plane, the dot would simply translate by +/- `radius` pixels
-  /// along the `orientation` [Axis] when rotating from 0 to +/- 90 degrees.
+  /// instead of a plane, the dot would translate by ± `radius` pixels
+  /// along the `orientation` [Axis] when rotating from 0 to ±90 degrees.
   ///
   /// A positive radius means the object is closest at 0 `angle` and a negative
   /// radius means the object is closest at π `angle` or 180 degrees.
@@ -476,7 +472,7 @@ class MatrixUtils {
   /// The `orientation` is the direction of the rotation axis.
   ///
   /// Because the viewing position is a point, it's never possible to see the
-  /// outer side of the cylinder at or past +/- π / 2 or 90 degrees and it's
+  /// outer side of the cylinder at or past ±π/2 or 90 degrees and it's
   /// almost always possible to end up seeing the inner side of the cylinder
   /// or the back side of the transformed plane before π / 2 when perspective > 0.
   static Matrix4 createCylindricalProjectionTransform({
@@ -485,10 +481,7 @@ class MatrixUtils {
     double perspective = 0.001,
     Axis orientation = Axis.vertical,
   }) {
-    assert(radius != null);
-    assert(angle != null);
     assert(perspective >= 0 && perspective <= 1.0);
-    assert(orientation != null);
 
     // Pre-multiplied matrix of a projection matrix and a view matrix.
     //
@@ -514,11 +507,10 @@ class MatrixUtils {
 
     // Model matrix by first translating the object from the origin of the world
     // by radius in the z axis and then rotating against the world.
-    result = result * ((
-        orientation == Axis.horizontal
-            ? Matrix4.rotationY(angle)
-            : Matrix4.rotationX(angle)
-    ) * Matrix4.translationValues(0.0, 0.0, radius)) as Matrix4;
+    result = result * (switch (orientation) {
+        Axis.horizontal => Matrix4.rotationY(angle),
+        Axis.vertical   => Matrix4.rotationX(angle),
+      } * Matrix4.translationValues(0.0, 0.0, radius)) as Matrix4;
 
     // Essentially perspective * view * model.
     return result;
@@ -537,8 +529,9 @@ class MatrixUtils {
 ///
 /// If the argument is null, returns a list with the single string "null".
 List<String> debugDescribeTransform(Matrix4? transform) {
-  if (transform == null)
+  if (transform == null) {
     return const <String>['null'];
+  }
   return <String>[
     '[0] ${debugFormatDouble(transform.entry(0, 0))},${debugFormatDouble(transform.entry(0, 1))},${debugFormatDouble(transform.entry(0, 2))},${debugFormatDouble(transform.entry(0, 3))}',
     '[1] ${debugFormatDouble(transform.entry(1, 0))},${debugFormatDouble(transform.entry(1, 1))},${debugFormatDouble(transform.entry(1, 2))},${debugFormatDouble(transform.entry(1, 3))}',
@@ -550,23 +543,13 @@ List<String> debugDescribeTransform(Matrix4? transform) {
 /// Property which handles [Matrix4] that represent transforms.
 class TransformProperty extends DiagnosticsProperty<Matrix4> {
   /// Create a diagnostics property for [Matrix4] objects.
-  ///
-  /// The [showName] and [level] arguments must not be null.
   TransformProperty(
-    String name,
-    Matrix4? value, {
-    bool showName = true,
-    Object? defaultValue = kNoDefaultValue,
-    DiagnosticLevel level = DiagnosticLevel.info,
-  }) : assert(showName != null),
-       assert(level != null),
-       super(
-         name,
-         value,
-         showName: showName,
-         defaultValue: defaultValue,
-         level: level,
-       );
+    String super.name,
+    super.value, {
+    super.showName,
+    super.defaultValue,
+    super.level,
+  });
 
   @override
   String valueToString({ TextTreeConfiguration? parentConfiguration }) {
