@@ -36,16 +36,12 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   /// doesn't know what direction it will be laid out in.
   double get beforePadding {
     assert(resolvedPadding != null);
-    switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
-      case AxisDirection.up:
-        return resolvedPadding!.bottom;
-      case AxisDirection.right:
-        return resolvedPadding!.left;
-      case AxisDirection.down:
-        return resolvedPadding!.top;
-      case AxisDirection.left:
-        return resolvedPadding!.right;
-    }
+    return switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
+      AxisDirection.up    => resolvedPadding!.bottom,
+      AxisDirection.right => resolvedPadding!.left,
+      AxisDirection.down  => resolvedPadding!.top,
+      AxisDirection.left  => resolvedPadding!.right,
+    };
   }
 
   /// The padding in the scroll direction on the side furthest from the 0.0 scroll offset.
@@ -54,16 +50,12 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   /// doesn't know what direction it will be laid out in.
   double get afterPadding {
     assert(resolvedPadding != null);
-    switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
-      case AxisDirection.up:
-        return resolvedPadding!.top;
-      case AxisDirection.right:
-        return resolvedPadding!.right;
-      case AxisDirection.down:
-        return resolvedPadding!.bottom;
-      case AxisDirection.left:
-        return resolvedPadding!.left;
-    }
+    return switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
+      AxisDirection.up    => resolvedPadding!.top,
+      AxisDirection.right => resolvedPadding!.right,
+      AxisDirection.down  => resolvedPadding!.bottom,
+      AxisDirection.left  => resolvedPadding!.left,
+    };
   }
 
   /// The total padding in the [SliverConstraints.axisDirection]. (In other
@@ -85,12 +77,10 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   /// doesn't know what direction it will be laid out in.
   double get crossAxisPadding {
     assert(resolvedPadding != null);
-    switch (constraints.axis) {
-      case Axis.horizontal:
-        return resolvedPadding!.vertical;
-      case Axis.vertical:
-        return resolvedPadding!.horizontal;
-    }
+    return switch (constraints.axis) {
+      Axis.horizontal => resolvedPadding!.vertical,
+      Axis.vertical => resolvedPadding!.horizontal,
+    };
   }
 
   @override
@@ -103,22 +93,18 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   @override
   void performLayout() {
     final SliverConstraints constraints = this.constraints;
-    assert(resolvedPadding != null);
+    double paintOffset({required double from, required double to}) => calculatePaintOffset(constraints, from: from, to: to);
+    double cacheOffset({required double from, required double to}) => calculateCacheOffset(constraints, from: from, to: to);
+
+    assert(this.resolvedPadding != null);
+    final EdgeInsets resolvedPadding = this.resolvedPadding!;
     final double beforePadding = this.beforePadding;
     final double afterPadding = this.afterPadding;
     final double mainAxisPadding = this.mainAxisPadding;
     final double crossAxisPadding = this.crossAxisPadding;
     if (child == null) {
-      final double paintExtent = calculatePaintOffset(
-          constraints,
-          from: 0.0,
-          to: mainAxisPadding,
-      );
-      final double cacheExtent = calculateCacheOffset(
-          constraints,
-          from: 0.0,
-          to: mainAxisPadding,
-      );
+      final double paintExtent = paintOffset(from: 0.0, to: mainAxisPadding);
+      final double cacheExtent = cacheOffset(from: 0.0, to: mainAxisPadding);
       geometry = SliverGeometry(
         scrollExtent: mainAxisPadding,
         paintExtent: math.min(paintExtent, constraints.remainingPaintExtent),
@@ -127,11 +113,7 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
       );
       return;
     }
-    final double beforePaddingPaintExtent = calculatePaintOffset(
-      constraints,
-      from: 0.0,
-      to: beforePadding,
-    );
+    final double beforePaddingPaintExtent = paintOffset(from: 0.0, to: beforePadding);
     double overlap = constraints.overlap;
     if (overlap > 0) {
       overlap = math.max(0.0, constraints.overlap - beforePaddingPaintExtent);
@@ -141,8 +123,8 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
         scrollOffset: math.max(0.0, constraints.scrollOffset - beforePadding),
         cacheOrigin: math.min(0.0, constraints.cacheOrigin + beforePadding),
         overlap: overlap,
-        remainingPaintExtent: constraints.remainingPaintExtent - calculatePaintOffset(constraints, from: 0.0, to: beforePadding),
-        remainingCacheExtent: constraints.remainingCacheExtent - calculateCacheOffset(constraints, from: 0.0, to: beforePadding),
+        remainingPaintExtent: constraints.remainingPaintExtent - paintOffset(from: 0.0, to: beforePadding),
+        remainingCacheExtent: constraints.remainingCacheExtent - cacheOffset(from: 0.0, to: beforePadding),
         crossAxisExtent: math.max(0.0, constraints.crossAxisExtent - crossAxisPadding),
         precedingScrollExtent: beforePadding + constraints.precedingScrollExtent,
       ),
@@ -155,30 +137,19 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
       );
       return;
     }
-    final double afterPaddingPaintExtent = calculatePaintOffset(
-      constraints,
-      from: beforePadding + childLayoutGeometry.scrollExtent,
-      to: mainAxisPadding + childLayoutGeometry.scrollExtent,
-    );
+    final double scrollExtent = childLayoutGeometry.scrollExtent;
+    final double beforePaddingCacheExtent = cacheOffset(from: 0.0, to: beforePadding);
+    final double afterPaddingCacheExtent = cacheOffset(from: beforePadding + scrollExtent, to: mainAxisPadding + scrollExtent);
+    final double afterPaddingPaintExtent = paintOffset(from: beforePadding + scrollExtent, to: mainAxisPadding + scrollExtent);
+    final double mainAxisPaddingCacheExtent = beforePaddingCacheExtent + afterPaddingCacheExtent;
     final double mainAxisPaddingPaintExtent = beforePaddingPaintExtent + afterPaddingPaintExtent;
-    final double beforePaddingCacheExtent = calculateCacheOffset(
-      constraints,
-      from: 0.0,
-      to: beforePadding,
-    );
-    final double afterPaddingCacheExtent = calculateCacheOffset(
-      constraints,
-      from: beforePadding + childLayoutGeometry.scrollExtent,
-      to: mainAxisPadding + childLayoutGeometry.scrollExtent,
-    );
-    final double mainAxisPaddingCacheExtent = afterPaddingCacheExtent + beforePaddingCacheExtent;
     final double paintExtent = math.min(
       beforePaddingPaintExtent + math.max(childLayoutGeometry.paintExtent, childLayoutGeometry.layoutExtent + afterPaddingPaintExtent),
       constraints.remainingPaintExtent,
     );
     geometry = SliverGeometry(
       paintOrigin: childLayoutGeometry.paintOrigin,
-      scrollExtent: mainAxisPadding + childLayoutGeometry.scrollExtent,
+      scrollExtent: mainAxisPadding + scrollExtent,
       paintExtent: paintExtent,
       layoutExtent: math.min(mainAxisPaddingPaintExtent + childLayoutGeometry.layoutExtent, paintExtent),
       cacheExtent: math.min(mainAxisPaddingCacheExtent + childLayoutGeometry.cacheExtent, constraints.remainingCacheExtent),
@@ -189,18 +160,17 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
       ),
       hasVisualOverflow: childLayoutGeometry.hasVisualOverflow,
     );
-
+    final double calculatedOffset = switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
+      AxisDirection.up    => paintOffset(from: resolvedPadding.bottom + scrollExtent, to: resolvedPadding.vertical + scrollExtent),
+      AxisDirection.left  => paintOffset(from: resolvedPadding.right + scrollExtent, to: resolvedPadding.horizontal + scrollExtent),
+      AxisDirection.right => paintOffset(from: 0.0, to: resolvedPadding.left),
+      AxisDirection.down  => paintOffset(from: 0.0, to: resolvedPadding.top),
+    };
     final SliverPhysicalParentData childParentData = child!.parentData! as SliverPhysicalParentData;
-    switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
-      case AxisDirection.up:
-        childParentData.paintOffset = Offset(resolvedPadding!.left, calculatePaintOffset(constraints, from: resolvedPadding!.bottom + childLayoutGeometry.scrollExtent, to: resolvedPadding!.bottom + childLayoutGeometry.scrollExtent + resolvedPadding!.top));
-      case AxisDirection.right:
-        childParentData.paintOffset = Offset(calculatePaintOffset(constraints, from: 0.0, to: resolvedPadding!.left), resolvedPadding!.top);
-      case AxisDirection.down:
-        childParentData.paintOffset = Offset(resolvedPadding!.left, calculatePaintOffset(constraints, from: 0.0, to: resolvedPadding!.top));
-      case AxisDirection.left:
-        childParentData.paintOffset = Offset(calculatePaintOffset(constraints, from: resolvedPadding!.right + childLayoutGeometry.scrollExtent, to: resolvedPadding!.right + childLayoutGeometry.scrollExtent + resolvedPadding!.left), resolvedPadding!.top);
-    }
+    childParentData.paintOffset = switch (constraints.axis) {
+      Axis.horizontal => Offset(calculatedOffset, resolvedPadding.top),
+      Axis.vertical   => Offset(resolvedPadding.left, calculatedOffset),
+    };
     assert(beforePadding == this.beforePadding);
     assert(afterPadding == this.afterPadding);
     assert(mainAxisPadding == this.mainAxisPadding);
@@ -211,7 +181,7 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   bool hitTestChildren(SliverHitTestResult result, { required double mainAxisPosition, required double crossAxisPosition }) {
     if (child != null && child!.geometry!.hitTestExtent > 0.0) {
       final SliverPhysicalParentData childParentData = child!.parentData! as SliverPhysicalParentData;
-      result.addWithAxisOffset(
+      return result.addWithAxisOffset(
         mainAxisPosition: mainAxisPosition,
         crossAxisPosition: crossAxisPosition,
         mainAxisOffset: childMainAxisPosition(child!),
@@ -233,14 +203,10 @@ abstract class RenderSliverEdgeInsetsPadding extends RenderSliver with RenderObj
   double childCrossAxisPosition(RenderSliver child) {
     assert(child == this.child);
     assert(resolvedPadding != null);
-    switch (applyGrowthDirectionToAxisDirection(constraints.axisDirection, constraints.growthDirection)) {
-      case AxisDirection.up:
-      case AxisDirection.down:
-        return resolvedPadding!.left;
-      case AxisDirection.left:
-      case AxisDirection.right:
-        return resolvedPadding!.top;
-    }
+    return switch (constraints.axis) {
+      Axis.horizontal => resolvedPadding!.top,
+      Axis.vertical   => resolvedPadding!.left,
+    };
   }
 
   @override
