@@ -2,6 +2,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+/// @docImport 'package:flutter/material.dart';
+library;
 
 import 'dart:math' as math;
 
@@ -14,6 +16,14 @@ export 'package:flutter/foundation.dart' show TargetPlatform;
 
 export 'text_input.dart' show TextEditingValue;
 
+// Examples can assume:
+// late RegExp _pattern;
+
+/// Mechanisms for enforcing maximum length limits.
+///
+/// This is used by [TextField] to specify how the [TextField.maxLength] should
+/// be applied.
+///
 /// {@template flutter.services.textFormatter.maxLengthEnforcement}
 /// ### [MaxLengthEnforcement.enforced] versus
 /// [MaxLengthEnforcement.truncateAfterCompositionEnds]
@@ -40,8 +50,8 @@ export 'text_input.dart' show TextEditingValue;
 /// See also:
 ///
 ///  * [TextField.maxLengthEnforcement] which is used in conjunction with
-///  [TextField.maxLength] to limit the length of user input. [TextField] also
-///  provides a character counter to provide visual feedback.
+///    [TextField.maxLength] to limit the length of user input. [TextField] also
+///    provides a character counter to provide visual feedback.
 enum MaxLengthEnforcement {
   /// No enforcement applied to the editing value. It's possible to exceed the
   /// max length.
@@ -80,6 +90,13 @@ enum MaxLengthEnforcement {
 ///  * [FilteringTextInputFormatter], a provided formatter for filtering
 ///    characters.
 abstract class TextInputFormatter {
+  /// This constructor enables subclasses to provide const constructors so that they can be used in const expressions.
+  const TextInputFormatter();
+
+  /// A shorthand to creating a custom [TextInputFormatter] which formats
+  /// incoming text input changes with the given function.
+  const factory TextInputFormatter.withFunction(TextInputFormatFunction formatFunction) = _SimpleTextInputFormatter;
+
   /// Called when text is being typed or cut/copy/pasted in the [EditableText].
   ///
   /// You can override the resulting text based on the previous text value and
@@ -91,14 +108,6 @@ abstract class TextInputFormatter {
     TextEditingValue oldValue,
     TextEditingValue newValue,
   );
-
-  /// A shorthand to creating a custom [TextInputFormatter] which formats
-  /// incoming text input changes with the given function.
-  static TextInputFormatter withFunction(
-    TextInputFormatFunction formatFunction,
-  ) {
-    return _SimpleTextInputFormatter(formatFunction);
-  }
 }
 
 /// Function signature expected for creating custom [TextInputFormatter]
@@ -110,8 +119,7 @@ typedef TextInputFormatFunction = TextEditingValue Function(
 
 /// Wiring for [TextInputFormatter.withFunction].
 class _SimpleTextInputFormatter extends TextInputFormatter {
-  _SimpleTextInputFormatter(this.formatFunction)
-    : assert(formatFunction != null);
+  const _SimpleTextInputFormatter(this.formatFunction);
 
   final TextInputFormatFunction formatFunction;
 
@@ -212,52 +220,67 @@ class _TextEditingValueAccumulator {
   }
 }
 
-/// A [TextInputFormatter] that prevents the insertion of characters
-/// matching (or not matching) a particular pattern.
+/// A [TextInputFormatter] that prevents the insertion of characters matching
+/// (or not matching) a particular pattern, by replacing the characters with the
+/// given [replacementString].
 ///
 /// Instances of filtered characters found in the new [TextEditingValue]s
-/// will be replaced with the [replacementString] which defaults to the empty
-/// string.
+/// will be replaced by the [replacementString] which defaults to the empty
+/// string, and the current [TextEditingValue.selection] and
+/// [TextEditingValue.composing] region will be adjusted to account for the
+/// replacement.
 ///
-/// Since this formatter only removes characters from the text, it attempts to
-/// preserve the existing [TextEditingValue.selection] to values it would now
-/// fall at with the removed characters.
+/// This formatter is typically used to match potentially recurring [Pattern]s
+/// in the new [TextEditingValue]. It never completely rejects the new
+/// [TextEditingValue] and falls back to the current [TextEditingValue] when the
+/// given [filterPattern] fails to match. Consider using a different
+/// [TextInputFormatter] such as:
+///
+/// ```dart
+/// // _pattern is a RegExp or other Pattern object
+/// TextInputFormatter.withFunction(
+///   (TextEditingValue oldValue, TextEditingValue newValue) {
+///     return _pattern.hasMatch(newValue.text) ? newValue : oldValue;
+///   },
+/// ),
+/// ```
+///
+/// for accepting/rejecting new input based on a predicate on the full string.
+/// As an example, [FilteringTextInputFormatter] typically shouldn't be used
+/// with [RegExp]s that contain positional matchers (`^` or `$`) since these
+/// patterns are usually meant for matching the whole string.
+///
+/// ### Quote characters on iOS
+///
+/// When filtering single (`'`) or double (`"`) quote characters, be aware that
+/// the default iOS keyboard actually inserts special directional versions of
+/// these characters (`‘` and `’` for single quote, and `“` and `”` for double
+/// quote). Consider including all three variants in your regular expressions to
+/// support iOS.
 class FilteringTextInputFormatter extends TextInputFormatter {
-  /// Creates a formatter that prevents the insertion of characters
-  /// based on a filter pattern.
+  /// Creates a formatter that replaces banned patterns with the given
+  /// [replacementString].
   ///
   /// If [allow] is true, then the filter pattern is an allow list,
   /// and characters must match the pattern to be accepted. See also
-  /// the `FilteringTextInputFormatter.allow` constructor.
-  // TODO(goderbauer): Cannot link to the constructor because of https://github.com/dart-lang/dartdoc/issues/2276.
+  /// the [FilteringTextInputFormatter.allow()] constructor.
   ///
   /// If [allow] is false, then the filter pattern is a deny list,
   /// and characters that match the pattern are rejected. See also
   /// the [FilteringTextInputFormatter.deny] constructor.
-  ///
-  /// The [filterPattern], [allow], and [replacementString] arguments
-  /// must not be null.
   FilteringTextInputFormatter(
     this.filterPattern, {
     required this.allow,
     this.replacementString = '',
-  }) : assert(filterPattern != null),
-       assert(allow != null),
-       assert(replacementString != null);
+  });
 
   /// Creates a formatter that only allows characters matching a pattern.
-  ///
-  /// The [filterPattern] and [replacementString] arguments
-  /// must not be null.
   FilteringTextInputFormatter.allow(
     Pattern filterPattern, {
     String replacementString = '',
   }) : this(filterPattern, allow: true, replacementString: replacementString);
 
   /// Creates a formatter that blocks characters matching a pattern.
-  ///
-  /// The [filterPattern] and [replacementString] arguments
-  /// must not be null.
   FilteringTextInputFormatter.deny(
     Pattern filterPattern, {
     String replacementString = '',
@@ -394,7 +417,7 @@ class FilteringTextInputFormatter extends TextInputFormatter {
       // The length added by adding the replacementString.
       final int replacedLength = originalIndex <= regionStart && originalIndex < regionEnd ? 0 : replacementString.length;
       // The length removed by removing the replacementRange.
-      final int removedLength = originalIndex.clamp(regionStart, regionEnd) - regionStart; // ignore_clamp_double_lint
+      final int removedLength = originalIndex.clamp(regionStart, regionEnd) - regionStart;
       return replacedLength - removedLength;
     }
 
