@@ -81,7 +81,7 @@ void check(int hr) {
 
 void main() {
   // Initialize COM
-  check(CoInitializeEx(nullptr, COINIT_APARTMENTTHREADED));
+  check(CoInitializeEx(nullptr, COINIT.COINIT_APARTMENTTHREADED));
 
   // Retrieve the list of available audio output devices.
   final pDeviceEnumerator = MMDeviceEnumerator.createInstance();
@@ -130,12 +130,12 @@ void main() {
     // Check the retrieved device friendly name.
     final varName = pVal.ref;
     if (varName.vt != VARENUM.VT_EMPTY) {
-      final ptr = Pointer<Utf16>.fromAddress(varName.val1);
-      final name = ptr.toDartString();
+      final bstrVal = varName.bstrVal;
+      final name = bstrVal.toDartString();
       print(" ID: $id Name: $name");
-      free(ptr);
+      SysFreeString(bstrVal);
     } else {
-      print(" Unknown device");
+      print(' Unknown device');
     }
 
     free(ppEndpoint);
@@ -153,7 +153,7 @@ void main() {
   final iidAudioClient = convertToIID(IID_IAudioClient3);
   final ppAudioClient = calloc<COMObject>();
   check(pDevice.activate(
-      iidAudioClient, CLSCTX_ALL, nullptr, ppAudioClient.cast()));
+      iidAudioClient, CLSCTX.CLSCTX_ALL, nullptr, ppAudioClient.cast()));
   free(iidAudioClient);
   final pAudioClient = IAudioClient3(ppAudioClient);
 
